!+
! Module: adapt_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: adapt_mod.f90,v 1.2 2009/02/20 13:27:08 aibarra Exp $
!-
!
!  SAS Version 13.0
!  Version 5.0  9 November 2012
!
!  Makes adaptively smoothed images
!
!  adapt created adaptively smothed exposure corrected and 
!  background subtracted images for the XMM-ESAS package.
!
!  INPUT:
!    Number of counts for smoothing
!    Threshold for masking
!    Detector selection
!          0: Use the output of comb
!          1: Use a specific exposure
!    Prefix if the detector selection is either 1 or 2
!    Binning control
!          1: No binning
!          2: Bin by 2
!          4: Bin by 4
!    Lower energy limit for the energy band in eV
!    Upper energy limit for the energy band in eV
!    Particle background control
!          0: do not include
!          1: include
!    Soft proton control
!          0: do not include
!          1: include
!    Mask file name (if absent defaults to using the exposure 
!        map for masking)
!
!  OUPUT:
!    Adaptively smoothed imag and radial profile QDP plot

module adapt_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public adapt

contains

  !---------------------------------------------------------------------
  subroutine adapt
  !---------------------------------------------------------------------
        REAL :: B, BACK(900,900), BCK(50), BLAST, BZERO, BSCALE, &
            C, CLAST, COUNTS(900,900), COUNTSO(900,900), &
            CTS(50), E, ELAST, EX, EXPO(50), EXPOSE(900,900), &
            NUM, NUMLAST, OOTSCA, &
            PROT(900,900), R, RAD, REX, RNB, RNC, &
            RNP, RNS, SCA, SN, SWCX(900,900), TEMP, &
            MASK(900,900), T, TLIM, TRAD, TEMSC, &
            X(50), XCEN, XH(10000), XMAX, &
            Y(50), Y2(50), YCEN, YH(1000), YMAX, YS(50)
        real, dimension(:,:), allocatable :: OUTPUT, SIZE
        DOUBLE PRECISION CDELT(2), CRPIX(2), CRVAL(2)
        integer :: BLOCKN, DETECT, I, I2, IBIN, IDET, &
            IEH, IEL, IFLAG, II, INI4(900,900), IOFF, IOS, IR, &
            J, J1, J2, JJ, JJ1, JJ2, K, LUN, N(50), xdim, ydim
        integer :: STATUS = 0
        logical :: CLOBBER, IPART, IPROT, ISWCX
        CHARACTER (len=1) ::  PRE(4), PREF(5) 
        CHARACTER (len=4) ::  PREFI
        CHARACTER (len=5) ::  PREFIX, PREFIXIN
        CHARACTER (len=8) ::  CTYPE(2), UNITS
        CHARACTER (len=25) :: MODE
        CHARACTER (len=60) :: COMMENT
        CHARACTER (len=80) :: COM
        CHARACTER (len=132) :: INFILE, INFILES, INFILEO, MASKFILE

        EQUIVALENCE (PREFIX,PREF)
        EQUIVALENCE (PREFI,PRE)

        SN = realParameter("smoothingcounts")
        TEMSC = realParameter("thresholdmasking")
        IDET = intParameter("detector")
        IBIN = intParameter("binning")
        IEL = intParameter("elow")
        IEH = intParameter("ehigh")
        IPART = booleanParameter("withpartcontrol")
        IPROT = booleanParameter("withsoftcontrol")
        ISWCX = booleanParameter("withswcxcontrol")
        CLOBBER = booleanParameter("clobber")
        PREFIXIN = stringParameter("prefix")
        MASKFILE = stringParameter("maskfile")

        PREFIX = PREFIXIN

        DO I=1,50
            X(I) = 0.0
            Y(I) = 0.0
            Y2(I) = 0.0
            YS(I) = 0.0
        ENDDO

        DO I=1,1000
            XH(I) = -0.1 + I/15.0
            YH(I) = 0.0
        ENDDO

!  Enter number of counts for smoothing.  The program smoothes using 
!  a circular region surrounding the pixel whose radius is increased
!  until the selected number of counts are enclosed.

        write(errstr,*) "Number of counts for smoothing: ", SN
        call message(AppMsg,Silent)

!  Enter the fractional limit for template, the program will exclude
!  all pixels in the field where the mask value is less than the 
!  fractional limit times the average non-zero value of the mask

        write(errstr,*) "Fractional limit for template: ", TEMSC
        call message(AppMsg,Silent)

!  Enter the detector control

        write(errstr,*) "Detector control: ", IDET
        call message(AppMsg,Silent)
        IF(IDET .GT. 0) THEN
            write(errstr,*) "Prefix: ", PREFIX
            call message(AppMsg,Silent)
            IF((PREF(1) .EQ. '1') .OR. (PREF(1) .EQ. '2')) THEN
                DETECT = 1
            ELSE
                DETECT = 2
                DO I=1,4
                    PRE(I) = PREF(I)
                ENDDO
            ENDIF
        ELSE
            DETECT = 0
        ENDIF

!  Print out program controld

        write(errstr,*) "Detector selection: ", DETECT
        call message(AppMsg,Silent)
        write(errstr,*) "Binning factor: ", IBIN
        call message(AppMsg,Silent)
        write(errstr,*) "Band energies in eV: ", IEL, IEH
        call message(AppMsg,Silent)
        write(errstr,*) "Particle background Control: ", IPART
        call message(AppMsg,Silent)
        write(errstr,*) "Soft proton background Control: ", IPROT
        call message(AppMsg,Silent)
        write(errstr,*) "SWCX background Control: ", ISWCX
        call message(AppMsg,Silent)
        write(errstr,*) "Mask File: ", trim(MASKFILE)
        call message(AppMsg,Silent)

        ioff = mod(900,ibin)/2
        xdim = 900/ibin
        ydim = 900/ibin
        allocate(output(xdim,ydim))
        allocate(size(xdim,ydim))

!  Enter the count array file name

 1000   FORMAT(A110)

        IF(IDET .GT. 0) THEN
            IF(DETECT .EQ. 1) THEN
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3001) PREFIX, IEL, IEH
 3001                   FORMAT('mos',A5,'-obj-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3002) PREFIX, IEL, IEH
 3002                   FORMAT('mos',A5,'-obj-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3003) PREFIX, IEL, IEH
 3003               FORMAT('mos',A5,'-obj-im-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3004) PREFIX, IEL, IEH
 3004               FORMAT('mos',A5,'-obj-im-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3005) PREFIX, IEL, IEH
 3005               FORMAT('mos',A5,'-obj-im-',I3,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3006) PREFI, IEL, IEH
 3006                   FORMAT('pn',A4,'-obj-im-',I4,'-',I5,'.fits')
                        WRITE(INFILEO,2906) PREFI, IEL, IEH
 2906                   FORMAT('pn',A4,'-obj-im-',I4,'-',I5,'-oot.fits')
                    ELSE
                        WRITE(INFILE,3007) PREFI, IEL, IEH
 3007                   FORMAT('pn',A4,'-obj-im-',I4,'-',I4,'.fits')
                        WRITE(INFILEO,2907) PREFI, IEL, IEH
 2907                   FORMAT('pn',A4,'-obj-im-',I4,'-',I4,'-oot.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3008) PREFI, IEL, IEH
 3008               FORMAT('pn',A4,'-obj-im-',I3,'-',I3,'.fits')
                    WRITE(INFILEO,2908) PREFI, IEL, IEH
 2908               FORMAT('pn',A4,'-obj-im-',I3,'-',I3,'-oot.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3009) PREFI, IEL, IEH
 3009               FORMAT('pn',A4,'-obj-im-',I3,'-',I5,'.fits')
                    WRITE(INFILEO,2909) PREFI, IEL, IEH
 2909               FORMAT('pn',A4,'-obj-im-',I3,'-',I5,'-oot.fits')
                ELSE
                    WRITE(INFILE,3010) PREFI, IEL, IEH
 3010               FORMAT('pn',A4,'-obj-im-',I3,'-',I4,'.fits')
                    WRITE(INFILEO,2910) PREFI, IEL, IEH
 2910               FORMAT('pn',A4,'-obj-im-',I3,'-',I4,'-oot.fits')
                ENDIF
            ENDIF
        ELSE
            IF(IEL .GE. 1000) THEN
                IF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3011) IEL, IEH
 3011               FORMAT('comb-obj-im-',I4,'-',I4,'.fits')
                ELSE
                    WRITE(INFILE,3012) IEL, IEH
 3012               FORMAT('comb-obj-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3013) IEL, IEH
 3013           FORMAT('comb-obj-im-',I3,'-',I3,'.fits')
            ELSEIF(IEH .GE. 10000) THEN
                WRITE(INFILE,3014) IEL, IEH
 3014           FORMAT('comb-obj-im-',I3,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3015) IEL, IEH
 3015           FORMAT('comb-obj-im-',I3,'-',I4,'.fits')
            ENDIF
        ENDIF

        CALL RETFIT (INFILE,COUNTS,INI4,1,900, &
                900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
        write(errstr,*) "ADAPT: Retrieving image: ", trim(infile)
        call message(AppMsg,Sparse)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT 01a: STATUS=",status, &
                    " on return from retfit"
            call message(AppMsg,Always)
            call error ("noRet","Error reading image")
        endif
        SCA = -1.0/(CDELT(1)*CDELT(2))
        IF(DETECT .EQ. 2) THEN
            CALL RETFIT (INFILEO,COUNTSO,INI4,1,900, &
                    900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            write(errstr,*) "ADAPT: Retrieving image: ", trim(infile)
            call message(AppMsg,Sparse)
            if(status .ne. 0) then
                write(errstr,*) "ADAPT 01b: STATUS=",status, &
                        " on return from retfit"
                call message(AppMsg,Always)
                call error ("noRet","Error reading image")
            endif

!  This is a bit painful, but we need to know the SUBMODE 
!  keyword for the proper scaling of the OOT events

!  Get an unused lun

            call ftgiou(lun,status)
            if(status .ne. 0) then
                write(errstr,*) "ADAPT 02: STATUS=",status, &
                    " on return from ftgiou"
                call message(AppMsg,Always)
                call error ("noLUN","Cannot aquire LUN")
            endif

!  Open the FITS file

            call ftopen (lun,infileo,0,blockn,status)
            write(errstr,*) "ADAPT: Opening file: ", trim(infile)
            call message(AppMsg,Sparse)
            if(status .ne. 0) then
                write(errstr,*) "ADAPT 03: STATUS=",status, &
                        " on return from ftopen"
                call message(AppMsg,Always)
                call error ("noOpen","Cannot open file")
            endif

!  Read in the mode information if a PN observation

            call ftgkys (lun,'SUBMODE',mode,com,status)
            if(status .ne. 0) then
                write(errstr,*) "ADAPT 04: STATUS=",status, &
                        " on return from ftgkys"
                call message(AppMsg,Always)
                call error ("noSubmode","Missing SUBMODE keyword")
            else
                if(mode .eq. 'PrimeFullWindow') ootsca = 0.063
                if(mode .eq. 'PrimeFullWindowExtended') ootsca = 0.023
                if(mode .eq. 'PrimeFullWindowExtended5') ootsca = 0.013
            endif
            write(errstr,*) "ADAPT: PN OOT scale factor: ", ootsca
            call message(AppMsg,Sparse)

!  Close the FITS file

            call ftclos(lun,status)
            if(status .ne. 0) then
                write(errstr,*) "ADAPT 05: STATUS=",status, &
                        " on return from ftclos"
                call message(AppMsg,Always)
                call error ("noClose","Cannot close file")
            endif
            DO I=1,900
                DO II=1,900
                    COUNTS(I,II) = COUNTS(I,II) - OOTSCA*COUNTSO(I,II)
                ENDDO
            ENDDO
        ENDIF

!  Enter the exposure array file name

        IF(IDET .GT. 0) THEN
            IF(DETECT .EQ. 1) THEN
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3101) PREFIX, IEL, IEH
 3101                   FORMAT('mos',A5,'-exp-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3102) PREFIX, IEL, IEH
 3102                   FORMAT('mos',A5,'-exp-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3103) PREFIX, IEL, IEH
 3103               FORMAT('mos',A5,'-exp-im-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3104) PREFIX, IEL, IEH
 3104               FORMAT('mos',A5,'-exp-im-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3105) PREFIX, IEL, IEH
 3105               FORMAT('mos',A5,'-exp-im-',I3,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3106) PREFI, IEL, IEH
 3106                   FORMAT('pn',A4,'-exp-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3107) PREFI, IEL, IEH
 3107                   FORMAT('pn',A4,'-exp-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3108) PREFI, IEL, IEH
 3108               FORMAT('pn',A4,'-exp-im-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3109) PREFI, IEL, IEH
 3109               FORMAT('pn',A4,'-exp-im-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3110) PREFI, IEL, IEH
 3110               FORMAT('pn',A4,'-exp-im-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF

        ELSE
            IF(IEL .GE. 1000) THEN
                IF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3111) IEL, IEH
 3111               FORMAT('comb-exp-im-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3112) IEL, IEH
 3112               FORMAT('comb-exp-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3113) IEL, IEH
 3113           FORMAT('comb-exp-im-',I3,'-',I3,'.fits')
            ELSEIF(IEH .GE. 10000) THEN
                WRITE(INFILE,3114) IEL, IEH
 3114           FORMAT('comb-exp-im-',I3,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3115) IEL, IEH
 3115           FORMAT('comb-exp-im-',I3,'-',I4,'.fits')
            ENDIF
        ENDIF

        CALL RETFIT (INFILE,EXPOSE,INI4,1,900, &
                900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
        write(errstr,*) "ADAPT: Retrieving image: ", trim(infile)
        call message(AppMsg,Sparse)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT 05a: STATUS=",status, &
                    " on return from retfit"
            call message(AppMsg,Always)
            call error ("noRet","Error reading image")
        endif

!  Enter the background count array file name

        IF(IPART) THEN
            IF(IDET .GT. 0) THEN
                IF(DETECT .EQ. 1) THEN
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3201) PREFIX, IEL, IEH
 3201                       FORMAT('mos',A5,'-back-im-sky-',I4,'-', &
                                    I5,'.fits')
                        ELSE
                            WRITE(INFILE,3202) PREFIX, IEL, IEH
 3202                       FORMAT('mos',A5,'-back-im-sky-',I4,'-', &
                                    I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3203) PREFIX, IEL, IEH
 3203                   FORMAT('mos',A5,'-back-im-sky-',I3,'-', &
                                I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3204) PREFIX, IEL, IEH
 3204                   FORMAT('mos',A5,'-back-im-sky-',I3,'-', &
                                I5,'.fits')
                    ELSE
                        WRITE(INFILE,3205) PREFIX, IEL, IEH
 3205                   FORMAT('mos',A5,'-back-im-sky-',I3,'-', &
                                I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3206) PREFI, IEL, IEH
 3206                       FORMAT('pn',A4,'-back-im-sky-',I4,'-', &
                                    I5,'.fits')
                        ELSE
                            WRITE(INFILE,3207) PREFI, IEL, IEH
 3207                       FORMAT('pn',A4,'-back-im-sky-',I4,'-', &
                                    I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3208) PREFI, IEL, IEH
 3208                   FORMAT('pn',A4,'-back-im-sky-',I3,'-', &
                                I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3209) PREFI, IEL, IEH
 3209                   FORMAT('pn',A4,'-back-im-sky-',I3,'-', &
                                I5,'.fits')
                    ELSE
                        WRITE(INFILE,3210) PREFI, IEL, IEH
 3210                   FORMAT('pn',A4,'-back-im-sky-',I3,'-', &
                                I4,'.fits')
                    ENDIF
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEL .GE. 10000) THEN
                        WRITE(INFILE,3211) IEL, IEH
 3211                   FORMAT('comb-back-im-sky-',I4,'-',I5,'.fits')
                    ELSE                IF(IEL .GE. 1000) THEN
                        WRITE(INFILE,3212) IEL, IEH
 3212                   FORMAT('comb-back-im-sky-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3213) IEL, IEH
 3213               FORMAT('comb-back-im-sky-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3214) IEL, IEH
 3214               FORMAT('comb-back-im-sky-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3215) IEL, IEH
 3215               FORMAT('comb-back-im-sky-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF
        ELSE
            INFILE = '    '
        ENDIF
        IF(INFILE .NE. '    ') THEN
            CALL RETFIT (INFILE,BACK,INI4,1,900, &
                        900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            write(errstr,*) "ADAPT: Retrieving image: ", trim(infile)
            call message(AppMsg,Sparse)
            if(status .gt. 0) then
                write(errstr,*) "ADAPT 06: STATUS=",status, &
                        " on return from retfit"
                call message(AppMsg,Always)
                call error ("noRet","Error reading image")
            endif
        ELSE
            DO I=1,900
                DO II=1,900
                    BACK(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF

!  Read in the soft proton count data

        IF(IPROT) THEN
            IF(IDET .GT. 0) THEN
                IF(DETECT .EQ. 1) THEN
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3301) PREFIX, IEL, IEH
 3301                       FORMAT('mos',A5,'-prot-im-sky-',I4,'-', &
                                    I5,'.fits')
                        ELSE
                            WRITE(INFILE,3302) PREFIX, IEL, IEH
 3302                       FORMAT('mos',A5,'-prot-im-sky-',I4,'-', &
                                    I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3303) PREFIX, IEL, IEH
 3303                   FORMAT('mos',A5,'-prot-im-sky-',I3,'-', &
                                I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3304) PREFIX, IEL, IEH
 3304                   FORMAT('mos',A5,'-prot-im-sky-',I3,'-', &
                                I5,'.fits')
                    ELSE
                        WRITE(INFILE,3305) PREFIX, IEL, IEH
 3305                   FORMAT('mos',A5,'-prot-im-sky-',I3,'-', &
                                I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3306) PREFI, IEL, IEH
 3306                       FORMAT('pn',A4,'-prot-im-sky-',I4,'-', &
                                    I5,'.fits')
                        ELSE
                            WRITE(INFILE,3307) PREFI, IEL, IEH
 3307                       FORMAT('pn',A4,'-prot-im-sky-',I4,'-', &
                                    I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3308) PREFI, IEL, IEH
 3308                   FORMAT('pn',A4,'-prot-im-sky-',I3,'-', &
                                I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3309) PREFI, IEL, IEH
 3309                   FORMAT('pn',A4,'-prot-im-sky-',I3,'-', &
                                I5,'.fits')
                    ELSE
                        WRITE(INFILE,3310) PREFI, IEL, IEH
 3310                   FORMAT('pn',A4,'-prot-im-sky-',I3,'-', &
                                I4,'.fits')
                    ENDIF
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3311) IEL, IEH
 3311                   FORMAT('comb-prot-im-sky-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3312) IEL, IEH
 3312                   FORMAT('comb-prot-im-sky-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3313) IEL, IEH
 3313               FORMAT('comb-prot-im-sky-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3314) IEL, IEH
 3314               FORMAT('comb-prot-im-sky-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3315) IEL, IEH
 3315               FORMAT('comb-prot-im-sky-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF
        ELSE
            INFILE = '    '
        ENDIF
        IF(INFILE .NE. '    ') THEN
            CALL RETFIT (INFILE,PROT,INI4,1,900, &
                        900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            write(errstr,*) "ADAPT: Retrieving image: ", trim(infile)
            call message(AppMsg,Sparse)
            if(status .gt. 0) then
                write(errstr,*) "ADAPT 07: STATUS=",status, &
                        " on return from retfit"
                call message(AppMsg,Always)
                call error ("noRet","Error reading image")
            endif
        ELSE
            DO I=1,900
                DO II=1,900
                    PROT(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF

!  Read in the SWCX count data

        IF(ISWCX) THEN
            IF(IDET .GT. 0) THEN
                IF(DETECT .EQ. 1) THEN
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3401) PREFIX, IEL, IEH
 3401                       FORMAT('mos',A5,'-swcx-im-sky-',I4,'-', &
                                    I5,'.fits')
                        ELSE
                            WRITE(INFILE,3402) PREFIX, IEL, IEH
 3402                       FORMAT('mos',A5,'-swcx-im-sky-',I4,'-', &
                                    I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3403) PREFIX, IEL, IEH
 3403                   FORMAT('mos',A5,'-swcx-im-sky-',I3,'-', &
                                I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3404) PREFIX, IEL, IEH
 3404                   FORMAT('mos',A5,'-swcx-im-sky-',I3,'-', &
                                I5,'.fits')
                    ELSE
                        WRITE(INFILE,3405) PREFIX, IEL, IEH
 3405                   FORMAT('mos',A5,'-swcx-im-sky-',I3,'-', &
                                I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEL .GE. 1000) THEN
                        IF(IEH .GE. 10000) THEN
                            WRITE(INFILE,3406) PREFI, IEL, IEH
 3406                       FORMAT('pn',A4,'-swcx-im-sky-',I4,'-', &
                                    I5,'.fits')
                        ELSE
                            WRITE(INFILE,3407) PREFI, IEL, IEH
 3407                       FORMAT('pn',A4,'-swcx-im-sky-',I4,'-', &
                                    I4,'.fits')
                        ENDIF
                    ELSEIF(IEH .LT. 1000) THEN
                        WRITE(INFILE,3408) PREFI, IEL, IEH
 3408                   FORMAT('pn',A4,'-swcx-im-sky-',I3,'-', &
                                I3,'.fits')
                    ELSEIF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3409) PREFI, IEL, IEH
 3409                   FORMAT('pn',A4,'-swcx-im-sky-',I3,'-', &
                                I5,'.fits')
                    ELSE
                        WRITE(INFILE,3410) PREFI, IEL, IEH
 3410                   FORMAT('pn',A4,'-swcx-im-sky-',I3,'-', &
                                I4,'.fits')
                    ENDIF
                ENDIF
            ELSE
                IF(IEL .GE. 1000) THEN
                    IF(IEH .GE. 10000) THEN
                        WRITE(INFILE,3411) IEL, IEH
 3411                   FORMAT('comb-swcx-im-sky-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3412) IEL, IEH
 3412                   FORMAT('comb-swcx-im-sky-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH .LT. 1000) THEN
                    WRITE(INFILE,3413) IEL, IEH
 3413               FORMAT('comb-swcx-im-sky-',I3,'-',I3,'.fits')
                ELSEIF(IEH .GE. 10000) THEN
                    WRITE(INFILE,3414) IEL, IEH
 3414               FORMAT('comb-swcx-im-sky-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3415) IEL, IEH
 3415               FORMAT('comb-swcx-im-sky-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF
        ELSE
            INFILE = '    '
        ENDIF
        IF(INFILE .NE. '    ') THEN
            CALL RETFIT (INFILE,SWCX,INI4,1,900, &
                        900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            write(errstr,*) "ADAPT: Retrieving image: ", trim(infile)
            call message(AppMsg,Sparse)
            if(status .gt. 0) then
                write(errstr,*) "ADAPT 08: STATUS=",status, &
                        " on return from retfit"
                call message(AppMsg,Always)
                call error ("noRet","Error reading image")
            endif
        ELSE
            DO I=1,900
                DO II=1,900
                    SWCX(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF

!  Find out how many counts we have to play with and the exposure center

        RNC = 0.0
        RNB = 0.0
        RNP = 0.0
        RNS = 0.0
        REX = 0.0
        EX = 0.0
        DO I=1,900
            DO II=1,900
                RNC = RNC + COUNTS(I,II)
                RNB = RNB + BACK(I,II)
                RNP = RNP + PROT(I,II)
                RNS = RNS + SWCX(I,II)
                REX = REX + EXPOSE(I,II)
                BACK(I,II) = BACK(I,II) + PROT(I,II) + SWCX(I,II)
                IF(EXPOSE(I,II) .GT. EX) THEN
                    EX = EXPOSE(I,II)
                    XCEN = I
                    YCEN = II
                ENDIF
            ENDDO
        ENDDO
        write(errstr,*) "Total Counts: ", RNC
        call message(AppMsg,Sparse)
        write(errstr,*) "Exposure: ", REX
        call message(AppMsg,Sparse)
        write(errstr,*) "QPB Counts: ", RNB
        call message(AppMsg,Sparse)
        write(errstr,*) "Proton Counts: ", RNP
        call message(AppMsg,Sparse)
        write(errstr,*) "SWCX Counts: ", RNS
        call message(AppMsg,Sparse)

!  Read in the template

        IF(MASKFILE .EQ. '      ') THEN
!  Use the exposure maps as the template

            DO I=1,900
                DO II=1,900
                    MASK(I,II) = EXPOSE(I,II)
                ENDDO
            ENDDO
        ELSE

!  Return the mask

            CALL RETFIT (MASKFILE,MASK,INI4,1,900, &
                        900,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            write(errstr,*) "ADAPT: Retrieving image: ", trim(maskfile)
            call message(AppMsg,Sparse)
            if(status .gt. 0) then
                write(errstr,*) "ADAPT 09: STATUS=",status, &
                        " on return from retfit"
                call message(AppMsg,Always)
                call error ("noRet","Error reading image")
            endif
        ENDIF

!  Set the template limit

        J = 0
        TLIM = 0.
        DO I=1,900
            DO II=1,900
                IF(MASK(I,II) .GT. 0) THEN
                    TLIM = TLIM + MASK(I,II)
                    J = J + 1
                ENDIF
            ENDDO
        ENDDO
        TLIM = TEMSC*TLIM/J

!  Make a radial profile

        DO I=1,50
            CTS(I) = 0.0
            BCK(I) = 0.0
            EXPO(I) = 0.0
            N(I) = 0
        ENDDO

        DO I=1,900
            DO II=1,900
               IF(MASK(I,II) .GT. 0.0) THEN
                    R = 2.5*SQRT((I-XCEN)**2 + (II-YCEN)**2)/60.0
                    IR = R + 1.0
                    IF(IR .LE. 50) THEN
                        CTS(IR) = CTS(IR) + COUNTS(I,II)
                        BCK(IR) = BCK(IR) + BACK(I,II)
                        EXPO(IR) = EXPO(IR) + EXPOSE(I,II)
                        N(IR) = N(IR) + 1
                    ENDIF
                ENDIF
            ENDDO
        ENDDO
        XMAX = 0.0
        YMAX = 0.0
        DO I=1,30
            IF(EXPO(I) .GT. 0.0) THEN
                X(I) = I - 0.5
                Y(I) = SCA*(CTS(I) - BCK(I))/EXPO(I)
                Y2(I) = SCA*CTS(I)/EXPO(I)
                YS(I) = SCA*SQRT(CTS(I))/EXPO(I)
                IF(CTS(I) .GT. 100.0) THEN
                    XMAX = I
                    IF(Y2(I) .GT. YMAX) YMAX = Y2(I)+YS(I)
                ENDIF
                write(errstr,2990) X(I), Y(I), YS(I), &
                        N(I), CTS(I)/N(I), &
                        BCK(I)/N(I), EXPO(I)/N(I)
 2990           FORMAT(1X,3f12.5,i12,2f12.5,f12.2)
                call message(AppMsg,Noisy)
            ELSE
                X(I) = 0.0
                Y(I) = 0.0
                Y2(I) = 0.0
                YS(I) = 0.0
            ENDIF
        ENDDO
        YMAX = 1.05*YMAX
        IF(IDET .EQ. 0) THEN
            IF(IEL .GE. 1000) THEN
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3500) IEL, IEH
 3500               FORMAT('radial-filt-',I4,'-',I5,'.qdp')
                    WRITE(INFILES,3550) IEL, IEH
 3550               FORMAT('size-',I4,'-',I5,'.qdp')
                ELSE
                    WRITE(INFILE,3505) IEL, IEH
 3505               FORMAT('radial-filt-',I4,'-',I4,'.qdp')
                    WRITE(INFILES,3555) IEL, IEH
 3555               FORMAT('size-',I4,'-',I4,'.qdp')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3510) IEL, IEH
 3510           FORMAT('radial-filt-',I3,'-',I3,'.qdp')
                WRITE(INFILES,3560) IEL, IEH
 3560           FORMAT('size-',I3,'-',I3,'.qdp')
            ELSE
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3520) IEL, IEH
 3520               FORMAT('radial-filt-',I3,'-',I5,'.qdp')
                    WRITE(INFILES,3570) IEL, IEH
 3570               FORMAT('size-',I3,'-',I5,'.qdp')
                ELSE
                    WRITE(INFILE,3525) IEL, IEH
 3525               FORMAT('radial-filt-',I3,'-',I4,'.qdp')
                    WRITE(INFILES,3575) IEL, IEH
 3575               FORMAT('size-',I3,'-',I4,'.qdp')
                ENDIF
            ENDIF
        ELSEIF(DETECT .EQ. 1) THEN
            IF(IEL .GE. 1000) THEN
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3501) PREFIX, IEL, IEH
 3501               FORMAT('mos',A5,'-radial-filt-',I4,'-',I5,'.qdp')
                    WRITE(INFILES,3551) PREFIX, IEL, IEH
 3551               FORMAT('mos',A5,'-size-',I4,'-',I5,'.qdp')
                ELSE
                    WRITE(INFILE,3506) PREFIX, IEL, IEH
 3506               FORMAT('mos',A5,'-radial-filt-',I4,'-',I4,'.qdp')
                    WRITE(INFILES,3556) PREFIX, IEL, IEH
 3556               FORMAT('mos',A5,'-size-',I4,'-',I4,'.qdp')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3511) PREFIX, IEL, IEH
 3511           FORMAT('mos',A5,'-radial-filt-',I3,'-',I3,'.qdp')
                WRITE(INFILES,3561) PREFIX, IEL, IEH
 3561           FORMAT('mos',A5,'-size-',I3,'-',I3,'.qdp')
            ELSE
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3521) PREFIX, IEL, IEH
 3521               FORMAT('mos',A5,'-radial-filt-',I3,'-',I5,'.qdp')
                    WRITE(INFILES,3571) PREFIX, IEL, IEH
 3571               FORMAT('mos',A5,'-size-',I3,'-',I5,'.qdp')
                ELSE
                    WRITE(INFILE,3526) PREFIX, IEL, IEH
 3526               FORMAT('mos',A5,'-radial-filt-',I3,'-',I4,'.qdp')
                    WRITE(INFILES,3576) PREFIX, IEL, IEH
 3576               FORMAT('mos',A5,'-size-',I3,'-',I4,'.qdp')
                ENDIF
            ENDIF
        ELSE
            IF(IEL .GE. 1000) THEN
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3502) PREFI, IEL, IEH
 3502               FORMAT('pn',A4,'-radial-filt-',I4,'-',I5,'.qdp')
                    WRITE(INFILES,3552) PREFI, IEL, IEH
 3552               FORMAT('pn',A4,'-size-',I4,'-',I5,'.qdp')
                ELSE
                    WRITE(INFILE,3507) PREFI, IEL, IEH
 3507               FORMAT('pn',A4,'-radial-filt-',I4,'-',I4,'.qdp')
                    WRITE(INFILES,3557) PREFI, IEL, IEH
 3557               FORMAT('pn',A4,'-size-',I4,'-',I4,'.qdp')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3512) PREFI, IEL, IEH
 3512           FORMAT('pn',A4,'-radial-filt-',I3,'-',I3,'.qdp')
                WRITE(INFILES,3562) PREFI, IEL, IEH
 3562           FORMAT('pn',A4,'-size-',I3,'-',I3,'.qdp')
            ELSE
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3522) PREFI, IEL, IEH
 3522               FORMAT('pn',A4,'-radial-filt-',I3,'-',I5,'.qdp')
                    WRITE(INFILES,3572) PREFI, IEL, IEH
 3572               FORMAT('pn',A4,'-size-',I3,'-',I5,'.qdp')
                ELSE
                    WRITE(INFILE,3527) PREFI, IEL, IEH
 3527               FORMAT('pn',A4,'-radial-filt-',I3,'-',I4,'.qdp')
                    WRITE(INFILES,3577) PREFI, IEL, IEH
 3577               FORMAT('pn',A4,'-size-',I3,'-',I4,'.qdp')
                ENDIF
            ENDIF
        ENDIF

!  Get an unused lun

        call ftgiou(lun,status)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT 10: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

        IF(CLOBBER) THEN
            OPEN(UNIT=LUN,STATUS='UNKNOWN',FORM='FORMATTED',FILE=INFILE)
        ELSE
            OPEN(UNIT=LUN,STATUS='NEW',FORM='FORMATTED',FILE=INFILE, &
                IOSTAT=IOS)
            IF(IOS .EQ. 0) THEN
                write(errstr,*) "ADAPT 11, Output file: ", trim(infile)
                call message(AppMsg,Always)
                call error ("noClobber","File exists and no clobber")
            ENDIF
        ENDIF
        WRITE(LUN,*) 'VIEWPORT 0.15 0.15 0.9 0.85'
        WRITE(LUN,*) 'READ SERR 2'
        WRITE(LUN,*) 'FONT ROMAN'
        WRITE(LUN,*) 'CSIZE 1.8'
        WRITE(LUN,*) 'LA F'
        WRITE(LUN,*) 'LA X Radius (arc minutes)'
        WRITE(LUN,*) 'LA Y Surface Brightness (counts/s/deg\u2\d)'
!        WRITE(LUN,*) &
!        'LA Y Surface Brightness (counts s\u-1\d deg\u-2\d)'
        WRITE(LUN,*) 'COL 1 ON 2'
        WRITE(LUN,*) 'COL 2 ON 3'
        WRITE(LUN,*) 'R X 0 ', XMAX
        WRITE(LUN,*) 'R Y 0 ', YMAX
        WRITE(LUN,*) 'LINE STEP'
        DO I=1,50
            IF(YS(I) .GT. 0.0) THEN
                WRITE(LUN,*) X(I), Y(I), YS(I), Y2(I)
            ENDIF
        ENDDO
        CLOSE(LUN)

!  Set the binning limits

       i2 = 900/ibin

!  Bin the data

        write(errstr,*) "START THE BINNING"
        call message(AppMsg,Sparse)
        DO I=1,I2
            J2 = I*IBIN + ioff
            J1 = J2 - IBIN + 1
            DO II=1,I2
                JJ2 = II*IBIN + ioff
                JJ1 = JJ2 - IBIN + 1
                B = 0.
                C = 0.
                E = 0.
                DO J=J1,J2
                    DO JJ=JJ1,JJ2
                        IF(MASK(J,JJ) .GT. TLIM) THEN
                            B = B + BACK(J,JJ)
                            C = C + COUNTS(J,JJ)
                            E = E + EXPOSE(J,JJ)
                        ENDIF
                    ENDDO
                ENDDO
                BACK(I,II) = B
                COUNTS(I,II) = C
                EXPOSE(I,II) = E
            ENDDO
        ENDDO

!  Now smooth the data

        write(errstr,*) "START THE SMOOTHING"
        call message(AppMsg,Sparse)
        DO I=1,I2
            IF(MOD(I,25) .EQ. 0) then
                write(errstr,*) "Column: ", I
                call message(AppMsg,Sparse)
            endif
            DO II=1,I2
                IF(EXPOSE(I,II) .GT. 0.) THEN
                    B = 0.
                    C = 0.
                    E = 0.
                    NUM = 0
                    K = -1
                    RAD = 0.0
                    DO WHILE (C .LT. SN)
                        BLAST = B
                        CLAST = C
                        ELAST = E
                        NUMLAST = NUM
                        K = K + 1
                        IF(K .EQ. 1) THEN
                            RAD = 1.5*1.5
                        ELSEIF(K .EQ. 2) THEN
                            RAD = 2.5*2.5
                        ELSEIF(K .EQ. 3) THEN
                            RAD = 3.2*3.2
                        ELSE
                            RAD = K*K*1.05*1.05
                        ENDIF
                        J1 = I - K
                        IF(J1 .LT. 1) J1 = 1
                        J2 = I + K
                        IF(J2 .GT. I2) J2 = I2
                        JJ1 = II - K
                        IF(JJ1 .LT. 1) JJ1 = 1
                        JJ2 = II + K
                        IF(JJ2 .GT. I2) JJ2 = I2
                        B = 0.
                        C = 0.
                        E = 0.
                        NUM = 0
                        DO J=J1,J2
                            DO JJ=JJ1,JJ2
                                TRAD = (J-I)**2 + (JJ-II)**2
                                IF((EXPOSE(J,JJ) .GT. 0.) .AND. &
                                       (TRAD .LE. RAD)) THEN
                                    IF(RAD .EQ. 0.0) THEN
                                        B = B + BACK(J,JJ)
                                        C = C + COUNTS(J,JJ)
                                        E = E + EXPOSE(J,JJ)
                                    ELSE
                                        T = EXP(-((TRAD/RAD)**2))
                                        B = B + BACK(J,JJ)*T
                                        C = C + COUNTS(J,JJ)*T
                                        E = E + EXPOSE(J,JJ)*T
                                    ENDIF
                                    NUM = NUM + 1
                                ENDIF
                            ENDDO
                        ENDDO
                    ENDDO
                    IF((K .GT. 1) .AND. (NUMLAST .GT. 0.)) THEN
                        TEMP = SCA*(CLAST - BLAST)/ELAST
                        OUTPUT(I,II) = TEMP
                    ELSEIF(NUM .GT. 0.) THEN
                        TEMP = SCA*(C - B)/E
                        OUTPUT(I,II) = TEMP
                    ELSE
                        OUTPUT(I,II) = TEMP
                    ENDIF
                    RAD = 2.5*SQRT(RAD)/60.0
                    SIZE(I,II) = RAD
                    IR = 15*RAD + 1
                    YH(IR) = YH(IR) + 1
                ENDIF
            ENDDO
        ENDDO

!  Get an unused lun

        call ftgiou(lun,status)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT 12: STATUS=",status, &
                " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

        YMAX = 0.0
        DO I=1,100
            IF(YH(I) .GT. YMAX) YMAX = YH(I)
        ENDDO

        IF(CLOBBER) THEN
            OPEN(UNIT=LUN,STATUS='UNKNOWN',FORM='FORMATTED', &
                FILE=INFILES)
        ELSE
            OPEN(UNIT=LUN,STATUS='NEW',FORM='FORMATTED',FILE=INFILES, &
                IOSTAT=IOS)
            IF(IOS .EQ. 0) THEN
                write(errstr,*) "ADAPT 13: Output file: ", trim(infile)
                call message(AppMsg,Always)
                call error ("noClobber","File exists and no clobber")
            ENDIF
        ENDIF
        WRITE(LUN,*) 'VIEWPORT 0.15 0.15 0.95 0.85'
        WRITE(LUN,*) 'FONT ROMAN'
        WRITE(LUN,*) 'CSIZE 1.8'
        WRITE(LUN,*) 'LA T Smoothing FWHM Histogram'
        WRITE(LUN,*) 'LA F'
        WRITE(LUN,*) 'LA Y Number'
        WRITE(LUN,*) 'LA X Conical Sampling FWHM (arcmin)'
        WRITE(LUN,*) 'R X 0 ', 5.0
        WRITE(LUN,*) 'R Y 0 ', 1.05*YMAX
        WRITE(LUN,*) 'LINE STEP'
        WRITE(LUN,*) 'COL 1 ON 2'
        DO I=1,50
            WRITE(LUN,*) XH(I), YH(I)
        ENDDO
        CLOSE(LUN)

!  Open the new FITS file

        IF(IDET .EQ. 0) THEN
            IF(IEL .GE. 1000) THEN
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3600) IEL, IEH
 3600               FORMAT('adapt-',I4,'-',I5,'.fits')
                    WRITE(INFILES,3650) IEL, IEH
 3650               FORMAT('size-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3605) IEL, IEH
 3605               FORMAT('adapt-',I4,'-',I4,'.fits')
                    WRITE(INFILES,3655) IEL, IEH
 3655               FORMAT('size-',I4,'-',I4,'.fits')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3610) IEL, IEH
 3610           FORMAT('adapt-',I3,'-',I3,'.fits')
                WRITE(INFILES,3660) IEL, IEH
 3660           FORMAT('size-',I3,'-',I3,'.fits')
            ELSE
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3620) IEL, IEH
 3620               FORMAT('adapt-',I3,'-',I5,'.fits')
                    WRITE(INFILES,3670) IEL, IEH
 3670               FORMAT('size-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3625) IEL, IEH
 3625               FORMAT('adapt-',I3,'-',I4,'.fits')
                    WRITE(INFILES,3675) IEL, IEH
 3675               FORMAT('size-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF
        ELSEIF(DETECT .EQ. 1) THEN
            IF(IEL .GE. 1000) THEN
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3601) PREFIX, IEL, IEH
 3601               FORMAT('mos',A5,'-adapt-',I4,'-',I5,'.fits')
                    WRITE(INFILES,3651) PREFIX, IEL, IEH
 3651               FORMAT('mos',A5,'-size-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3606) PREFIX, IEL, IEH
 3606               FORMAT('mos',A5,'-adapt-',I4,'-',I4,'.fits')
                    WRITE(INFILES,3656) PREFIX, IEL, IEH
 3656               FORMAT('mos',A5,'-size-',I4,'-',I4,'.fits')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3611) PREFIX, IEL, IEH
 3611           FORMAT('mos',A5,'-adapt-',I3,'-',I3,'.fits')
                WRITE(INFILES,3661) PREFIX, IEL, IEH
 3661           FORMAT('mos',A5,'-size-',I3,'-',I3,'.fits')
            ELSE
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3621) PREFIX, IEL, IEH
 3621               FORMAT('mos',A5,'-adapt-',I3,'-',I5,'.fits')
                    WRITE(INFILES,3671) PREFIX, IEL, IEH
 3671               FORMAT('mos',A5,'-size-',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3626) PREFIX, IEL, IEH
 3626               FORMAT('mos',A5,'-adapt-',I3,'-',I4,'.fits')
                    WRITE(INFILES,3676) PREFIX, IEL, IEH
 3676               FORMAT('mos',A5,'-size-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF
        ELSE
            IF(IEL .GE. 1000) THEN
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3602) PREFI, IEL, IEH
 3602               FORMAT('pn',A4,'-adapt-',I4,'-',I5,'.fits')
                    WRITE(INFILES,3652) PREFI, IEL, IEH
 3652               FORMAT('pn',A4,'-size-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3607) PREFI, IEL, IEH
 3607               FORMAT('pn',A4,'-adapt-',I4,'-',I4,'.fits')
                    WRITE(INFILES,3657) PREFI, IEL, IEH
 3657               FORMAT('pn',A4,'-size-',I4,'-',I4,'.fits')
                ENDIF
            ELSEIF(IEH .LT. 1000) THEN
                WRITE(INFILE,3612) PREFI, IEL, IEH
 3612           FORMAT('pn',A4,'-adapt-',I3,'-',I3,'.fits')
                WRITE(INFILES,3662) PREFI, IEL, IEH
 3662           FORMAT('pn',A4,'-size-',I3,'-',I3,'.fits')
            ELSE
                IF(IEH .GT. 10000) THEN
                    WRITE(INFILE,3622) PREFI, IEL, IEH
 3622               FORMAT('pn',A4,'-adapt-',I3,'-',I5,'.fits')
                    WRITE(INFILES,3672) PREFI, IEL, IEH
 3672               FORMAT('pn',A4,'-size',I3,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3627) PREFI, IEL, IEH
 3627               FORMAT('pn',A4,'-adapt-',I3,'-',I4,'.fits')
                    WRITE(INFILES,3677) PREFI, IEL, IEH
 3677               FORMAT('pn',A4,'-size-',I3,'-',I4,'.fits')
                ENDIF
            ENDIF
        ENDIF

!  Check to see if there is an existing file

        CALL FTOPEN (LUN,INFILE,0,BLOCKN,STATUS)
        IF(STATUS .EQ. 0) THEN
            write(errstr,*) "File exists with output file name: ", &
                    trim(infile)
            call message(AppMsg,Sparse)
            IF(CLOBBER) THEN
                write(errstr,*) "Clobber the file and continue"
                call message(AppMsg,Sparse)
                CALL FTDELT (LUN,STATUS)
                if(status .ne. 0) then
                    write(errstr,*) "ADAPT 14: STATUS=",status, &
                            " on return from ftdelt"
                    call message(AppMsg,Always)
                    call error ("badDelete","Error deleting file")
                endif
            ELSE
                write(errstr,*) "ADAPT 15: Output file: ", trim(infile)
                call message(AppMsg,Always)
                call error ("noClobber","File exists and no clobber")
            ENDIF
        ENDIF
        STATUS = 0

        COMMENT = 'Adaptive-filter smoothed count-rate map for EPIC'

        IFLAG = 1
        CDELT(1) = IBIN*CDELT(1)
        CDELT(2) = IBIN*CDELT(2)
        CRPIX(1) = xdim/2 + 0.5
        CRPIX(2) = ydim/2 + 0.5
        UNITS = 'c/s/deg2'
        BZERO = 0.0
        BSCALE = 1.0

        CALL PUTFIT(INFILE,OUTPUT,INI4,IFLAG,UNITS, &
                COMMENT,xdim,ydim,BZERO,BSCALE,CTYPE,CRVAL, &
                CDELT,CRPIX,CLOBBER,STATUS)
        write(errstr,*) "ADAPT: Writing image: ", trim(infile)
        call message(AppMsg,Sparse)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT 16: STATUS=",status, &
                    " on return from putfit"
            call message(AppMsg,Always)
            call error ("noPut","Error writing image")
        endif

!  Check to see if there is an existing file

        CALL FTOPEN (LUN,INFILES,0,BLOCKN,STATUS)
        IF(STATUS .EQ. 0) THEN
            write(errstr,*) "File exists with output file name: ", &
                    trim(infiles)
            call message(AppMsg,Sparse)
            IF(CLOBBER) THEN
                write(errstr,*) "Clobber the file and continue"
                call message(AppMsg,Sparse)
                CALL FTDELT (LUN,STATUS)
                if(status .ne. 0) then
                    write(errstr,*) "ADAPT 17: STATUS=",status, &
                            " on return from ftdelt"
                    call message(AppMsg,Always)
                    call error ("badDelete","Error deleting file")
                endif
            ELSE
                write(errstr,*) "ADAPT 18: Output file: ", trim(infile)
                call message(AppMsg,Always)
                call error ("noClobber","File exists and no clobber")
            ENDIF
        ENDIF
        STATUS = 0

        COMMENT = 'Adaptive-filter smoothing FWHM (arcmin)'

        CALL PUTFIT(INFILES,SIZE,INI4,IFLAG,UNITS, &
                COMMENT,xdim,ydim,BZERO,BSCALE,CTYPE,CRVAL, &
                CDELT,CRPIX,CLOBBER,STATUS)
        write(errstr,*) "ADAPT: Writing image: ", trim(infile)
        call message(AppMsg,Sparse)
        if(status .ne. 0) then
            write(errstr,*) "ADAPT 19: STATUS=",status, &
                    " on return from putfit"
            call message(AppMsg,Always)
            call error ("noPut","Error writing image")
        endif

        RETURN

  end subroutine adapt

end module adapt_mod
