!+
! Module: bin_image_merge_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: bin_image_merge_mod.f90,v 1.0 2010/03/11 snowden Exp $
!-
!
!  Version 1.0  11 March 2010
!
!  Bins merged images from the ESAS processed data
!
!  INPUT:
!    temsc: Threshold for masking
!    nbands: Number of bands to merge
!    iel: List of lower energy limits for the energy bands in eV
!    ieh: List of upper energy limits for the energy bands in eV
!    ibin: 1: No binning, 2: Bin by 2, 4: Bin by 4, 8: Bin by 8, 
!          16: Bin by 16, 32: Bin by 32
!    ipart: Particle background control - 1: include, 2: exclude
!    isoft: Soft proton background control - 1: include, 2: exclude
!    maskfile: Mask file name 
!
!    Examples: bin_image_merge thresholdmasking=0.02 nbands=1 elow=2000 ehigh=7200 
!                   ibin=2 withpartcontrol=1 withsoftcontrol=0 
!              bin_image_merge thresholdmasking=0.02 nbands=2 elow="400 750" ehigh="750 1250" 
!                   withpartcontrol=1 withsoftcontrol=1 mask=maskfile.fits
!
!  OUPUT:
!    Binned count rate image and count rate uncertainty image
!

module bin_image_merge_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public bin_image_merge

contains

  !---------------------------------------------------------------------
  subroutine bin_image_merge
  !---------------------------------------------------------------------

        character(len=8) :: ctype(2), units
        character(len=60) :: comment
        character(len=132) :: infile, maskfile
        logical :: clobber, extend, imask, ipart, isoft, iswcx, simple
        integer, dimension(:), allocatable :: iel, ieh
        integer, dimension(:,:), allocatable :: ini4
        integer :: bitpix, blockn, gcount, i, ib, &
            ibin, iehmax, ielmin, iex, iey, iflag, &
            ii, j, j1, j2, jj, jj1, jj2, lun, naxes(2), naxis, &
            nbands, pcount, xdim, xdimo, ydim, ydimo
        integer :: status = 0
        real :: b, bscale, bzero, c, e, r, s, sca, temsc, tlim
        real, dimension(:,:), allocatable :: back, counts, expose, &
            mask, output, outsig, prot, swcx
        double precision cdelt(2), crpix(2), crval(2)

        temsc = realParameter("thresholdmasking")
        nbands = parameterCount("elowlist")

        allocate(IEL(nbands),IEH(nbands))
        do i=1,nbands 
            IEL(i) = intParameter("elowlist",i-1)
            IEH(i) = intParameter("ehighlist",i-1)
        end do

        ibin = intParameter("binning")
        ipart = booleanParameter("withpartcontrol")
        isoft = booleanParameter("withsoftcontrol")
        iswcx = booleanParameter("withswcxcontrol")
        imask = booleanParameter("withmaskcontrol")
        clobber = booleanParameter("clobber")

        maskfile = stringParameter("maskfile")
        if(trim(maskfile) .eq. 'none') imask = .false.

!  Set the scale factor for the maps, this is the ratio of
!  one square degree to the solid angle of the original map pixel

        SCA = (60.0*60.0)*(60.0*60.0)/(2.5*2.5)

        write(errstr,*) "Fractional limit for template: ", temsc
        call message(AppMsg,Always)
        write(errstr,*) "Number of Energy Bands to Bin: ", nbands
        call message(AppMsg,Always)
        DO IB=1,NBANDS
            write(errstr,*) "Band ", ib," Energy Limits", &
                        iel(ib), ieh(ib)
            call message(AppMsg,Always)
        ENDDO
        IELMIN = 10000
        IEHMAX = 0
        DO IB=1,NBANDS
            IF(IEL(IB) .LT. IELMIN) IELMIN = IEL(IB)
            IF(IEH(IB) .GT. IEHMAX) IEHMAX = IEH(IB)
        ENDDO
        write(errstr,*) "Binning factor: ", ibin
        call message(AppMsg,Always)

!  Find the size of the input arrays

        ib = 1
        IF(IEL(IB) .GE. 10000) THEN
            WRITE(INFILE,3100) IEL(IB), IEH(IB)
 3100       FORMAT('obj-im-',I5,'-',I5,'.fits')
        ELSEIF(IEL(IB) .GE. 1000) THEN
            IF(IEH(IB) .GE. 10000) THEN
                WRITE(INFILE,3110) IEL(IB), IEH(IB)
 3110           FORMAT('obj-im-',I4,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3120) IEL(IB), IEH(IB)
 3120           FORMAT('obj-im-',I4,'-',I4,'.fits')
            ENDIF
        ELSE
            IF(IEH(IB) .GE. 10000) THEN
                WRITE(INFILE,3130) IEL(IB), IEH(IB)
 3130           FORMAT('obj-im-',I3,'-',I5,'.fits')
            ELSEIF(IEH(IB) .GE. 1000) THEN
                WRITE(INFILE,3140) IEL(IB), IEH(IB)
 3140           FORMAT('obj-im-',I3,'-',I4,'.fits')
            ELSE
                WRITE(INFILE,3150) IEL(IB), IEH(IB)
 3150           FORMAT('obj-im-',I3,'-',I3,'.fits')
            ENDIF
        ENDIF
        write(errstr,*) "Reading input file for image size: ", &
                trim(infile)
        call message(AppMsg,Sparse)

!  Get an unused lun

        call ftgiou(lun,status)
        if(status .ne. 0) then
            write(errstr,*) "BIN_IMAGE_MERGE 01: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("noLUN","Cannot aquire LUN")
        endif

!  Open the FITS file for the first observation exposure

        write(errstr,*) "BIN_IMAGE_MERGE: Opening image:", trim(infile)
        call message(AppMsg,Sparse)
        call ftopen(lun,infile,0,blockn,status)
        if(status .ne. 0) then
            write(errstr,*) "BIN_IMAGE_MERGE 02: STATUS=",status, &
                    " on return from ftopen"
            call message(AppMsg,Always)
            call error ("noOpen","Cannot open image file")
        endif

!  Read the FITS file header information

        call ftghpr (lun,10,simple,bitpix,naxis,naxes, &
                pcount,gcount,extend,status)
        if(status .ne. 0) then
            write(errstr,*) "BIN_IMAGE_MERGE 03: STATUS=",status, &
                    " on return from ftghpr"
            call message(AppMsg,Always)
            call error ("noHeader","Cannot access header information")
        endif
        xdim = naxes(1)
        ydim = naxes(2)
        call ftclos (lun,status)
        if(status .ne. 0) then
            write(errstr,*) "BIN_IMAGE_MERGE 04: STATUS=",status, &
                    " on return from ftclos"
            call message(AppMsg,Always)
            call error ("noClose","Cannot close image file")
        endif
        call ftfiou (lun,status)
        if(status .ne. 0) then
            write(errstr,*) "BIN_IMAGE_MERGE 05: STATUS=",status, &
                    " on return from ftfiou"
            call message(AppMsg,Always)
            call error ("noFree","Cannot free LUN")
        endif

!  Allocate the input array sizes

        allocate(back(xdim,ydim))
        allocate(counts(xdim,ydim))
        allocate(expose(xdim,ydim))
        allocate(mask(xdim,ydim))
        allocate(prot(xdim,ydim))
        allocate(swcx(xdim,ydim))
        allocate(ini4(xdim,ydim))

!  Now take care of the output array dimensions

        xdimo = xdim/ibin
        ydimo = ydim/ibin
        iex = mod(xdim,ibin)/2
        iey = mod(ydim,ibin)/2
        print *, 'Input Dimensions: ', xdim, ydim
        print *, 'Output Dimensions: ', xdimo, ydimo
        allocate(output(xdimo,ydimo))
        allocate(outsig(xdimo,ydimo))

        do i=1,xdimo
            do ii=1,ydimo
                output(i,ii) = 0.0
                outsig(i,ii) = 0.0
            enddo
        enddo

!  Now start going through the data


!  Create the count array file names and read in the data

 1000   FORMAT(A80)

        DO IB=1,NBANDS
            IF(IEL(IB) .GE. 10000) THEN
                WRITE(INFILE,3100) IEL(IB), IEH(IB)
            ELSEIF(IEL(IB) .GE. 1000) THEN
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3110) IEL(IB), IEH(IB)
                ELSE
                    WRITE(INFILE,3120) IEL(IB), IEH(IB)
                ENDIF
            ELSE
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3130) IEL(IB), IEH(IB)
                ELSEIF(IEH(IB) .GE. 1000) THEN
                    WRITE(INFILE,3140) IEL(IB), IEH(IB)
                ELSE
                    WRITE(INFILE,3150) IEL(IB), IEH(IB)
                ENDIF
            ENDIF

            write(errstr,*) &
                    "BIN_IMAGE_MERGE: Retrieving count image: ", &
                    trim(infile)
            call message(AppMsg,Sparse)
            IF(IB .EQ. 1) THEN   
                CALL RETFIT (INFILE,COUNTS,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                if(status .ne. 0) then
                    write(errstr,*) "BIN_IMAGE_MERGE 06a: STATUS=", &
                            status, " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading count image")
                endif
            ELSE
                CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                if(status .ne. 0) then
                    write(errstr,*) "BIN_IMAGE_MERGE 06b: STATUS=", &
                            status, " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading count image")
                endif
                DO I=1,xdim
                    DO II=1,ydim
                        COUNTS(I,II) = COUNTS(I,II) + MASK(I,II)
                    ENDDO
                ENDDO
            ENDIF
        ENDDO

!  Enter the exposure array file names and read in the data

        DO IB=1,NBANDS
            IF(IEL(IB) .GE. 10000) THEN
                WRITE(INFILE,3200) IEL(IB), IEH(IB)
 3200           FORMAT('exp-im-',I5,'-',I5,'.fits')
            ELSEIF(IEL(IB) .GE. 1000) THEN
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3210) IEL(IB), IEH(IB)
 3210               FORMAT('exp-im-',I4,'-',I5,'.fits')
                ELSE
                    WRITE(INFILE,3220) IEL(IB), IEH(IB)
 3220               FORMAT('exp-im-',I4,'-',I4,'.fits')
                ENDIF
            ELSE
                IF(IEH(IB) .GE. 10000) THEN
                    WRITE(INFILE,3230) IEL(IB), IEH(IB)
 3230               FORMAT('exp-im-',I3,'-',I5,'.fits')
                ELSEIF(IEH(IB) .GE. 1000) THEN
                    WRITE(INFILE,3240) IEL(IB), IEH(IB)
 3240               FORMAT('exp-im-',I3,'-',I4,'.fits')
                ELSE
                    WRITE(INFILE,3250) IEL(IB), IEH(IB)
 3250               FORMAT('exp-im-',I3,'-',I3,'.fits')
                ENDIF
            ENDIF

            write(errstr,*) &
                    "BIN_IMAGE_MERGE: Retrieving exposure image: ", &
                    trim(infile)
            call message(AppMsg,Sparse)
            IF(IB .EQ. 1) THEN   
                CALL RETFIT (INFILE,EXPOSE,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                if(status .ne. 0) then
                    write(errstr,*) "BIN_IMAGE_MERGE 07a: STATUS=", &
                            status, " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading exposure image")
                endif
            ELSE
                CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                call message(AppMsg,Sparse)
                if(status .ne. 0) then
                    write(errstr,*) "BIN_IMAGE_MERGE 07b: STATUS=", &
                            status, " on return from retfit"
                    call message(AppMsg,Always)
                    call error ("noRet","Error reading exposure image")
                endif
                DO I=1,xdim
                    DO II=1,ydim
                        EXPOSE(I,II) = EXPOSE(I,II) + MASK(I,II)
                    ENDDO
                ENDDO
            ENDIF
        ENDDO
        IF(NBANDS .GT. 1) THEN
            DO I=1,xdim
                DO II=1,ydim
                    EXPOSE(I,II) = EXPOSE(I,II)/NBANDS
                ENDDO
            ENDDO
        ENDIF

!  Enter the background count array file names and read in the data

        IF(IPART) THEN
            DO IB=1,NBANDS
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3300) IEL(IB), IEH(IB)
 3300               FORMAT('back-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3310) IEL(IB), IEH(IB)
 3310                   FORMAT('back-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3320) IEL(IB), IEH(IB)
 3320                   FORMAT('back-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSE
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3330) IEL(IB), IEH(IB)
 3330                   FORMAT('back-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3340) IEL(IB), IEH(IB)
 3340                   FORMAT('back-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3350) IEL(IB), IEH(IB)
 3350                   FORMAT('back-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF

                write(errstr,*) &
                        "BIN_IMAGE_MERGE: Retrieving QPB image: ", &
                        trim(infile)
                call message(AppMsg,Sparse)
                IF(IB .EQ. 1) THEN   
                    CALL RETFIT (INFILE,BACK,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    if(status .ne. 0) then
                        write(errstr,*) "BIN_IMAGE_MERGE 08a: STATUS=", &
                                status, " on return from retfit"
                        call message(AppMsg,Always)
                        call error ("noRet","Error reading QPB image")
                    endif
                ELSE
                    CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    if(status .ne. 0) then
                        write(errstr,*) "BIN_IMAGE_MERGE 08b: STATUS=", &
                                status, " on return from retfit"
                        call message(AppMsg,Always)
                        call error ("noRet","Error reading QPB image")
                    endif
                    DO I=1,xdim
                        DO II=1,ydim
                            BACK(I,II) = BACK(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
            ENDDO
        ELSE
            DO I=1,xdim
                DO II=1,ydim
                    BACK(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF
 
!  Enter the proton count array file names and read in the data

        IF(ISOFT) THEN
            DO IB=1,NBANDS
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3400) IEL(IB), IEH(IB)
 3400               FORMAT('prot-im-',I5,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3410) IEL(IB), IEH(IB)
 3410                   FORMAT('prot-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3420) IEL(IB), IEH(IB)
 3420                   FORMAT('prot-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH(IB) .LT. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3430) IEL(IB), IEH(IB)
 3430                   FORMAT('prot-im-',I3,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3440) IEL(IB), IEH(IB)
 3440                   FORMAT('prot-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3450) IEL(IB), IEH(IB)
 3450                   FORMAT('prot-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF

                write(errstr,*) &
                        "BIN_IMAGE_MERGE: Retrieving SP image: ", &
                        trim(infile)
                call message(AppMsg,Sparse)
                IF(IB .EQ. 1) THEN   
                    CALL RETFIT (INFILE,PROT,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    if(status .ne. 0) then
                        write(errstr,*) "BIN_IMAGE_MERGE 09a: STATUS=", &
                                status, " on return from retfit"
                        call message(AppMsg,Always)
                        call error ("noRet","Error reading SP image")
                    endif
                ELSE
                    CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    if(status .ne. 0) then
                        write(errstr,*) "BIN_IMAGE_MERGE 09b: STATUS=", &
                                status, " on return from retfit"
                        call message(AppMsg,Always)
                        call error ("noRet","Error reading SP image")
                    endif
                    DO I=1,xdim
                        DO II=1,ydim
                            PROT(I,II) = PROT(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
            ENDDO
        ELSE
            DO I=1,xdim
                DO II=1,ydim
                    PROT(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF

!  Enter the solar wind charge exchange count array file names and 
!  read in the data

        IF(ISWCX) THEN
            DO IB=1,NBANDS
                IF(IEL(IB) .GE. 10000) THEN
                    WRITE(INFILE,3500) IEL(IB), IEH(IB)
 3500               FORMAT('swcx-im-',I4,'-',I5,'.fits')
                ELSEIF(IEL(IB) .GE. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3510) IEL(IB), IEH(IB)
 3510                   FORMAT('swcx-im-',I4,'-',I5,'.fits')
                    ELSE
                        WRITE(INFILE,3520) IEL(IB), IEH(IB)
 3520                   FORMAT('swcx-im-',I4,'-',I4,'.fits')
                    ENDIF
                ELSEIF(IEH(IB) .LT. 1000) THEN
                    IF(IEH(IB) .GE. 10000) THEN
                        WRITE(INFILE,3530) IEL(IB), IEH(IB)
 3530                   FORMAT('swcx-im-',I4,'-',I5,'.fits')
                    ELSEIF(IEH(IB) .GE. 1000) THEN
                        WRITE(INFILE,3540) IEL(IB), IEH(IB)
 3540                   FORMAT('swcx-im-',I3,'-',I4,'.fits')
                    ELSE
                        WRITE(INFILE,3550) IEL(IB), IEH(IB)
 3550                   FORMAT('swcx-im-',I3,'-',I3,'.fits')
                    ENDIF
                ENDIF

                write(errstr,*) &
                        "BIN_IMAGE_MERGE: Retrieving SWCX image: ", &
                        trim(infile)
                call message(AppMsg,Sparse)
                IF(IB .EQ. 1) THEN   
                    CALL RETFIT (INFILE,SWCX,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    if(status .ne. 0) then
                        write(errstr,*) "BIN_IMAGE_MERGE 10a: STATUS=", &
                                status, " on return from retfit"
                        call message(AppMsg,Always)
                        call error ("noRet","Error reading SWCX image")
                    endif
                ELSE
                    CALL RETFIT (INFILE,MASK,INI4,1,xdim, &
                            ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
                    if(status .ne. 0) then
                        write(errstr,*) "BIN_IMAGE_MERGE 10b: STATUS=", &
                                status, " on return from retfit"
                        call message(AppMsg,Always)
                        call error ("noRet","Error reading SWCX image")
                    endif
                    DO I=1,xdim
                        DO II=1,ydim
                            SWCX(I,II) = SWCX(I,II) + MASK(I,II)
                        ENDDO
                    ENDDO
                ENDIF
            ENDDO
        ELSE
            DO I=1,xdim
                DO II=1,ydim
                    SWCX(I,II) = 0.0
                ENDDO
            ENDDO
        ENDIF

!  Add the background components

        DO I=1,xdim
            DO II=1,ydim
                BACK(I,II) =  BACK(I,II) + PROT(I,II) + SWCX(I,II)
            ENDDO
        ENDDO

!  Prepare the template

        if(imask) then

!  Read in the mask

            write(errstr,*) &
                    "BIN_IMAGE_MERGE: Retrieving MASK image: ", &
                    trim(infile)
            call message(AppMsg,Sparse)
            CALL RETFIT (MASKFILE,MASK,INI4,1,xdim, &
                        ydim,CTYPE,CRVAL,CDELT,CRPIX,STATUS)
            if(status .ne. 0) then
                write(errstr,*) "BIN_IMAGE_MERGE 11: STATUS=", &
                        status, " on return from retfit"
                call message(AppMsg,Always)
                call error ("noRet","Error reading MASK image")
            endif

!  Make sure that the mask is zero where there is no exposure

            DO I=1,xdim
                DO II=1,ydim
                    IF(EXPOSE(I,II) .LE. 0.0) MASK(I,II) = 0.0
                ENDDO
            ENDDO

!  Use the exposure map as the template

        ELSE
            DO I=1,xdim
                DO II=1,ydim
                    MASK(I,II) = EXPOSE(I,II)
                ENDDO
            ENDDO
        ENDIF

!  Set the template limit

        J = 0
        TLIM = 0.
        DO I=1,xdim
            DO II=1,ydim
                IF(MASK(I,II) .GT. 0) THEN
                    TLIM = TLIM + MASK(I,II)
                    J = J + 1
                ENDIF
            ENDDO
        ENDDO
        TLIM = TEMSC*TLIM/J

!  Bin the data

        PRINT *, 'START THE BINNING'
        DO I=1,xdimo
            j2 = i*ibin + iex
            j1 = j2 - ibin + 1
            if(j2 .gt. xdim) j2 = xdim
            DO II=1,ydimo
                jj2 = ii*ibin + iey
                jj1 = jj2 - ibin + 1
                if(jj2 .gt. ydim) jj2 = ydim
                B = 0.
                C = 0.
                E = 0.
                DO J=J1,J2
                    DO JJ=JJ1,JJ2
                        IF(MASK(J,JJ) .GT. TLIM) THEN
                            B = B + BACK(J,JJ)
                            C = C + COUNTS(J,JJ)
                            E = E + EXPOSE(J,JJ)
                        ENDIF
                    ENDDO
                ENDDO

                IF((E .GT. 0.0) .AND. (C .GT. 0.0)) THEN
                    R = SCA*(C - B)/E
                    S = SCA*SQRT(C)/E
                ELSE
                    R = 0.0
                    S = 0.0
                ENDIF
                OUTPUT(I,II) = R
                OUTSIG(I,II) = S
            ENDDO
        ENDDO

!  Create the output count rate file name

        IF(IELMIN .GE. 10000) THEN
            WRITE(INFILE,3600) IELMIN, IEHMAX
 3600       FORMAT('bin-',I4,'-',I4,'.fits')
        ELSEIF(IELMIN .GE. 1000) THEN
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3610) IELMIN, IEHMAX
 3610           FORMAT('bin-',I4,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3620) IELMIN, IEHMAX
 3620           FORMAT('bin-',I4,'-',I4,'.fits')
            ENDIF
        ELSE
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3630) IELMIN, IEHMAX
 3630           FORMAT('bin-',I3,'-',I5,'.fits')
            ELSEIF(IEHMAX .GE. 1000) THEN
                WRITE(INFILE,3640) IELMIN, IEHMAX
 3640           FORMAT('bin-',I3,'-',I4,'.fits')
            ELSE
                WRITE(INFILE,3650) IELMIN, IEHMAX
 3650           FORMAT('bin-',I3,'-',I3,'.fits')
            ENDIF
        ENDIF

!  Now write out the binned count rate image

        COMMENT = 'Binned count-rate image for XMM'
        UNITS = 'c/s/deg2'
        CDELT(1) = ibin*CDELT(1)
        CDELT(2) = ibin*CDELT(2)
        CRPIX(1) = xdimo/2 + 0.5
        CRPIX(2) = ydimo/2 + 0.5
        BZERO = 0.0
        BSCALE = 1.0
        IFLAG = 1
        write(errstr,*) "BIN_IMAGE_MERGE: Writing rate image: ", &
                trim(infile)
        call message(AppMsg,Sparse)
        CALL PUTFIT(INFILE,OUTPUT,INI4,IFLAG,UNITS, &
                COMMENT,xdimo,ydimo,BZERO,BSCALE,CTYPE,CRVAL, &
                CDELT,CRPIX,CLOBBER,STATUS)
        if(status .ne. 0) then
            write(errstr,*) "BIN_IMAGE_MERGE 12: STATUS=", &
                    status, " on return from putfit"
            call message(AppMsg,Always)
            call error ("noPut","Error writing rate image")
        endif

!  Create the output count rate uncertainty file name

        IF(IELMIN .GE. 10000) THEN
            WRITE(INFILE,3605) IELMIN, IEHMAX
 3605       FORMAT('sig-',I4,'-',I4,'.fits')
        ELSEIF(IELMIN .GE. 1000) THEN
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3615) IELMIN, IEHMAX
 3615           FORMAT('sig-',I4,'-',I5,'.fits')
            ELSE
                WRITE(INFILE,3625) IELMIN, IEHMAX
 3625           FORMAT('sig-',I4,'-',I4,'.fits')
            ENDIF
        ELSE
            IF(IEHMAX .GE. 10000) THEN
                WRITE(INFILE,3635) IELMIN, IEHMAX
 3635           FORMAT('sig-',I3,'-',I5,'.fits')
            ELSEIF(IEHMAX .GE. 1000) THEN
                WRITE(INFILE,3645) IELMIN, IEHMAX
 3645           FORMAT('sig-',I3,'-',I4,'.fits')
            ELSE
                WRITE(INFILE,3655) IELMIN, IEHMAX
 3655           FORMAT('sig-',I3,'-',I3,'.fits')
            ENDIF
        ENDIF

!  Now write out the binned count rate uncertainty image

        COMMENT = 'Binned count-rate sigma map for XMM'

        write(errstr,*) "BIN_IMAGE_MERGE: Writing sigma image: ", &
                trim(infile)
        call message(AppMsg,Sparse)
        CALL PUTFIT(INFILE,OUTSIG,INI4,IFLAG,UNITS, &
                COMMENT,xdimo,ydimo,BZERO,BSCALE,CTYPE,CRVAL, &
                CDELT,CRPIX,CLOBBER,STATUS)
        if(status .ne. 0) then
            write(errstr,*) "BIN_IMAGE_MERGE 13: STATUS=", &
                    status, " on return from putfit"
            call message(AppMsg,Always)
            call error ("noPut","Error writing sigma image")
        endif

        return

    deallocate(IEL,IEH)
  end subroutine bin_image_merge

end module bin_image_merge_mod
