!
! Module: conv_reg_mod
!
! Authors:  Steve Snowden (steven.l.snowden@nasa.gov) - NASA/GSFC
!  modified by: Brendan Perry (brendan.h.perry@nasa.gov) NASA/GSFC
!               (for sas, hera compliance, xmm-sas ccf integration)
!          
! Description: Converts region descriptors from sky to detector coords
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module conv_reg_mod

  use dal
  use param 
  use errorHandling

  implicit none 

  private 

  ! task parameter names
  
  character(len=*), parameter :: &
       modepar    = "mode", &      ! mandatory for all 
       imagepar   = "imagefile", & ! mandatory for all
       infilepar  = "inputfile", & ! mandatory for modes 2,3
       outfilepar = "outputfile", & ! mandatory for modes 2,3
       shapepar   = "shape", &      ! mandatory for mode 1
       rapar      = "ra", &         ! mandatory for mode 1
       decpar     = "dec", &        ! mandatory for mode 1
       majorpar   = "semimajor", &  ! mandatory for mode 1 ELLIPSE
       minorpar   = "semiminor", &  ! mandatory for mode 1 ELLIPSE
       rotangpar  = "rotangle",&    ! mandatory for mode 1 ELLIPSE
       radiuspar  = "radius"        ! mandatory for mode 1 CIRCLE
  public conv_reg

contains 

  !---------------------------------------------------------------------
  subroutine conv_reg
    !---------------------------------------------------------------------

    ! declare input parameters first

    character(len=1) :: mode
    character(len=5) :: test(1000)
    character(len=8) :: tform(30), ttype(30), tunit(30), instrume
    character(len=12) :: decw, raw
    character(len=16), allocatable, dimension(:) :: shape
    character(len=20) :: extname
    character(len=60) :: com
    character(len=255) :: imset, inputfile, outputfile
    character(len=1024) :: command
    integer :: blockn, blocko, hdutypei, i, lundet, lunim, lunin, lunout, &
         numregions, tfields, varidat
    integer :: status = 0
    integer, allocatable, dimension(:) :: sh
    logical :: anyf, exists, mos1, mos2, pn
    real :: imrotang
    real, allocatable, dimension(:) :: dec, detx, &
         dety, ra, rada, radb, radius, rotang

    status = 0
    mode = stringParameter(modepar)
    write(errstr,*) "Mode is : ",mode
    call message(AppMsg,Noisy)    
    !
    !  mode = 3 - Input a single region
    !
    if (mode .eq. "3") then ! command line input single region
       numregions = 1
       allocate(shape(numregions))
       shape(1) = '                '
       allocate(sh(numregions))
       allocate(detx(numregions))
       allocate(dety(numregions))
       allocate(ra(numregions))
       allocate(dec(numregions))
       ra(1) = realParameter(rapar)
       dec(1) = realParameter(decpar)
       shape(1) = stringParameter(shapepar)
       write(errstr,*) "Shape: ",shape(1),"  RA: ",ra(1), &
            "  Dec: ",dec(1)
       call message(AppMsg,Sparse)
       if((shape(1) .eq. 'circle          ') .or. &
            (shape(1) .eq. 'CIRCLE          ') .or. &
            (shape(1) .eq. '!circle         ') .or. &
            (shape(1) .eq. '!CIRCLE         ')) then
          allocate(radius(numregions))
          radius(1) = realParameter(radiuspar)
          sh(1) = 1
       elseif ((shape(1) .eq. 'ellipse         ') .or. & 
            (shape(1) .eq. 'ELLIPSE         ') .or. & 
            (shape(1) .eq. '!ellipse        ') .or. & 
            (shape(1) .eq. '!ELLIPSE        ')) then
          allocate(rada(numregions))
          allocate(radb(numregions))
          allocate(rotang(numregions))
          rada(1) = realParameter(majorpar) ! semimajor
          radb(1) = realParameter(minorpar) ! semiminor
          rotang(1) = realParameter(rotangpar) ! rotation angle
          sh(1) = 2
       else
          write(errstr,*)&
               "Input region shape: ",trim(shape(1))," unsupported."
          call error ("BadShape",trim(errstr))
       endif
       !
       !  mode = 2 - Input an ascii file with regions
       !
    elseif (mode .eq. "2") then ! ascii file list of region(s)
       inputfile = stringParameter(infilepar)
       write(errstr,*) "Inputfile is :  ",trim(inputfile)
       call message(AppMsg,Noisy) 
       outputfile = stringParameter(outfilepar) 
       write(errstr,*) "Outputfile is : ",trim(outputfile)
       call message(AppMsg,Noisy) 
       !
       call ftgiou(lunin,status)
       status = 0
       open(unit=lunin,status='old',form='formatted', &
            file=trim(inputfile),iostat=status)
       if(status .ne. 0) then
          write(errstr,*) "Cannot open input file: ",trim(inputfile)
          call error ("noOpenInfile",trim(errstr))
       else
          write(errstr,*) "Success opening ASCII inputfile."
          call message(AppMsg,Noisy) 
       endif
       numregions = 1
       status = 0
       do 
          read(lunin,1000,iostat=status) test(numregions)
1000      format(a5)
          if(status .ne. 0) exit
          numregions = numregions + 1
       end do
       numregions = numregions - 1

       write(errstr,*) "Number of Regions: ", numregions
       call message(AppMsg,Sparse)

       allocate(shape(numregions))
       allocate(sh(numregions))
       allocate(ra(numregions))
       allocate(dec(numregions))
       allocate(detx(numregions))
       allocate(dety(numregions))
       allocate(radius(numregions))
       allocate(rada(numregions))
       allocate(radb(numregions))
       allocate(rotang(numregions))
       rewind(lunin) ! re-read file and loop thru shapes
       do i=1,numregions
          if((test(i) .eq. 'circl') .or. &
               (test(i) .eq. 'CIRCL') .or. &
               (test(i) .eq. '!circ') .or. &
               (test(i) .eq. '!CIRC')) then
             read(lunin,*) shape(i), ra(i), &
                  dec(i), radius(i)
             write(errstr,*) i, shape(i), ra(i), &
                  dec(i), radius(i)
             call message(AppMsg,Sparse)
             sh(i) = 1
          elseif ((test(i) .eq. 'ellip') .or. &
               (test(i) .eq. 'ELLIP') .or. &
               (test(i) .eq. '!elli') .or. &
               (test(i) .eq. '!ELLI')) then
             read(lunin,*) shape(i), ra(i), &
                  dec(i), rada(i), radb(i), rotang(i)
             sh(i) = 2
             write(errstr,*) i, shape(i), ra(i), &
                  dec(i), rada(i), radb(i), rotang(i)
             call message(AppMsg,Sparse)
          else
             write(errstr,*) "Shape: ",trim(test(i)),&
                  " from ASCII file is unsupported."
             call error ("BadShape",trim(errstr))
          endif
       enddo
       write(errstr,*) "Closing mode 2 ASCII inputfile: ",trim(inputfile)
       call message(AppMsg,Noisy)
       close(lunin)
       !
       ! open output ASCII file for writing
       !
       exists = .FALSE.
       inquire(FILE=trim(outputfile),EXIST=exists)
       if(exists) then 
          if(clobber()) then
             write(errstr,*) &
                  "Clobber set, replacing existing file and continuing."
             call message(AppMsg,Always)
             call ftgiou(lunout,status)
             open(unit=lunout,status='replace', &
                  form='formatted',file=trim(outputfile))
          else 
             write(errstr,*) "NoClobber set but outputfile exists. Exiting."
             call error("noClobber",trim(errstr))
          endif
       else
          write(errstr,*) "Output file ",trim(outputfile),& 
               " does not exist, opening new one."
          call message(AppMsg,Noisy)
          status = 0
          call ftgiou(lunout,status)
          if (status .ne. 0) then 
              write(errstr,*) "Error obtaining out LUN"
             call error("noLUN",trim(errstr))
          end if
          open(unit=lunout,status='new',form='formatted', &
                  file=trim(outputfile),IOSTAT=status)
          if (status .ne. 0) then 
              write(errstr,*) &
                   "Error opening output file for write, status = ",status
             call error("noLUN",trim(errstr))
          end if
       end if
!
    elseif(mode .eq. "1") then ! FITS region file
       inputfile = stringParameter(infilepar)
       outputfile = stringParameter(outfilepar)
       write(errstr,*) &
            "Input file : ",trim(inputfile)
       call message(AppMsg,Noisy) 
       write(errstr,*) &
            "Output file : ",trim(outputfile)
       call message(AppMsg,Noisy) 
       !
       !  Open the input source list
       !
       status = 0
       call ftgiou(lunin,status)
       call ftopen (lunin,trim(inputfile),0,blockn,status)
       if(status .ne. 0) then !bhp: fix this with set/file param!
          write(errstr,*) &
               "Cannot open input region file : ",trim(inputfile),&
               " status = ",status
          call error ("noOpenInRegionfile",trim(errstr))
       endif
       !
       !  Initialize the output source list
       !
       exists = .FALSE.
       inquire(FILE=trim(outputfile), EXIST=exists) 
       if(exists) then
          if(clobber()) then
             write(errstr,*) &
                  "Clobber set, replacing existing file and continuing."
             call message(AppMsg,Always)
             call ftgiou(lunout,status)
             status = 0
             open(lunout,file=trim(outputfile),status='unknown',iostat=status)
             if (status .eq. 0) then
                close(lunout, status='delete')
             else 
                write(errstr,*) &
                     "Cannot delete existing output file, status = ",status
                call error ("badDelete",trim(errstr))
             endif
             ! deleted outfile, open new one
             blocko = 1
             call ftinit (lunout,trim(outputfile),blocko,status)
             if(status .ne. 0) then
                write(errstr,*) "1. Cannot open output FITS region file : ",&
                     trim(outputfile)," status = ",status
                call error ("noOpenOutRegionfile",trim(errstr))
             endif
          else
             write(errstr,*) &
                  "NoClobber set but outputfile exists. Exiting."
             call error("noClobber",trim(errstr))
          endif
       else
          write(errstr,*) "Output file does not exist, opening new."
          call message(AppMsg,Noisy)
          call ftgiou(lunout,status)
          status = 0
          blocko = 1
          call ftinit (lunout,trim(outputfile),blocko,status)
          if(status .ne.  0) then
             write(errstr,*) "2. Cannot open output FITS region file : ",&
                  trim(outputfile)," status = ",status
             call error ("noOpenOutRegionfile",trim(errstr))
          endif
          blocko = 1
          call ftinit (lunout,trim(outputfile),blocko,status)
       endif
       
       !  Copy the primary header from the input to the output

       call ftcopy (lunin,lunout,0,status)
       if(status .ne.  0) then
          write(errstr,*) &
               "Error copying Primary header from inputfile to outputfile."
          call error ("noCopyHeader",trim(errstr))
       endif

       !  Move to the region list extension of the input file

       call ftmrhd (lunin,1,hdutypei,status)

       !  Create a new empty extension in the output file and move to it

       call ftcrhd(lunout,status)
       if(status .ne.  0) then
          write(errstr,*) &
               "Error creating extension in outputfile, status = ",status
          call error ("noCreateExt",trim(errstr))
       endif

       call ftcopy (lunin,lunout,0,status)
       if(status .ne.  0) then
          write(errstr,*) &
               "Error copying extension keywords from inputfile to outputfile."
          call error ("noCopyHeader",trim(errstr))
       endif

       !  Read the FITS file header information

       call ftghbn (lunin,30,numregions,tfields,ttype,tform, &
            tunit,extname,varidat,status)
       if(status .ne.  0) then
          write(errstr,*) "Cannot read binary table STATUS = ",status
          call error ("noReadHeader",trim(errstr))
       endif

       if(extname .ne. 'REGION') then
          write(errstr,*) "EXTNAME = ", trim(extname)
          call message(AppMsg,Always)
          write(errstr,*) &
               "Input region file does not contain REGION extension."
          call error ("NoRegionExt",trim(errstr))
       endif

       !  Delete rows then add back blank rows

       call ftdrow (lunout,1,numregions,status)
       if(status .ne.  0) then
          write(errstr,*) &
               "Unable to delete rows from FITS table, status = ",status
          call error ("noDeleteRows",trim(errstr))
       endif

       call ftirow (lunout,0,numregions,status)
       if(status .ne.  0) then
          write(errstr,*) &
               "Unable to write new rows to FITS table, status = ",status
          call error ("noWriteRows",trim(errstr))
       endif

       allocate(shape(numregions))
       allocate(sh(numregions))
       allocate(ra(numregions))
       allocate(dec(numregions))
       allocate(detx(numregions))
       allocate(dety(numregions))
       allocate(radius(numregions))
       allocate(rada(numregions))
       allocate(radb(numregions))
       allocate(rotang(numregions))
       do i=1,numregions
          shape(i) = '          '
       enddo

       !  Read in the data

       do i=1,numregions
          call ftgcvs(lunin,1,i,1,1,0,shape(i),anyf,status)
          call ftgcve(lunin,2,i,1,1,0,ra(i),anyf,status)
          call ftgcve(lunin,3,i,1,1,0,dec(i),anyf,status)
          if(status .ne. 0) then
             write(errstr,*) &
                  "Unable to read FITS column data, status = ",status
             call error ("noReadCol",trim(errstr))
          endif
          if ((shape(i) .eq. 'circle          ') .or. &
               (shape(i) .eq. 'CIRCLE          ') .or. &
               (shape(i) .eq. '!circle         ') .or. &
               (shape(i) .eq. '!CIRCLE         ')) then
             call ftgcve(lunin,4,i,1,1,0,radius(i),anyf,status)
             if(status .ne. 0) then
                write(errstr,*) &
                     "Unable to read FITS column data, status = ",status
                call error ("noReadCol",trim(errstr))
             else
                sh(i) = 1
                write(errstr,*) "Shape: ",trim(shape(i)), &
                     "  RA: ",ra(i),"  Dec: ",dec(i),&
                     "  Radius: ",radius(i)
                call message(AppMsg,Noisy)
             endif
          elseif ((shape(i) .eq. 'ellipse         ') .or. &
               (shape(i) .eq. 'ELLIPSE         ') .or. &
               (shape(i) .eq. '!ellipse        ') .or. &
               (shape(i) .eq. '!ELLIPSE        ')) then 
             call ftgcve(lunin,4,i,1,1,0,rada(i),anyf,status)
             call ftgcve(lunin,4,i,2,1,0,radb(i),anyf,status)
             call ftgcve(lunin,5,i,1,1,0,rotang(i),anyf,status)
             if(status .ne. 0) then
                write(errstr,*) &
                     "Unable to read FITS column data, status = ",status
                call error ("noReadCol",trim(errstr))
             else
                sh(i) = 2
                write(errstr,*) "Shape: ",trim(shape(i)), &
                     "  RA: ",ra(i),"  Dec: ",dec(i),&
                     "  Radii: ",rada(i),radb(i), &
                     "  Rotang: ",rotang(i)
                call message(AppMsg,Noisy)
             endif
          else
             write(errstr,*) "Shape: ",trim(test(i)),&
                  " from FITS file is unsupported."
             call error ("BadShape",trim(errstr))
          endif
       enddo
    endif

    imset = stringParameter(imagepar)

    !  Get an unused LUN

    call ftgiou(lunim,status)

    !  Open the FITS file for the image and get the rotation angle

    call ftopen (lunim,trim(imset),0,blockn,status)
    if(status .ne. 0) then
       write(errstr,*) "Cannot open Image file: ",trim(imset)
       call error ("noOpen",trim(errstr))
    endif

    call ftgkye (lunim,'PA_PNT',imrotang,com,status)
    if(status .ne. 0) then
       write(errstr,*) &
            "Cannot read PA_PNT from Image file STATUS=",status
       call error ("noPA_PNT",trim(errstr))
    endif
    call ftgkys (lunim,'INSTRUME',instrume,com,status)
    if(status .ne. 0) then
       write(errstr,*) &
            "Cannot read INSTRUME from Image file STATUS=",status
       call error ("noINSTRUME",trim(errstr))
    endif
    mos1 = .FALSE.
    pn = .FALSE.
    select CASE (trim(instrume))
    CASE ('EMOS1')
       mos1 = .TRUE.
    CASE ('EMOS2')
       mos2 = .TRUE.
    CASE ('EPN','PN')
       pn = .true.
    CASE default
       write(errstr,*)&
            "INSTRUME must be EMOS1|EMOS2|EPN. It is: ",trim(instrume)
       call error ("BadINSTRUME",trim(errstr))
    end select
    call ftclos (lunim, status)
    if(status .ne. 0) then
       write(errstr,*) "Error closing Image file STATUS = ",status
       call error ("noClose",trim(errstr))
    endif
    call ftfiou (lunim,status)
    if(status .ne. 0) then
       write(errstr,*) "Error freeing Image file LUN, STATUS = ",status
       call error ("noFree",trim(errstr))
    endif

    do i=1,numregions
       write(raw,3020) ra(i)
3020   format(f12.6)
       write(decw,3020) dec(i)
       command = "esky2det datastyle=user" // & 
            " ra=" // trim(adjustl(raw)) // &
            " dec=" // trim(adjustl(decw)) // &
            " instrument=" // trim(adjustl(instrume)) // &
            " checkfov=no " // & 
            " outunit=det withheader=no calinfostyle=set " // &
            " calinfoset=" // trim(adjustl(imset)) // &
            " verbosity=0 > detpos.txt"
       write(errstr,*) "command = ", trim(command)
       call message(AppMsg,Always)
       call system(trim(command))
       !
       ! read esky2det output in detpos.txt
       !
       status = 0
       call ftgiou(lundet,status)
       if((status .ne. 0)) then
          write(errstr,*) &
               "Error obtaining LUN for detpos.txt file, status = ",status
          call error ("baddetposLUN",trim(errstr))
       endif
       open(lundet,file='detpos.txt',status='old',&
            form='formatted',IOSTAT=status)
       if((status .ne. 0)) then
          write(errstr,*) &
               "Error opening detpos.txt file, status = ",status
          call error ("baddetposOPEN",trim(errstr))
       endif
       read(lundet,*,IOSTAT=status) detx(i), dety(i)
       if((status .lt. -1) .or. (status .gt. 0)) then
          write(errstr,*) &
               "Error reading detpos.txt from esky2det, status = ",status
          call error ("baddetposREAD",trim(errstr))
       endif

       close(lundet)
       !
       status = 0
       if(sh(i) .eq. 1) then
          radius(i) = 20.0*60.0*radius(i)
          print 2000, i, shape(i), detx(i), dety(i), radius(i)
2000      format(1x,i3,1x,a10,3f12.3)
          !
          if( mode .eq. "2" ) then
             write(lunout,2001) shape(i), detx(i), dety(i), &
                  radius(i)
2001         format(1x,a10,3f12.3)
          elseif(mode .eq. "1") then
             call ftpcls(lunout,1,i,1,1,shape(i),status)
             call ftpcle(lunout,2,i,1,1,detx(i),status)
             call ftpcle(lunout,3,i,1,1,dety(i),status)
             call ftpcle(lunout,4,i,1,1,radius(i),status)
             call ftpcle(lunout,5,i,1,1,0.0,status)
             call ftpclj(lunout,6,i,1,1,1,status)
          endif
          if(status .ne. 0) then
             write(errstr,*) &
                  "Error writing data to a FITS column, STATUS = ",status
             call error ("noWriteCol",trim(errstr))
          endif
       elseif(sh(i) .eq. 2) then
          rada(i) = 20.0*60.0*rada(i)
          radb(i) = 20.0*60.0*radb(i)
          if( mos1 ) then
             rotang(i) = imrotang - rotang(i)
          elseif( mos2 ) then
             rotang(i) = imrotang - rotang(i) + 90.0
          elseif( pn ) then
             rotang(i) = imrotang - rotang(i) + 90.0
          else ! should never happen
             call error ("notEPIC","Selected detector not M1/M2/PN")
          endif
          if(rotang(i) .lt. 0.0) rotang(i) = rotang(i) + 360.0
          if(rotang(i) .ge. 360.0) rotang(i) = rotang(i) - 360.0
          print 2010, i, shape(i), detx(i), dety(i), rada(i), &
               radb(i), rotang(i)
2010      format(1x,i3,1x,a10,5f12.3)
          if(mode .eq. "2") then
             write(lunout,2011) shape(i), detx(i), dety(i), &
                  rada(i), radb(i), rotang(i)
2011         format(1x,a10,5f12.3)
          elseif(mode .eq. "1") then
             call ftpcls(lunout,1,i,1,1,shape(i),status)
             call ftpcle(lunout,2,i,1,1,detx(i),status)
             call ftpcle(lunout,3,i,1,1,dety(i),status)
             call ftpcle(lunout,4,i,1,1,rada(i),status)
             call ftpcle(lunout,4,i,2,1,radb(i),status)
             call ftpcle(lunout,5,i,1,1,rotang(i),status)
             call ftpclj(lunout,6,i,1,1,1,status)
          endif
          if(status .ne. 0) then
             write(errstr,*) &
                  "Error writing data to a FITS column, STATUS = ",status
             call error ("noWriteCol",trim(errstr))
          endif
       endif
    enddo
!
    if(mode .eq. "2") then ! output ascii file
       write(errstr,*) "Closing mode 2 ASCII outputfile"
       call message(AppMsg,Noisy)
       close(lunout)
    elseif(mode .eq. "1") then
       call ftmkys (lunout,'TTYPE2','DETX', &
            'Event X position DET coordinates',status)
       call ftmkys (lunout,'TTYPE3','DETY', &
            'Event Y position DET coordinates',status)
       call ftmkys (lunout,'MFORM1','DETX,DETY', &
            '[mm]',status)
       if(status .ne. 0) then
          write(errstr,*) &
               "Error modifying coordinate keywords to outputfile, ",&
               "status = ",status
          call error ("noKeyModify",trim(errstr))
       endif
       call ftclos (lunout,status)
       if(status .ne. 0) then
          write(errstr,*) "Error closing outputfile, status = ",status
          call message(AppMsg,Always)
          call error ("noClose","Cannot close file")
       endif
    endif

    return

  end subroutine conv_reg

end module conv_reg_mod
