!+
! Module: fill_list_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
! SLS 11 March 2011
!
! Comments:
!
! $Id: merge_comp_xmm_mod.f90,v 1.4 2009/05/26 12:53:39 aibarra Exp $
!-
!  Version 1.0  08 March 2011
!
!  Fills in entries in the emllist.fits source list
!
!
!  INPUT:
!    Input file with a list of directories
!
!    Example: fill_list srclistin=emllist.fits srclistout=emllistout.fits
!
!  OUPUT:
!    Merged ascii source list

module fill_list_mod

  use dal
  use param
  use errorHandling

  implicit none

  private

  public fill_list

contains

  !---------------------------------------------------------------------
  subroutine fill_list
  !---------------------------------------------------------------------

        real :: flux, fluxerr, ft, nft, sft
        integer :: blockl, hdutype, i, ib, iband, ibt, icol, &
            idec, idist, iflux, ifluxerr, iml, ira, iradecerr, &
            irate, irateerr, is, iscts, isel, isour, ist, lunin, &
            lunout, nchan,  ncols, tfields, varidat
        integer :: err = 0 
        integer :: status = 0
        character(len=8) ::  tform(300), ttype(300), tunit(300)
        character(len=10) :: colnam(300)
        character(len=20) :: extname
        character(len=132) :: srclistin, srclistout
        logical anyf

        srclistin = stringParameter("srclistin")
        srclistout = stringParameter("srclistout")

!  Get an unused lun

        call ftgiou(lunin,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'MERGE_SOURCE_LIST POSITION 01'
        endif

!  Get a second unused lun

        call ftgiou(lunout,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'MERGE_SOURCE_LIST POSITION 01a'
        endif

!  Open the input source list

        call ftopen(lunin,srclistin,0,blockl,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTOPEN =', status
            print *, 'MERGE_SOURCE_LIST POSITION 02'
            print *, srclistin
        endif

!  Initialize the output source list

        call ftinit(lunout,srclistout,blockl,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTINIT =', status
            print *, 'MERGE_SOURCE_LIST POSITION 02a'
            print *, srclistout
        endif

!  Copy the primary header from the input to the output

        call ftcopy(lunin,lunout,0,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTINIT =', status
            print *, 'MERGE_SOURCE_LIST POSITION 02a'
            print *, srclistin, srclistout
        endif

!  Move to the source list extension of the input file

        call ftmrhd (lunin,1,hdutype,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTMRHD =', status
            print *, 'MERGE_SOURCE_LIST POSITION 03'
            print *, 'Could not find the source list extension'
            print *, srclistin
        endif

!  Create a new empty extension in the output file and move to it

        call ftcrhd(lunout,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTCRHD =', status
            print *, 'MERGE_SOURCE_LIST POSITION 03'
            print *, 'Could not find the source list extension'
            print *, srclistout
        endif

!  Copy the source list from the input to the output file

        call ftcopy(lunin,lunout,0,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTCOPY =', status
            print *, 'MERGE_SOURCE_LIST POSITION 02a'
            print *, srclistin, srclistout
        endif

!  Read the FITS file header information

        call ftghbn (lunout,300,nchan,tfields,ttype,tform, &
                tunit,extname,varidat,status)
         if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGHBN =', status
            print *, 'MERGE_SOURCE_LIST POSITION 04'
            print *, srclistout
        endif
        if(extname .ne. 'SRCLIST') then
            print *, extname
            print *, 'This was supposed to be a source list ext'
            stop
        endif

!  Read in the column names

        call ftgkns (lunout,'TTYPE',1,300,colnam,ncols,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGKNS =', status
            print *, 'MERGE_SOURCE_LIST POSITION 05'
            print *, srclistout
        endif

!  Extract the column numbers for the desired columns

        isour = 0
        iband = 0
        iml = 0
        iflux = 0
        ifluxerr = 0
        irate = 0
        irateerr = 0
        ira = 0
        idec = 0
        iradecerr = 0
        idist = 0
        do icol=1,ncols
            if(colnam(icol) .eq. 'ML_ID_SRC') isour = icol
            if(colnam(icol) .eq. 'ID_BAND') iband = icol
            if(colnam(icol) .eq. 'SCTS') iscts = icol
            if(colnam(icol) .eq. 'DET_ML') iml = icol
            if(colnam(icol) .eq. 'FLUX') iflux = icol
            if(colnam(icol) .eq. 'FLUX_ERR') ifluxerr = icol
            if(colnam(icol) .eq. 'RATE') irate = icol
            if(colnam(icol) .eq. 'RATE_ERR') irateerr = icol
            if(colnam(icol) .eq. 'RA') ira = icol
            if(colnam(icol) .eq. 'DEC') idec = icol
            if(colnam(icol) .eq. 'RADEC_ERR') iradecerr = icol
            if(colnam(icol) .eq. 'DIST_NN') idist = icol
        enddo

        i = 1

!  Loop through all the data

        do while (i .lt. nchan)

!  Read the band number

            call ftgcvj(lunout,iband,i,1,1,0,ib,anyf,status)
            if(ib .eq. 0) then

!  Read in the flux

                call ftgcve(lunout,iflux,i,1,1,-999,flux,anyf,status)
                if(anyf) then

!  Read the source number

                    call ftgcvj(lunout,isour,i,1,1,0,is,anyf,status)
                    ft = 0.0
                    sft = 0.0
                    nft = 0.0
                    ist = is
                    isel = i
                    do while ((ist .eq. is) .and. (i .lt. nchan))

!  Read the source number and band

                        i = i + 1
                        call ftgcvj(lunout,isour,i,1,1,0,ist, &
                                anyf,status)
                        call ftgcvj(lunout,iband,i,1,1,0,ibt, &
                                anyf,status)
                        if((ist .eq. is) .and. (ibt .eq. 1) .and.  &
                                (i .le. nchan)) then

!  Read in the flux

                            call ftgcve(lunout,iflux,i,1,1,-999, &
                                    flux,anyf,status)
                            if(.not. anyf) then    

!  Read in the flux uncertainty

                                call ftgcve(lunout,ifluxerr,i,1,1,0, &
                                        fluxerr,anyf,status)
                                ft = ft + flux
                                sft = sft + fluxerr*fluxerr
                                nft = nft + 1.0
                            endif
                        endif
                    enddo

                    if(nft .gt. 0.0) then
                        ft = ft/nft
                        sft = sqrt(sft)/nft
                        call ftpcle(lunout,iflux,isel,1,1,ft,status)
                        call ftpcle(lunout,ifluxerr,isel,1,1, &
                                sft,status)
                    endif

                else
                    i = i + 1
                endif
            else
                i = i + 1
            endif
        enddo

        call ftclos (lunout,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTCLOS =', status
            print *, 'MERGE_SOURCE_LIST POSITION 08'
        endif

        return

      end subroutine fill_list
      
    end module fill_list_mod
    
