!+
! Module: make_mask_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: make_mask_mod.f90,v 1.2 2009/02/20 13:27:08 aibarra Exp $
!-
!  Version 5.0  9 November 2012
!
!  Called by cheese in mode=2 to create a cheese mask
!
!  MAKE-MASK is called by the PERL script CHEESE to create a
!  swiss cheese mask.
!
!  INPUT:
!    The event image for the exposure.
!    The exposure mask.
!    The output file name for the cheese mask.
!    The filtered source region list.
!
!    Example: make-mask mos1S001-obj-im.fits mos1S001-mask-im.fits
!        mos1S001-cheese.fits mos1S001-bkg_region-sky.fits
!
!  OUPUT:
!    Mask image in sky coordinates
!
!  Usage:
!    event image
!    mask image
!    output cheese image
!    region file
!  
!  Example:
!       make-mask image.fits mask.fits cheese.fits reglist.fits
! 
! Determin program parameters

module make_mask_mod

  use dal
  use param
  use errorHandling

  implicit none

  private

  public make_mask

contains

  !---------------------------------------------------------------------
  subroutine make_mask
  !---------------------------------------------------------------------

        CHARACTER (len=4) ::  CTYP
        CHARACTER (len=8) ::  tform(300), ttype(300), tunit(300), &
            UNITS, CTYPE(2)
        CHARACTER (len=10) :: colnam(300), shape(10000)
        CHARACTER (len=20) :: extname
        CHARACTER (len=60) :: COM
        character (len=132) :: inimage, inmask, outmask, reglist
        LOGICAL :: ANYF, clobber, EXTEND, SIMPLE

        REAL :: rad, rad1, rad2, r(10000,2), rot(10000), rotang, rr, &
            x(10000), xrc, xrefpix, xp, xpixdelt, xx, &
            y(10000), yrc, yrefpix, yp, ypixdelt, yy
        DOUBLE PRECISION :: CDELT(2), CRPIX(2), CROT, CRVAL(2), EXPOSE
        INTEGER :: BITPIX, BLOCKN, BLOCKL, GCOUNT, GROUP, &
            hdutype, I, icol, ir, irad, irot, ishape, ix, ixrc, iy, &
            iyrc, j, j1, j2, jj, jj1, jj2, LUN, MASK(900,900), &
            NAXES(10), NAXIS, nchan,  ncols, pcount, &
            sourcelist, tfields, varidat
        INTEGER :: ERR = 0
        INTEGER :: STATUS = 0

        INIMAGE = stringParameter("inimage")
        INMASK = stringParameter("inmask")
        OUTMASK = stringParameter("outmask")
        REGLIST = stringParameter("reglist")
        CLOBBER = booleanParameter("clobber")

        print *, 'Event Image: ', INIMAGE
        print *, 'Input Mask: ', INMASK
        print *, 'Output Cheese Image: ', OUTMASK
        print *, 'Region File: ', REGLIST

!  Get an unused lun

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'MAKE-MASK POSITION 01'
            stop
        endif

!  Open the FITS file for the template
 
        print *, 'Input Image: ', inimage
        CALL FTOPEN(lun,inimage,0,BLOCKN,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTOPEN =', STATUS
            PRINT *, 'MAKE-MASK POSITION 02'
            PRINT *, inimage
            stop
        ENDIF

!  Read the FITS file header information
 
        CALL FTGHPR (LUN,10,SIMPLE,BITPIX,NAXIS,NAXES, &
                    PCOUNT,GCOUNT,EXTEND,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGHPR =', STATUS
            PRINT *, 'MAKE-MASK POSITION 03'
            PRINT *, inimage
            stop
        ENDIF

!  If necessary, step through extensions until a NAXIS=2 is found

        DO WHILE (NAXIS .NE. 2)

!  Move to the next extension

            CALL FTMRHD (LUN,1,HDUTYPE,STATUS)
            IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
                ERR = 1
                PRINT *, 'STATUS FTMRHD =', STATUS
                PRINT *, 'MAKE-MASK POSITION 3a'
                PRINT *, inimage
            ENDIF
            CALL FTGHPR (LUN,10,SIMPLE,BITPIX,NAXIS,NAXES, &
                    PCOUNT,GCOUNT,EXTEND,STATUS)
            IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
                ERR = 1
                PRINT *, 'STATUS FTGHPR =', STATUS
                PRINT *, 'MAKE-MASK POSITION 3b'
                PRINT *, inmask
                stop
            ENDIF
        ENDDO

!  Get the useful keywords

        CALL FTGICS (lun,CRVAL(1),CRVAL(2),CRPIX(1),CRPIX(2), &
                CDELT(1),CDELT(2),CROT,CTYP,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGICS =', STATUS
            PRINT *, 'MAKE-MASK POSITION 04'
            PRINT *, inimage
            stop
        ENDIF

!  And a few more

        CALL FTGKYE (lun,'REFXCRPX',XREFPIX,COM,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGKYE =', STATUS
            PRINT *, 'MAKE-MASK POSITION 05'
            PRINT *, inimage
            stop
        ENDIF
        CALL FTGKYE (lun,'REFYCRPX',YREFPIX,COM,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGKYE =', STATUS
            PRINT *, 'MAKE-MASK POSITION 06'
            PRINT *, inimage
            stop
        ENDIF
        CALL FTGKYE (lun,'REFXCDLT',XPIXDELT,COM,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGKYE =', STATUS
            PRINT *, 'MAKE-MASK POSITION 07'
            PRINT *, inimage
            stop
        ENDIF
        CALL FTGKYE (lun,'REFYCDLT',YPIXDELT,COM,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGKYE =', STATUS
            PRINT *, 'MAKE-MASK POSITION 08'
            PRINT *, inimage
            stop
        ENDIF

!  Close the file

        CALL FTCLOS (lun,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTCLOS =', STATUS
            PRINT *, 'MAKE-MASK POSITION 09'
            PRINT *, inimage
            stop
        ENDIF

!  Get an unused lun

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'MAKE-MASK POSITION 10'
            stop
        endif

!  Open the FITS file for the template
 
        print *, 'Input Mask: ', inmask
        CALL FTOPEN(lun,inmask,0,BLOCKN,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTOPEN =', STATUS
            PRINT *, 'MAKE-MASK POSITION 11'
            PRINT *, inmask
            stop
        ENDIF

!  Read the FITS file header information

        CALL FTGHPR (LUN,10,SIMPLE,BITPIX,NAXIS,NAXES, &
                    PCOUNT,GCOUNT,EXTEND,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGHPR =', STATUS
            PRINT *, 'MAKE-MASK POSITION 12'
            PRINT *, inmask
            stop
        ENDIF

!  If necessary, step through extensions until a NAXIS=2 is found

        DO WHILE (NAXIS .NE. 2)


!  Move to the next extension

            CALL FTMRHD (LUN,1,HDUTYPE,STATUS)
            IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
                ERR = 1
                PRINT *, 'STATUS FTMRHD =', STATUS
                PRINT *, 'MAKE-MASK POSITION 12a'
                PRINT *, inimage
            ENDIF
            CALL FTGHPR (LUN,10,SIMPLE,BITPIX,NAXIS,NAXES, &
                    PCOUNT,GCOUNT,EXTEND,STATUS)
            IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
                ERR = 1
                PRINT *, 'STATUS FTGHPR =', STATUS
                PRINT *, 'MAKE-MASK POSITION 12b'
                PRINT *, inmask
                stop
            ENDIF
        ENDDO

!  Get the useful keywords

        CALL FTGICS (LUN,CRVAL(1),CRVAL(2),CRPIX(1),CRPIX(2), &
                CDELT(1),CDELT(2),CROT,CTYP,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGICS =', STATUS
            PRINT *, 'MAKE-MASK POSITION 13'
            PRINT *, inmask
            stop
        ENDIF
        CALL FTGKYS (lun,'CTYPE1',CTYPE(1),COM,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGKYS =', STATUS
            PRINT *, 'MAKE-MASK POSITION 13a'
            PRINT *, inmask
            stop
        ENDIF
        CALL FTGKYS (lun,'CTYPE2',CTYPE(2),COM,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGKYS =', STATUS
            PRINT *, 'MAKE-MASK POSITION 13b'
            PRINT *, inmask
            stop
        ENDIF

!  Read in the mask image

        CALL FTG2DJ (lun,0,0,NAXES(1),NAXES(1),NAXES(2),MASK, &
               ANYF,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTG2DJ =', STATUS
            PRINT *, 'MAKE-MASK POSITION 14'
            PRINT *, inmask
            stop
        ENDIF

!  Close the file

        CALL FTCLOS (lun,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTCLOS =', STATUS
            PRINT *, 'MAKE-MASK POSITION 15'
            PRINT *, inmask
            stop
        ENDIF

!  Print the details of the template image

        PRINT *, 'Template File'
        PRINT *, inmask
        PRINT *, 'Template image coordinate system: ', CTYP
        PRINT *, 'Template image coordinate ID: ', &
                CTYPE(1), '  ', CTYPE(2)
        PRINT *, 'Template image central coordinates: ', &
                CRVAL(1), CRVAL(2)
        PRINT *, 'Template image reference pixel: ', &
                CRPIX(1), CRPIX(2)
        PRINT *, 'Template image pixel sizes: ', &
                CDELT(1), CDELT(2)
        PRINT *, 'Template image NAXES, NAXIS: ', NAXIS, &
                (NAXES(I),I=1,NAXIS)

!  Get an unused lun

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'MAKE-MASK POSITION 16'
            stop
        endif

!  Open and read in the source list

        sourcelist = 1
        print *, reglist
        CALL FTOPEN(lun,reglist,0,BLOCKL,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            print *, 'STATUS FTOPEN =', STATUS
            print *, 'MAKE-MASK POSITION 17'
            print *, 'Region file does not exist, continue ', &
                'with now source removal'
            print *, reglist
            sourcelist = 0
            status = 0
            stop
        ENDIF

!  Move to the source list extension

        if(sourcelist .eq. 1) then
            call ftmrhd (lun,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTMRHD =', status
                print *, 'MAKE-MASK POSITION 18'
                print *, 'Could not find the source list extension'
                print *, reglist
                stop
            else

!  Read the FITS file header information

                call ftghbn (lun,300,nchan,tfields,ttype,tform, &
                        tunit,extname,varidat,status)
                if(extname .ne. 'REGION') then
                    print *, 'MAKE-MASK POSITION 19'
                    print *, extname
                    print *, 'This was supposed to be a region list ext'
                    stop
                endif

!  Read in the column names

                call ftgkns (lun,'TTYPE',1,300,colnam,ncols,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'STATUS FTGKNS =', status
                    print *, 'MAKE-MASK POSITION 20'
                    print *, reglist
                    stop
                endif

!  Extract the column numbers for the desired columns

                ishape = 0
                ix = 0
                iy = 0
                ir = 0
                irot = 0
                do icol=1,ncols
                    if(colnam(icol) .eq. 'SHAPE') ishape = icol
                    if(colnam(icol) .eq. 'X') ix = icol
                    if(colnam(icol) .eq. 'Y') iy = icol
                    if(colnam(icol) .eq. 'R') ir = icol
                    if(colnam(icol) .eq. 'ROTANG') irot = icol
                enddo

!  Now read in the data

                do i=1,nchan
                    if (ishape .ne. 0) then
                        call ftgcvs(lun,ishape,i,1,1,'Null', &
                                shape(i),anyf,status)
                    endif
                    if (ix .ne. 0) &
                        call ftgcve(lun,ix,i,1,1,0,x(i),anyf,status)
                    if (iy .ne. 0) &
                        call ftgcve(lun,iy,i,1,1,0,y(i),anyf,status)
                    if (ir .ne. 0) then
                        call ftgcve(lun,ir,i,1,1,0,r(i,1),anyf,status)
                        call ftgcve(lun,ir,i,2,1,0,r(i,2),anyf,status)
                    endif
                    if (irot .ne. 0)  &
                        call ftgcve(lun,irot,i,1,1,0,rot(i),anyf,status)
                enddo

                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'STATUS FTGCV* =', status
                    print *, 'POSITION 21'
                    print *, 'Problem reading in source list data'
                    print *, reglist
                    stop
                endif
            endif

!  Close the file

            CALL FTCLOS (lun,STATUS)
            IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
                err = 1
                print *, 'STATUS FTCLOS =', STATUS
                print *, 'MAKE-MASK POSITION 22'
                print *, reglist
                stop
            endif

!  Now start over the loop to mask sources

            print *, 'Number of excluded sources: ', nchan
            do i=1,nchan
                if(shape(i) .eq. '!CIRCLE') then
!                    print *, shape(i),x(i),y(i),r(i,1)
                    xrc = crpix(1) + &
                            xpixdelt*(x(i) - xrefpix)/cdelt(1)
                    yrc = crpix(2) + &
                            ypixdelt*(y(i) - yrefpix)/cdelt(2)
                    rad = r(i,1)*ypixdelt/cdelt(2)
                    ixrc = xrc
                    iyrc = yrc
                    irad = rad

!  Check to see whether the radius has exploded
 
                    if(irad .gt. 0) then
                        j1 = ixrc - irad
                        j2 = ixrc + irad
                        if(j1 .lt. 1) j1 = 1
                        if(j2 .gt. 900) j2 = 900
                        do j=j1,j2
                            jj1 = iyrc - irad
                            jj2 = iyrc + irad
                            if(jj1 .lt. 1) jj1 = 1
                            if(jj2 .gt. 900) jj2 = 900
                            do jj=jj1,jj2
                                rr = sqrt((j-xrc)**2 + (jj-yrc)**2)
                                if (rr .lt. rad) mask(j,jj) = 0
                            enddo
                        enddo
                    endif
                elseif(shape(i) .eq. '!ELLIPSE') then
!                    print *, shape(i),x(i),y(i),r(i,1),r(i,2),rot(i)
                    xrc = crpix(1) + &
                            xpixdelt*(x(i) - xrefpix)/cdelt(1)
                    yrc = crpix(2) + &
                            ypixdelt*(y(i) - yrefpix)/cdelt(2)
                    rad1 = r(i,1)*ypixdelt/cdelt(2)
                    rad2 = r(i,2)*ypixdelt/cdelt(2)
                    ixrc = xrc
                    iyrc = yrc
                    irad = rad1
                    if(rad1 .lt. rad2) irad = rad2
                    rotang = -rot(i)*3.14159263/180.0

!                    print *, xrc, yrc, rad1, rad2, rot(i), rotang

!  Check to see whether the radius has exploded

!                    if((irad .gt. 0) .and. (irad .le. 48)) then

                    rad1 = rad1*rad1
                    rad2 = rad2*rad2
                    if(irad .gt. 0) then
                        j1 = ixrc - irad
                        j2 = ixrc + irad
                        if(j1 .lt. 1) j1 = 1
                        if(j2 .gt. 900) j2 = 900
                        jj1 = iyrc - irad
                        jj2 = iyrc + irad
                        if(jj1 .lt. 1) jj1 = 1
                        if(jj2 .gt. 900) jj2 = 900
                        do j=j1,j2
                            do jj=jj1,jj2
                                xx = j - ixrc
                                yy = jj - iyrc
                                xp = xx*cos(rotang) - yy*sin(rotang)
                                yp = yy*cos(rotang) + xx*sin(rotang)
                                rr = xp*xp/rad1 + yp*yp/rad2
                                if (rr .lt. 1.0) mask(j,jj) = 0
                            enddo
                        enddo
                    endif
                endif
            enddo
        endif

!  Write out the new mask
!  Get an unused lun

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'MAKE-MASK POSITION 23'
            stop
        endif

!  Check to see if there is an existing file

        call ftopen (lun,outmask,0,blockn,status)
        if(status .eq.  0) then
            print *, 'File exists with output file name'
            print *, outmask
            if(clobber) then
                print *, 'Clobber the file and continue'
                call ftdelt(lun,status)
            else
                print *, 'Will not clobber file'
                print *, 'MAKE-MASK POSITION 24'
                stop
            endif
        endif
        status = 0

!  Create the FITS file for the output image

        GROUP = 1
        BITPIX = 32
        CALL FTINIT(lun,outmask,2880,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTINIT =', STATUS
            PRINT *, 'MAKE-MASK POSITION 25'
            PRINT *, outmask
            stop
        ENDIF

!  Set the output file header

        CALL FTPHPR(lun,SIMPLE,BITPIX,NAXIS,NAXES,PCOUNT,GCOUNT, &
                    EXTEND,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTGHPR =', STATUS
            PRINT *, 'MAKE-MASK POSITION 26'
            PRINT *, outmask
            stop
        ENDIF

!  Write some useful header keywords

        CALL FTPKYS(lun,'CONTENT','IMAGE','  ',STATUS)
        CALL FTPKYS(lun,'TELESCOP','XMM','mission name',STATUS)
        CALL FTPKYS(lun,'INSTRUME','EPIC','instrument name',STATUS)
        CALL FTPKYS(lun,'OBS_MODE','POINTING', &
                'obs mode: POINTING,SLEW, OR SCAN',STATUS)
        CALL FTPKYS(lun,'RADECSYS','FK5', &
                'Equatorial system reference',STATUS)
        CALL FTPKYJ(lun,'EQUINOX',2000,'Equinox',STATUS)
        CALL FTPKYD(lun,'EXPOSURE',EXPOSE,6,'Reference pixel',STATUS)
        CALL FTPKYF(lun,'BZERO',0.0,8,' ',STATUS)
        CALL FTPKYF(lun,'BSCALE',1.0,8,' ',STATUS)
        UNITS = 'counts'
        CALL FTPKYS(lun,'BUNIT',UNITS,'Units of data',STATUS)
        CALL FTPKYD(lun,'CRPIX1',CRPIX(1),8,'Reference pixel',STATUS)
        CALL FTPKYS(lun,'CTYPE1',CTYPE(1),'Projection',STATUS)
        CALL FTPKYD(lun,'CRVAL1',CRVAL(1),8,'Central longitude',STATUS)
        CALL FTPKYD(lun,'CDELT1',CDELT(1),8,'Pixel size',STATUS)
        CALL FTPKYS(lun,'CUNIT1','deg','Units of coordinate',STATUS)

        CALL FTPKYD(lun,'CRPIX2',CRPIX(2),8,'Reference pixel',STATUS)
        CALL FTPKYS(lun,'CTYPE2',CTYPE(2),'Projection',STATUS)
        CALL FTPKYD(lun,'CRVAL2',CRVAL(2),8,'Central latitude',STATUS)
        CALL FTPKYD(lun,'CDELT2',CDELT(2),8,'Pixel size',STATUS)
        CALL FTPKYS(lun,'CUNIT2','deg','Units of coordinate',STATUS)

        CALL FTPCOM(lun, &
     'This file was produced using the XMM-ESAS software written by', &
        STATUS)
        CALL FTPCOM(lun, &
        'S. L. Snowden, and written using the FITSIO package of ', &
        STATUS)
        CALL FTPCOM(lun,'W. D. Pence.',STATUS)
        CALL FTPCOM(lun, &
        'This is a source mask (Swiss cheese mask) created by the ', &
        STATUS)
        CALL FTPCOM(lun, &
        'program make-mask using a merged source list output from', &
        STATUS)
        CALL FTPCOM(lun,'the SAS task srcmatch',STATUS)

        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS various keyword writing =', STATUS
            PRINT *, 'MAKE-MASK POSITION 27'
            PRINT *, outmask
            stop
        ENDIF

!  Print the details of the output image

        PRINT *, 'Output image coordinate system: ', CTYP
        PRINT *, 'Output image coordinate ID: ', &
                CTYPE(1), '  ', CTYPE(2)
        PRINT *, 'Output image central coordinates: ', &
                CRVAL(1), CRVAL(2)
        PRINT *, 'Output image reference pixel: ', &
                CRPIX(1), CRPIX(2)
        PRINT *, 'Output image pixel sizes: ', &
                CDELT(1), CDELT(2)
        PRINT *, 'Output image NAXES, NAXIS: ', NAXIS, &
                (NAXES(I),I=1,NAXIS)

!  Define the file format

        CALL FTPDEF(lun,BITPIX,NAXIS,NAXES,PCOUNT,GCOUNT, &
                    EXTEND,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTPDEF =', STATUS
            PRINT *, 'MAKE-MASK POSITION 28'
            PRINT *, outmask
            stop
        ENDIF

!  Write the file

        CALL FTP2DJ(lun,GROUP,NAXES(1),NAXES(1),NAXES(2), &
                mask,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTP2DJ =', STATUS
            PRINT *, 'MAKE-MASK POSITION 29'
            PRINT *, outmask
            stop
        ENDIF

!  Close the file
 
        CALL FTCLOS(lun,STATUS)
        IF((STATUS .NE. 0) .AND. (ERR .EQ. 0)) THEN
            ERR = 1
            PRINT *, 'STATUS FTCLOS =', STATUS
            PRINT *, 'MAKE-MASK POSITION 30'
            PRINT *, outmask
        ENDIF
 
        RETURN

  end subroutine make_mask

end module make_mask_mod
