!+
! Module: merge_off_xmm_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
! SLS 7 May 2009
!
! Comments:
!
! $Id: merge_off_xmm_mod.f90,v 1.4 2009/05/26 12:53:39 aibarra Exp $
!-
!  SAS Version 13.0
!  Version 5.0  2 November 2012
!
!  Merges the offset counts of various observations into one large mosaic
!
!  merge_off_xmm mosaics the modeled offset counts (determined by the program
!  OVERLAP) from multiple pointings into one image.  The output projection 
!  is the zenith equal-area azimuthal, which can be approximated for 
!  display purposes by using tangential-projection software
!
!  INPUT:
!    Coordinate Control:
!       1 => ecliptic
!       2 => equatorial
!       3 => galactic
!    Central Coordinates, longitude and latitude
!    Pixel Size (decimal degrees)
!    Upper energy limit for the energy band in eV
!    Lower energy limit for the energy band in eV
!    Mask control
!        0 - No Masking
!        1 - Point Source Masking
!        2 - Good Area Masking
!        3 - Mask from merged source list
!
!  OUTPUT:
!    Mosaicked image
!
!
!  Example: merge_off_xmm caldb=/CalDB coord=2 
!                 crvaln1=194.6 crvaln2=27.7 pixelsize=0.0666667  
!                 elow=400 ehigh=750 maskcontrol=0
!
!  OUTPUT:
!    Mosaicked image
!

module merge_off_xmm_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public merge_off_xmm

contains

  !---------------------------------------------------------------------
  subroutine merge_off_xmm
  !---------------------------------------------------------------------

        real :: aveexp, avesp, BZERO, BSCALE, exp(900,900), &
            input(900,900), LAT(1000), LONG(1000), &
            mask(900,900), prot(900,900), sca, scal(2000), scale, &
            temp, tot, totexp, totnum, totsc, totsp
        real, allocatable, dimension(:,:) :: merg
        character(len=1) :: COMPI(30), dirstr(132), &
            infi(132), MASKI(30) 
        character(len=4)  :: ctypn, ctypo
        character(len=8)  :: ctype(2), ctypen(2), ctypeo(2), UNITS
        character(len=30) :: comp, diag, MASKNA
        character(len=80) :: dir(2000)
        character(len=60) :: COMMENT
        character(len=132) :: dirtem, infile
        integer :: COORD, i, idir, ieh, iel, ierr, &
            iflag, ii, INUM, IOS, IPIXCRD(2), ISTAT, &
            lundi, lun, luni, luno, MASKCON, NUM, xdim, ydim
        real :: alpha = 2073600.0
        integer :: err = 0
        integer :: status = 0
        integer ini4(900,900)
        integer, allocatable, dimension(:,:) :: ini4l
        double precision :: cdec, cra, crotn, croto, &
            DBIN, DBOUT, DJUL, DLIN, DLOUT, ddeccen, dracen, &
            xpixin, ypixin
        double precision, dimension(2) :: cdelt, cdeltn, cdelto, &
            crpix, crpixn, crpixo, crval, crvaln, crvalo, pixcrd 
        double precision :: drad = 3.141592654D0
        logical :: clobber, NOMASK

        EQUIVALENCE (infile,infi)
        EQUIVALENCE (dirtem,dirstr)
        EQUIVALENCE (INPUT,INI4)
        EQUIVALENCE (COMP,COMPI)
        EQUIVALENCE (MASKNA,MASKI)

        data maskna /'-cheese.fits'/

        coord = intParameter("coord")
        crvaln(1) = realParameter("crvaln1")
        crvaln(2) = realParameter("crvaln2")
        scale = realParameter("pixelsize")
        iel = intParameter("elow")
        ieh = intParameter("ehigh")
        xdim = intParameter("xdim")
        ydim = intParameter("ydim")
        maskcon = intParameter("maskcontrol")
        clobber = booleanParameter("clobber")

        allocate(merg(xdim,ydim))
        allocate(ini4l(xdim,ydim))

        drad = drad/180.D0
        do i=1,xdim
            do ii=1,ydim
                merg(i,ii) = 0.0
                ini4l(i,ii) = 0
            enddo
        enddo

!  Get unused luns

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_OFF_XMM POSITION 01a'
            print *, 'STATUS FTGIOU =', status
            stop
        endif

        call ftgiou(lundi,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_OFF_XMM POSITION 01b'
            print *, 'STATUS FTGIOU =', status
            stop
        endif

        call ftgiou(luni,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'MERGE_OFF_XMM POSITION 01c'
            print *, 'STATUS FTGIOU =', status
            stop
        endif

        call ftgiou(luno,status)
        if(status .ne. 0) then
            print *, 'MERGE-COMP-XMM POSITION 01d'
            print *, 'STATUS FTGIOU =', status
        endif

!  List the program control parameters.

        print *, 'Coord System', coord
        if(coord .eq. 1) then
            ctypen(1) = 'ELON-TAN'
            ctypen(2) = 'ELAT-TAN'
        elseif(coord .eq. 2) then
            ctypen(1) = 'RA---TAN'
            ctypen(2) = 'DEC--TAN'
        elseif(coord .eq. 3) then
            ctypen(1) = 'GLON-TAN'
            ctypen(2) = 'GLAT-TAN'
        endif

        ctypn = '-TAN'
        ctypo = '-TAN'
        djul = 2451544.5D0
        print *, 'Energy Range: ', iel, ieh
        print *, 'Output Array Size: ', xdim, ydim
        print *, 'Pixel Size: ', scale
        print *, 'Mask Control: ', maskcon
        print *, 'Clobber: ', clobber

        if(coord .eq. 1) then
            dlin = drad*crvaln(1)
            dbin = drad*crvaln(2)
            call equecl(djul,.FALSE.,.FALSE., &
                    dlin,dbin,dlout,dbout,istat)
            if(istat .NE. 0) then
                print*, 'MERGE_OFF_XMM POSITION 07'
                print*,'MERGE_OFF_XMM: Error on return ', &
                        'from eqUECL'
                print*,'istat =', istat
                ierr = 3
            else
                dracen = dlout/drad
                ddeccen = dbout/drad
            endif

!  Leave in equatorial coordinates

        elseif(coord .eq. 2) then
            dracen = crvaln(1)
            ddeccen = crvaln(2)

!  Transform from galactic to equatorial coordinates

        elseif(coord .eq. 3) then
            dlin = drad*crvaln(1)
            dbin = drad*crvaln(2)
            call equgal(djul,.FALSE.,dlin,dbin,dlout,dbout,istat)
            if(istat .NE. 0) then
                print*,'MERGE_OFF_XMM POSITION 08'
                print*,'MERGE_OFF_XMM: Error on return ', &
                        'from eqUGAL'
                ierr = 3
            else
                dracen = dlout/drad
                ddeccen = dbout/drad
            endif
        endif
        dracen = drad*dracen
        ddeccen = drad*ddeccen

! Create output file name

        if(iel .ge. 1000) then
            if(ieh .ge. 10000) then
                write(comp,3105) iel, ieh
 3105           format('off-im-',I4,'-',I5,'.fits')
                write(diag,3106) iel, ieh
 3106           format('diag-',I4,'-',I5,'.out')
            else
                write(comp,3115) iel, ieh
 3115           format('off-im-',I4,'-',I4,'.fits')
                write(diag,3116) iel, ieh
 3116           format('diag-',I4,'-',I4,'.fits')
            endif
        elseif(ieh .LT. 1000) then
            write(comp,3125) iel, ieh
 3125       format('off-im-',I3,'-',I3,'.fits')
            write(diag,3126) iel, ieh
 3126       format('diag-',I3,'-',I3,'.fits')
        elseif(ieh .GE. 10000) then
            write(comp,3135) iel, ieh
 3135       format('off-im-',I3,'-',I5,'.fits')
            write(diag,3136) iel, ieh
 3136       format('diag-',I3,'-',I5,'.fits')
        else
            write(comp,3145) iel, ieh
 3145       format('off-im-',I3,'-',I4,'.fits')
            write(diag,3146) iel, ieh
 3146       format('diag-',I3,'-',I4,'.fits')
        endif

        crpixn(1) = xdim/2.0 + 0.5
        crpixn(2) = ydim/2.0 + 0.5
        cdeltn(1) = -scale/60.0
        cdeltn(2) = scale/60.0
        crotn = 0.0
        print *, 'Merged image coordinate system: ', ctypn
        print *, 'Merged image coordinate ID: ', &
                        ctypen(1), '  ', ctypen(2)
        print *, 'Merged image central coordinates: ', &
                        crvaln(1), crvaln(2)
        print *, 'Merged image reference pixel: ', &
                        crpixn(1), crpixn(2)
        print *, 'Merged image pixel sizes: ', &
                        cdeltn(1), cdeltn(2)
        print *, 'Merged image rotation angle: ', crotn

!  Read in the information for the observations to be merged

        if(iel .ge. 1000) then
            write(infile,3000) iel, ieh
 3000       format('offset-',I4,'-',I4,'.out')
        elseif(ieh .LT. 1000) then
            write(infile,3010) iel, ieh
 3010       format('offset-',I3,'-',I3,'.out')
        else
            write(infile,3020) iel, ieh
 3020       format('offset-',I3,'-',I4,'.out')
        endif
        print *, ' '
        print *, 'Input Data List: ', infile

        OPEN(UNIT=luni,STATUS='OLD',FORM='formatted', &
                FILE=infile)
        ios = 0
        num = 1
        do while (ios .eq. 0)
            read(luni,1010,iostat=ios) dirtem, long(num), lat(num), &
                    scal(num)
            dir(num) = dirtem
 1010       format(a80,3f10.2)
            if(ios .eq. 0) then
                print 2010, dir(num), long(num), lat(num), scal(num)
                num = num + 1
 2010           format(1x,a80,4f10.2)
            endif
        enddo
        close(luni)
        call ftfiou (luni,status)
        num = num - 1
        print *, num, ' Observations'

        open(unit=lundi,status='unknown',form='formatted',file=diag)
        do inum=1,num
            tot = 0.0
            totsc = 0.0

!  Find the length of the directory string

            dirtem = dir(inum)
            I = 1
            do while (dirstr(i) .ne. ' ')
                    i = i + 1
            enddo
            idir = i - 1

!  Make the exposure image file name

            if(iel .ge. 1000) then
                if(ieh .ge. 10000) then
                    write(comp,3201) iel, ieh
 3201               format('-exp-im-',I4,'-',I5,'.fits')
                else
                    write(comp,3202) iel, ieh
 3202               format('-exp-im-',I4,'-',I4,'.fits')
                endif
            else
                if(ieh .ge. 10000) then
                    write(comp,3203) iel, ieh
 3203               format('-exp-im-',I3,'-',I5,'.fits')
                elseif(ieh .ge. 1000) then
                    write(comp,3204) iel, ieh
 3204               format('-exp-im-',I3,'-',I4,'.fits')
                else
                    write(comp,3205) iel, ieh
 3205               format('-exp-im-',I3,'-',I3,'.fits')
                endif
            endif
            do i=1,30
                dirstr(idir+i) = compi(i)
            enddo
            infile = dirtem

!  Read in the SP event image

            call retfit(infile,exp,ini4,1,900,900,ctypeo, &
                        crvalo,cdelto,crpixo,status)

!  Make the soft proton image file name

            if(iel .ge. 1000) then
                if(ieh .ge. 10000) then
                    write(comp,3206) iel, ieh
 3206               format('-prot-im-sky-',I4,'-',I5,'.fits')
                else
                    write(comp,3207) iel, ieh
 3207               format('-prot-im-sky-',I4,'-',I4,'.fits')
                endif
            else
                if(ieh .ge. 10000) then
                    write(comp,3208) iel, ieh
 3208               format('-prot-im-sky-',I3,'-',I5,'.fits')
                elseif(ieh .ge. 1000) then
                    write(comp,3209) iel, ieh
 3209               format('-prot-im-sky-',I3,'-',I4,'.fits')
                else
                    write(comp,3210) iel, ieh
 3210               format('-prot-im-sky-',I3,'-',I3,'.fits')
                endif
            endif
            do i=1,30
                dirstr(idir+i) = compi(i)
            enddo
            infile = dirtem

!  Read in the SP event image

            call retfit(infile,prot,ini4,1,900,900,ctypeo, &
                        crvalo,cdelto,crpixo,status)

!  Print the central coords and other information of the observation

            print *, infile
            print *, 'Status: ', STATUS
            print *, 'Original image coordinate system: ', &
                    ctypeo(1), '  ', ctypeo(2)
            print *, 'Original image central coordinates: ', & 
                    crvalo(1), '  ', crvalo(2)
            print *, 'Original image reference pixel: ', &
                    crpixo(1), '  ', crpixo(2)
            print *, 'Original image pixel sizes: ', &
                    cdelto(1), '  ', cdelto(2)

!  Read in point source masks to go with the observation

            if((maskcon .eq. 1) .or. (maskcon .eq. 11) &
                    .or. (maskcon .eq. 12)) then
                if(maskcon .eq. 1) then
                    maskna = '-cheese.fits'
                elseif(maskcon .eq. 11) then
                    maskna = '-cheese-s.fits'
                elseif(maskcon .eq. 12) then
                    maskna = '-cheese-h.fits'
                endif

                do i=1,30
                    infi(idir+i) = maski(i)
                enddo
                print *, infile
                call retfit(infile,mask, ini4,1,900,900,ctype,crval, &
                        cdelt,crpix,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'MERGE-COMP-XMM POSITION 22'
                    print *, 'STATUS RETFIT =', status
                    print *, 'Missing a cheese mask'
                    print *, infile
                    stop
                endif
                nomask = .FALSE.

!  Read in good area masks

            elseif(maskcon .eq. 2) then
                if(iel .ge. 1000) then
                    write(maskna,3610) iel, ieh
 3610               format('-mask-im-',i4,'-',i4,'.fits')
                elseif(ieh .lt. 1000) then
                    write(maskna,3620) iel, ieh
 3620               format('-mask-im-',i3,'-',i3,'.fits')
                else
                    write(maskna,3630) iel, ieh
 3630               format('-mask-im-',i3,'-',i4,'.fits')
                endif
                do i=1,30
                    infi(idir+i) = maski(i)
                enddo
                print *, infile
                call retfit(infile,mask,ini4,1,900,900,ctype,crval, &
                        cdelt,crpix,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'MERGE-COMP-XMM POSITION 23'
                    print *, 'STATUS RETFIT =', status
                    print *, 'Missing a normal mask'
                    print *, infile
                    stop
                endif
                nomask = .FALSE.

!  Read in the merged source list masks

            elseif((maskcon .eq. 3) .or. (maskcon .eq. 31) &
                    .or. (maskcon .eq. 32)) then
                if(maskcon .eq. 3) then
                    maskna = '-msl-cheese.fits'
                elseif(maskcon .eq. 31) then
                    maskna = '-msl-cheese-s.fits'
                elseif(maskcon .eq. 32) then
                    maskna = '-msl-cheese-h.fits'
                endif
                do i=1,30
                    infi(idir+i) = maski(i)
                enddo
                print *, '  '
                print *, 'Mask: ', infile
                call retfit(infile,mask,ini4,1,900,900,ctype,crval, &
                        cdelt,crpix,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'MERGE-COMP-XMM POSITION 24'
                    print *, 'STATUS RETFIT =', status
                    print *, 'Missing a merged mask'
                    print *, infile
                    stop
                endif
                nomask = .FALSE.
            else

!  Make full-field masks

                do i=1,900
                    do ii=1,900
                        mask(i,ii) = 1.0
                    enddo
                enddo
                nomask = .TRUE.
            endif

!  Normalize the SP event image

            totexp = 0.0
            totsp = 0.0
            totnum = 0.0
            do i=1,900
                do ii=1,900
                    totexp = totexp + exp(i,ii)
                    totsp = totsp + prot(i,ii)
                    totnum = totnum + 1.0
                enddo
            enddo
            aveexp = totexp/totnum
            avesp = totsp/totnum
            sca = scal(inum)*totexp/(totsp*alpha)
            print *, 'sca, scal, totexp, totsp, alpha: ', sca, &
                    scal(inum), totexp, totsp, alpha

!  Cast the soft proton image observation into the mosaic

            do i=1,900
                xpixin = i
                do ii=1,900
                    ypixin = ii
                    if((prot(i,ii) .ne. 0.0) .and. &
                            (mask(i,ii) .gt. 0.0)) then
                        call ftwldp (xpixin,ypixin,crvalo(1), &
                                crvalo(2),crpixo(1),crpixo(2), &
                                cdelto(1),cdelto(2),croto,ctypo, &
                                cra,cdec,status)
                        if((status .ne. 0) .and. (err .eq. 0)) then
                            err = 1
                            print *, 'Status FTWLDP =', status
                            print *, 'MERGE_OFF_XMM Position 18'
                        endif

!  Transform from equatorial to ecliptic coordinates

                        if(coord .eq. 1) then
                            dlin = drad*cra
                            dbin = drad*cdec
                            call equecl(djul,.FALSE.,.TRUE., &
                                    dlin,dbin,dlout,dbout,istat)
                            if(istat .ne. 0) then
                                print *, 'MERGE-OFF-XMM POSITION 26'
                                print *, 'istat =', istat
                                print *, 'Error on return from equecl'
                                ierr = 3
                            else
                                cra = dlout/drad
                                cdec = dbout/drad
                            endif

!  Leave in equatorial coordinates

                        elseif(coord .eq. 2) then
                            continue

!  Transform from equatorial to galactic coordinates

                        elseif(coord .eq. 3) then
                            dlin = drad*cra
                            dbin = drad*cdec
                            call equgal(djul,.TRUE.,dlin, &
                                    dbin,dlout,dbout,istat)
                            if(istat .ne. 0) then
                                print *, 'MERGE-OFF-XMM POSITION 27'
                                print *, 'Error on return from equgal'
                                ierr = 3
                            else
                                cra = dlout/drad
                                cdec = dbout/drad
                            endif
                        endif

!  Transform into the merged system

                        call ftxypx (cra,cdec,crvaln(1),crvaln(2), &
                                crpixn(1),crpixn(2),cdeltn(1), &
                                cdeltn(2),crotn,ctypn,pixcrd(1), &
                                pixcrd(2),status)
                        ipixcrd(1) = nint(pixcrd(1))
                        ipixcrd(2) = nint(pixcrd(2))
                        if((status .ne. 0) .and. (err .eq. 0)) then
                            err = 1
                            print *, 'Status FTXYPX =', status
                            print *, 'MERGE-OFF-XMM Position 19'
                            print *, infile
                        endif

                        if((ipixcrd(1) .ge. 1) .and. &
                                (ipixcrd(1) .le. xdim) .and. & 
                                (ipixcrd(2) .ge. 1) .and. &
                                (ipixcrd(2) .le. ydim)) then
                            temp = sca*prot(i,ii)
                            merg(ipixcrd(1),ipixcrd(2)) = &
                                merg(ipixcrd(1),ipixcrd(2)) + temp
                            tot = tot + prot(i,ii)
                            totsc = totsc + sca*prot(i,ii)
                        endif
                    endif
                enddo
            enddo
            write(lundi,2000) tot, totsc, crvalo(1), crvalo(2), &
                    trim(infile)
 2000       format(1x,4f10.4,a50)
        enddo
        close(lundi)

!  Create the output file name

        do I=1,110
            INFI(I) = ' '
        enddo
        if(iel .ge. 10000) then
            if(iel .ge. 1000) then
                write(infile,3401) iel, ieh
 3401           format('offset-im-',I4,'-',I5,'.fits')
            else
                write(infile,3402) iel, ieh
 3402           format('offset-im-',I3,'-',I5,'.fits')
            endif
        elseif(ieh .ge. 1000) then
            if(iel .ge. 1000) then
                write(infile,3403) iel, ieh
 3403           format('offset-im-',I4,'-',I4,'.fits')
            else
                write(infile,3404) iel, ieh
 3404           format('offset-im-',I3,'-',I4,'.fits')
            endif
        else
            write(infile,3405) iel, ieh
 3405       format('offset-im-',I3,'-',I3,'.fits')
        endif
        print *,infile

!  Write out the merged offset array

        units = 'counts'
        comment = 'Merged offset count map'
        iflag = 1
        bzero = 0.0
        bscale = 1.0
        if(status .eq. 0) then
            call putfit (infile,merg,ini4l,iflag,units, &
                comment,xdim,ydim,bzero,bscale,ctypen,crvaln, &
                cdeltn,crpixn,clobber,status)
        endif

  end subroutine merge_off_xmm

end module merge_off_xmm_mod

