!+
! Module: mos_back_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: mos_back_mod.f90,v 1.2 2009/02/20 13:27:08 aibarra Exp $
!-
!
!  SAS Version 13.0
!  Version 5.0  6 November 2012
!
!  Creates particle background spectra and images
!
!  xmm-back is part of the XMM-ESAS package and it produces
!  model particle background spectra and images (in detector
!  coordinates) from the output files of the perl script mos-spectra.
!
!  Input:
!    prefix: Exposure ID prefix - e.g., prefix=1S001
!    caldb: QPB data directory - e.g., caldb=/XMM/esas/caldb
!    diag: diagnostic flag - e.g., diag=1
!    elow: Lower energy limit for the energy band in eV, if only spectra
!        are desired set to 0 - e.g., elow=0 or elow=400
!    ehigh: Upper energy limit for the energy band in eV, if only spectra
!        are desired set to 0 - e.g., ehigh=0 or ehigh=1250
!    ccd1: CCD control for CCD#1, 1 to include, 0 to not - e.g.,
!        ccd1=0 or ccd1=1
!    ccd2: CCD control for CCD#2, 1 to include, 0 to not - e.g.,
!        ccd2=0 or ccd2=1
!    ccd3: CCD control for CCD#3, 1 to include, 0 to not - e.g.,
!        ccd3=0 or ccd3=1
!    ccd4: CCD control for CCD#4, 1 to include, 0 to not - e.g.,
!        ccd4=0 or ccd4=1
!    ccd5: CCD control for CCD#5, 1 to include, 0 to not - e.g.,
!        ccd5=0 or ccd5=1
!    ccd6: CCD control for CCD#6, 1 to include, 0 to not - e.g.,
!        ccd6=0 or ccd6=1
!    ccd7: CCD control for CCD#7, 1 to include, 0 to not - e.g.,
!        ccd7=0 or ccd7=1
!
!    Example: mos_back prefix=1S003 caldb=/XMM/esas/caldb diag=0 elow=0
!                 ehigh=0 ccd1=1 ccd2=1 ccd3=1 ccd4=1 ccd5=1 ccd6=1 ccd7=1

module mos_back_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public mos_back

contains

  !---------------------------------------------------------------------
  subroutine mos_back
    !---------------------------------------------------------------------

    character (len=1) :: db_fi(132), mosnam, qpb_fi(25), xmmd(132), can_fi(25) 
    character (len=4) :: detname = '    '
    character (len=5) :: prefix
    character (len=8) :: caform(10), catype(10), caunit(10)
    character (len=8) :: tform(10), ctype(2), ttype(10), tunit(10)
    character (len=20) :: extname
    character (len=24) :: title
    character (len=25) :: mode
    character (len=80) :: back_file, fname, imo_name, obj_file, &
         obj_rmf, spec_file, spec_name
    character (len=132) :: im_name, xmmdir
    character (len=25) :: qpb_file = '/mos1-qpb.fits.gz'
    !    character (len=25) :: can_file = '/mos-can.fits.gz'
    character (len=80) :: db_file = '      '

    logical :: anyf, chisqfl, clobber
    logical :: canflag = .FALSE.

    integer :: bitpix, blockn, ccd(7), ccd1, ccd2, ccd3, ccd4, &
         ccd5, ccd6, ccd7, ccdnu, chan(2400), channel(2400), diag, &
         eh, el, errr, gcount, group, grup(2400), hdutype, &
         i, i1, i2, icanflag, ican, ichan, ii, iii, ini4(780,780), &
         irow, ispec(2400), it, ites, itest, j, jj, &
         lun, luna, lunb, mosnum, n, naxes(2), naxis, nchan, &
         nchanrin, nchanrout, n_real, n_sims, nord, obsid, &
         obsidf, out(2), pcount, plc_al(2), plc_alsi(2), &
         plc_aua(2), plc_aub(2), plc_auc(2), plc_h1(2), &
         plc_h2(2), plc_h3(2), plc_l3h(2), plc_hi(2), plc_lo(2), &
         qual(2400), revl, revlt, sim_chips(7), tfields, varidat
    integer, dimension(2) :: band = (/0, 0/)
    integer :: err = 0
    integer :: ipix = 1
    integer :: nels = 13
    integer :: pflag = 1
    integer :: smo_fact = 4
    integer :: status = 0
    integer, dimension(13) :: mine = (/118, 379, 460, 766, 1108, &
         1213, 1318, 1423, 1513, 1633, 1753, 1993, 2359/)
    integer, dimension(13) :: maxe = (/237, 417, 765,1050, 1155, &
         1245, 1380, 1470, 1575, 1680, 1875, 2205, 2397/)

    real area, aug_expo, aug_rate, aug_hrate, aug_spec(2400), &
         avtot, back, bandl, bandh, coef(10), c_val(3), &
         detm(2400,2400), emax(2400), emaxin(2400), emin(2400), &
         eminin(2400), ener(2400), fc1(2400), expo, fc_area(7), &
         fc_exp(7), fc_spec(2400,7), fc_uncr(2400,7), &
         fillin(2400), hard, high, hrate, image(780,780), &
         lim_area, low, mask(2400), &
         mk1(2400), modl(2400), ntot, obj_back, obj_cnts(2400), &
         obj_expo, oc_area(7), oc_exp_obs(7), oc_mask(2400,7), &
         oc_spec(2400,7), oc_uncr(2400,7), oc1(2400), &
         outim(780,780), rate, reg_area(7), reg_area_obs(7), &
         reg_exp(7), reg_exp_fwc(7), reg_exp_obs(7), &
         reg_mask(2400,7), reg_rati(2400,7), reg_spec(2400,7), &
         reg_uncr(2400,7), rmocfob, rmocfori, rmocftb, &
         soft, spec(2400), stat(2400), &
         t, t1, t2, temp_bot(2400), temp_top(2400), tempe(2400), &
         temps(2400), tempu(2400), tmodcounts, tot, &
         tot1, tot2, uc1(2400), uncr(2400), ymax, ymin

    real, dimension(3) :: aug_scal = (/0.01, 1.0, 0.0/)
    !        real, dimension(3) :: aug_scal = (/1.0, 1.0, 0.0/)
    real :: aug_time = 1.0e6

    double precision cdelt(2), crpix(2), crval(2)

    !   equivalence (can_file,can_fi)
    equivalence (db_file,db_fi)
    equivalence (qpb_file,qpb_fi)
    equivalence (xmmdir,xmmd)

    prefix = stringParameter("prefix")
    xmmdir = stringParameter("caldb")
    diag = intParameter("diag")
    el = intParameter("elow")
    eh = intParameter("ehigh")
    ccd1 = intParameter("ccd1")
    ccd2 = intParameter("ccd2")
    ccd3 = intParameter("ccd3")
    ccd4 = intParameter("ccd4")
    ccd5 = intParameter("ccd5")
    ccd6 = intParameter("ccd6")
    ccd7 = intParameter("ccd7")
    chisqfl = booleanParameter("chisqflag")
    !    canflag = booleanParameter("canflag") SAS-18
    clobber = booleanParameter("clobber")

    !  Zero a few arrays

    errr = 0
    do i=1,2400
       fc1(i) = 0.0
       fillin(i) = 0.0
       mask(i) = 0.0
       mk1(i) = 0.0
       modl(i) = 0.0
       oc1(i) = 0.0
       spec(i) = 0.0
       uc1(i) = 0.0
       uncr(i) = 0.0
    enddo
    do i=1,780
       do ii=1,780
          outim(i,ii) = 0.0
       enddo
    enddo

    !  Get the prefix and write out some file names

    read(prefix,2989) mosnum
2989 format(i1)
    write(obj_file,2990) prefix
2990 format('mos',a5,'-obj.pi')
    write(obj_rmf,2991) prefix
2991 format('mos',a5,'.rmf')
    write(spec_file,2992) prefix
2992 format('mos',a5,'-spec.qdp')

    ccd(1) = ccd1
    ccd(2) = ccd2
    ccd(3) = ccd3
    ccd(4) = ccd4
    ccd(5) = ccd5
    ccd(6) = ccd6
    ccd(7) = ccd7

    print *, 'xmmdir: ', xmmdir
    print *, 'diag: ', diag
    print *, 'prefix: ', prefix
    print *, 'mosnum: ', mosnum
    print *, 'obj_file: ', obj_file
    print *, 'obj_rmf: ', obj_rmf
    print *, 'el, eh: ', el, eh
    print *, 'CCD selection', ccd

    lim_area = 0.0

    print *, 'WARNING: if chip 3, 4 or 6 are missing spurious '
    print *, 'gold features can arise in the 2.04-2.33, '
    print *, '9.38-9.98, 11.03-11.82 keV regions'

    !  Check to see whether MOS1 CCD#6 is missing

    if(mosnum .eq. 1) then
       call read_pha (obj_file,2400,nchan,channel,obj_cnts,stat, &
            qual,grup,obj_expo,obj_back,area,revl,obsid,mode,err)
       if(revl .gt. 961) then
          print *, 'Observation took place after MOS1 CCD6 ', &
               'was lost'
          if(ccd(6) .eq. 1) then
             print *, 'Set ccd(6) equal to 0'
             ccd(6) = 0
             print *, 'Revised CCD selection '
             print *, 'CCD selection', ccd
          endif
       elseif (revl .eq. 961) then
          print *, 'Observation took place during the ', &
               'orbit where MOS1 CCD6 was lost.  '
          print *, 'Exposure U002 has CCD6 data, U003 does not'
       endif
       if(revl .gt. 2382) then
          print *, 'Observation took place after MOS1 CCD3 ', &
               'was lost'
          if(ccd(3) .eq. 1) then
             print *, 'Set ccd(3) equal to 0'
             ccd(3) = 0
             print *, 'Revised CCD selection '
             print *, 'CCD selection', ccd
          endif
       endif
    endif
    print *, 'Revolution', revl

    !  Do some setup
    !  This sets the outer CCDs which will be used as cognates
    !  for the central CCD

    n = 0
    if(mosnum .eq. 1) then
       if(ccd(2) .eq. 1) then
          n = n + 1
          sim_chips(n) = 2
       endif
       if(ccd(3) .eq. 1) then
          n = n + 1
          sim_chips(n) = 3
       endif
       if(ccd(4) .eq. 1) then
          if(revl .lt. 2383) then
             n = n + 1
             sim_chips(n) = 4
          endif
       endif
       if(ccd(5) .eq. 1) then
          n = n + 1
          sim_chips(n) = 5
       endif
       if(ccd(6) .eq. 1) then
          n = n + 1
          sim_chips(n) = 6
       endif
       if(ccd(7) .eq. 1) then
          n = n + 1
          sim_chips(n) = 7
       endif
       n_sims = n
       mosnam = '1'
       n = n + 1
       do while (n .le. 7)
          sim_chips(n) = 0
          n = n + 1
       enddo
    elseif(mosnum .eq. 2) then
       if(ccd(2) .eq. 1) then
          n = n + 1
          sim_chips(n) = 2
       endif
       if(ccd(3) .eq. 1) then
          n = n + 1
          sim_chips(n) = 3
       endif
       if(ccd(4) .eq. 1) then
          n = n + 1
          sim_chips(n) = 4
       endif
       if(ccd(5) .eq. 1) then
          n = n + 1
          sim_chips(n) = 5
       endif
       if(ccd(6) .eq. 1) then
          n = n + 1
          sim_chips(n) = 6
       endif
       if(ccd(7) .eq. 1) then
          n = n + 1
          sim_chips(n) = 7
       endif
       n_sims = n
       mosnam = '2'
       n = n + 1
       do while (n .le. 7)
          sim_chips(n) = 0
          n = n + 1
       enddo
    else
       print *, 'Illegal mos number: ', mosnum
       stop
    endif

    !  Get information about the object spectrum

    call read_pha (obj_file,2400,nchan,channel,obj_cnts,stat,qual, &
         grup,obj_expo,obj_back,area,revl,obsid,mode,err)

    !  Get the channel to energy conversion

    call read_rmf (obj_rmf,2400,detm,chan,eminin,emaxin,nchanrin, &
         emin,emax,nchanrout)
    do ichan=1,nchan
       ener(ichan) = (emin(ichan) + emax(ichan))/2.
    enddo

    bandl = el/1000.0
    bandh = eh/1000.0
    print *, 'Band range in keV: ', bandl, bandh

    do i=1,nchan-1

       !  Set channel ranges for some energy bands

       if((ener(i) .lt. 0.3) .and. (ener(i+1) .ge. 0.3)) &
            plc_al(1) = i + 1
       if((ener(i) .le. 10.0) .and. (ener(i+1) .gt. 10.0)) &
            plc_al(2) = i
       if((ener(i) .lt. 0.4) .and. (ener(i+1) .ge. 0.4)) &
            plc_lo(1) = i + 1
       if((ener(i) .lt. 0.8) .and. (ener(i+1) .ge. 0.8)) &
            plc_lo(2) = i
       if((ener(i) .lt. 2.5) .and. (ener(i+1) .ge. 2.5)) &
            plc_hi(1) = i + 1
       if((ener(i) .lt. 5.0) .and. (ener(i+1) .ge. 5.0)) &
            plc_hi(2) = i
       if((ener(i) .lt. 2.2) .and. (ener(i+1) .ge. 2.2)) &
            plc_h1(1) = i + 1
       if((ener(i) .lt. 5.3) .and. (ener(i+1) .ge. 5.3)) &
            plc_h1(2) = i
       if((ener(i) .lt. 6.5) .and. (ener(i+1) .ge. 6.5)) &
            plc_h2(1) = i + 1
       if((ener(i) .lt. 9.6) .and. (ener(i+1) .ge. 9.6)) &
            plc_h2(2) = i
       if((ener(i) .lt. 9.9) .and. (ener(i+1) .ge. 9.9)) &
            plc_h3(1) = i + 1
       if((ener(i) .lt. 11.1) .and. (ener(i+1) .ge. 11.1)) &
            plc_h3(2) = i

       !  Set channel ranges for the gold contamination

       if((ener(i) .lt. 2.04) .and. (ener(i+1) .ge. 2.04)) &
            plc_aua(1) = i + 1
       if((ener(i) .le. 2.33) .and. (ener(i+1) .gt. 2.33)) &
            plc_aua(2) = i
       if((ener(i) .lt. 9.38) .and. (ener(i+1) .ge. 9.38)) &
            plc_aub(1) = i + 1
       if((ener(i) .le. 9.98) .and. (ener(i+1) .gt. 9.98)) &
            plc_aub(2) = i
       if((ener(i) .lt. 11.03) .and. (ener(i+1) .ge. 11.03)) &
            plc_auc(1) = i + 1
       if((ener(i) .le. 11.82) .and. (ener(i+1) .gt. 11.82)) &
            plc_auc(2) = i

       !  Set channel ranges for the Al and Si Kalpha contamination

       if((ener(i) .lt. 1.2) .and. (ener(i+1) .ge. 1.2)) &
            plc_alsi(1) = i + 1
       if((ener(i) .le. 1.9) .and. (ener(i+1) .gt. 1.9)) &
            plc_alsi(2) = i

       !  Set channel ranges for the energy band

       if((bandl .eq. 0.0) .and. (bandh .eq. 0.0)) then
          if((ener(i) .lt. 0.3) .and. (ener(i+1) .ge. 0.3)) &
               band(1) = i + 1
          if((ener(i) .le. 8.0) .and. (ener(i+1) .gt. 8.0)) &
               band(2) = i
       else
          if((ener(i) .lt. bandl) .and. (ener(i+1) .ge. bandl)) &
               band(1) = i + 1
          if((ener(i) .le. bandh) .and. (ener(i+1) .gt. bandh)) &
               band(2) = i
       endif

       !  Set channel ranges for the out

       if((ener(i) .lt. 1.0) .and. (ener(i+1) .ge. 1.0)) &
            out(1) = i + 1
       if((ener(i) .le. 2.5) .and. (ener(i+1) .gt. 2.5)) &
            out(2) = i
    enddo
    print *, 'Band channel rage: ', band(1), band(2)

    !  Read the observation chip spectra to get the exposure

    !  The obj data are the observation data

    print *, ' '
    print *, 'Process the observation data'
    if(diag .ge. 2) print 4000
4000 format(1x,' Bins   Expose   Back   Area   Rev  Stat')
    do i=1,7
       if(ccd(i) .eq. 1) then
          write(spec_name,3000) prefix, i
3000      format('mos',a5,'-',i1,'obj.pi')
          call read_pha (spec_name,2400,nchan,channel,spec,uncr, &
               qual,grup,expo,back,area,revl,obsid,mode,err)
          if(diag .ge. 1) print *, spec_name
          if(diag .ge. 2) print 4005, nchan,expo,back/1440000.0, &
               area,revl,err
4005      format(1x,i5,f9.1,2f7.1,2i6)

          !  If err = 0 then the file exists

          if(err .eq. 0) then
             reg_exp_obs(i) = expo
             reg_area_obs(i) = back
          else
             errr=1
             print *,'Missing file: ', spec_name
             print *,'     in XMM_BACK, noted on return from ', &
                  'READ_PHA'
             reg_exp_obs(i) = 0.0
             reg_area_obs(i) = 0.0
          endif
       endif
    enddo
    print *, 'Backscale and exposure from the observation data'
    do i=1,7
       if(ccd(i) .eq. 1) then
          print 4010, i, reg_exp_obs(i), reg_area_obs(i)/1440000.0 
4010      format(1x,'Chip ',i1,'  exposure =',f12.1, &
               '  backscale =',2f7.1)
       endif
    enddo

    !  Read the fwc spectra from the object region
    !  Figure out which chips are involved
    !  Use reg_area as a vector describing which chips are in use

    !  The ff data are the FWC data from the object region

    print *, ' '
    print *, 'Process the FWC data from the object region'
    if(diag .ge. 2) print 4000
    do i=1,7
       if(ccd(i) .eq. 1) then
          write(spec_name,3005) prefix, i
3005      format('mos',a5,'-',i1,'ff.pi')
          call read_pha (spec_name,2400,nchan,channel,spec,uncr, &
               qual,grup,expo,back,area,revlt,obsidf,mode,err)
          if(diag .ge. 1) print *, spec_name
          if(diag .ge. 2) print 4005, nchan,expo,back/1440000.0, &
               area,revlt,err
          reg_exp_fwc(i) = expo

          !  If err = 0 then the file exists

          if(err .eq. 0) then
             if((back .ne. 0) .and. (expo .ne. 0)) then

                !  There is active area on this chip, normalize the spectrum

                do ichan=1,nchan
                   reg_spec(ichan,i) = spec(ichan)/back/expo
                   reg_uncr(ichan,i) = uncr(ichan)/back/expo
                enddo
                reg_area(i) = back
             else
                do ichan=1,nchan
                   reg_spec(ichan,i) = 0.0
                   reg_uncr(ichan,i) = 0.0
                enddo
                reg_area(i) = 0.0
             endif
          else
             do ichan=1,nchan
                reg_spec(ichan,i) = 0.0
                reg_uncr(ichan,i) = 0.0
             enddo
             reg_area(i) = 0.0
          endif
       endif
    enddo
    print *, 'Backscale and exposure from the FWC data'
    do i=1,7
       if(ccd(i) .eq. 1) then
          print 4010, i, reg_exp_fwc(i), reg_area(i)/1440000.0 
       endif
    enddo

    !  Remove any chip that has less area than the limiting area, lim_area

    do i=1,7
       if(ccd(i) .eq. 1) then
          if((reg_area(i) .lt. lim_area) .or. &
               (reg_area(i) .eq. 0.0) .or. &
               (reg_area_obs(i) .eq. 0.0)) then
             reg_area(i) = 0.0
             do ichan=1,nchan
                reg_spec(ichan,i) = 0.0
                reg_uncr(ichan,i) = 0.0
             enddo
          endif
       endif
    enddo

    !  Read corner spectra from the observation
    !  First set up the spectrum masks

    !  Testing here to see if we really do need to exclude the corner 
    !  data from chips 2,7 because of Au issues

    do i=1,7
       if(ccd(i) .eq. 1) then
          do ichan=1,nchan
             oc_mask(ichan,i) = 1
          enddo
       endif
    enddo

    !  Even if one of the outer chips is not being used
    !  we still need the corner data if chip 1 is on

    !  The oc data are observation data from the corners

    print *, ' '
    print *, 'Process the observation data from the corner regions'
    do i=2,7
       if(ccd(i) .eq. 1) then
          write(spec_name,3010) prefix, i
3010      format('mos',a5,'-',i1,'oc.pi')

          call read_pha (spec_name,2400,nchan,channel,spec,uncr, &
               qual,grup,expo,back,area,revl,obsid,mode,err)
          if(diag .ge. 1) print *, spec_name
          if(diag .ge. 2) print *, nchan,expo,back,area,revl,err
          if(err .ne. 1) then
             oc_exp_obs(i) = expo
             hrate = 0.0
             rate = 0.0
             soft = 0.0
             hard = 0.0
             high = 0.0
             low = 0.0
             do ichan=plc_al(1),plc_al(2)
                rate = rate + spec(ichan)
             enddo
             rate = rate/expo
             do ichan=plc_h1(1),plc_h1(2)
                hrate = hrate + spec(ichan)
             enddo
             do ichan=plc_h2(1),plc_h2(2)
                hrate = hrate + spec(ichan)
             enddo
             do ichan=plc_h3(1),plc_h3(2)
                hrate = hrate + spec(ichan)
             enddo
             hrate = hrate/expo

             !  Scale the MOS1 CCD#4 count rate if needed

             if((mosnum .eq. 1) .and. (i .eq. 4) .and. &
                  (revl .ge. 2383)) then
                rmocftb = 32.2
                rmocfob = back/1440000.0
                rmocfori = rate
                rate = rate*rmocftb/rmocfob
             endif

             do ichan=plc_hi(1),plc_hi(2)
                high = high + spec(ichan)
             enddo
             do ichan=plc_lo(1),plc_lo(2)
                low = low + spec(ichan)
             enddo
             hard = high/low

             ! Check whether a canned spectrum can be used

             icanflag = 1
             if((mosnum .eq. 1) .and. (i .eq. 4)) then
                if(hard .lt. 3.4) icanflag = 0
                if(hard .lt. 2.0) icanflag = -1
             endif
             if((mosnum .eq. 1) .and. (i .eq. 5)) then
                if(hard .lt. 3.0) icanflag = 0
                if(hard .lt. 2.0) icanflag = -1
             endif
             if((mosnum .eq. 2) .and. (i .eq. 2)) then
                if(hard .lt. 2.5) icanflag = 0
                if(hard .lt. 0.0) icanflag = -1
             endif
             if((mosnum .eq. 2) .and. (i .eq. 5)) then
                if(hard .lt. 3.5) icanflag = 0
                if(hard .lt. 3.0) icanflag = -1
             endif
             if(hard .lt. 1.0) icanflag = -1
             print *, 'mosnum, i, hard, icanflag: ', &
                  mosnum, i, hard, icanflag

             canflag = .FALSE. ! until SAS-18
             if((canflag) .and. (icanflag .eq. 1)) then
                !
                !                !  Use the canned spectrum if possible
                !                !  Set the CalDB file name
                !
                !                ii = 1
                !                do while (xmmd(ii) .ne. ' ')
                !                   db_fi(ii) = xmmd(ii)
                !                   ii = ii + 1
                !                enddo
                !                do iii=ii,132
                !                   db_fi(iii) = ' '
                !                enddo
                !                print *,'CalDB Directory: ', db_fi
                !                print *,'can_fi ', can_fi
                !                do iii=0,20
                !                   db_fi(ii+iii) = can_fi(iii+1)
                !                enddo
                !                print *,'CalDB Canned File: ', db_fi
                !
                !                !  Get an unused lun for the caldb file
                !
                !                err = 0
                !                status = 0
                !                call ftgiou(lun,status)
                !                if((status .ne. 0) .and. (err .eq. 0)) then
                !                   err = 1
                !                   print *, 'AUG_GET_EXP POSITION 02'
                !                   print *, 'STATUS FTGIOU =', status
                !                endif
                !
                !                !  Open the FITS file
                !
                !                call ftopen (lun,db_file,0,blockn,status)
                !                if((status .ne. 0) .and. (err .eq. 0)) then
                !                   err = 1
                !                   print *, 'AUG_GET_EXP POSITION 03'
                !                   print *, 'STATUS FTOPEN =', status
                !                   print *, db_file
                !                endif
                !
                !                !  Move to the correct extension
                !
                !                if(err .eq. 0) then
                !                   call ftmahd (lun,2,hdutype,status)
                !                   if((status .ne. 0) .and. (err .eq. 0)) then
                !                      err = 1
                !                      print *, 'AUG_GET_EXP POSITION 04'
                !                      print *, 'STATUS FTMAHD =', status
                !                      print *, db_file
                !                      print *, 'Could not find the spectrum ', &
                !                           'extension'
                !                      stop
                !                   else
                !
                !                      !  Read the FITS file header information
                !
                !                      call ftghbn (lun,23,nchan,tfields,ttype, &
                !                           tform,tunit,extname,varidat,status)
                !                      !                              print *, lun,nchan,tfields,ttype, &
                !                      !                                     tform,tunit,extname,varidat,status
                !                      if(extname .ne. 'SPECTRA') then
                !                         print *, 'AUG_GET_EXP POSITION 05'
                !                         print *, extname
                !                         print *, 'This was supposed to be ', &
                !                              'SPECTRA'
                !                         stop
                !                      endif
                !                      if(status .ne. 0) then
                !                         err = 1
                !                         print *, 'AUG_GET_EXP POSITION 06'
                !                         print *, 'STATUS FTGHPR =', status
                !                         print *, 'Failure on reading the ', &
                !                              'header information'
                !                         stop
                !                      else
                !
                !                         !  Read in the CCD data
                !
                !                         irow = (mosnum - 1)*6 - 1 + i
                !                         call ftgcvs(lun,1,irow,1,1,'    ', &
                !                              detname,anyf,status)
                !                         call ftgcvj(lun,2,irow,1,1,0, &
                !                              ccdnu,anyf,status)
                !                         call ftgcve(lun,3,irow,1,1,0, &
                !                              expo,anyf,status)
                !                         call ftgcve(lun,4,irow,1,1,0, &
                !                              back,anyf,status)
                !                         call ftgcve(lun,5,irow,1,2400,0, &
                !                              spec,anyf,status)
                !                         nchan = 2400
                !                         !                                  print *, spec
                !                         print *, irow, detname, ccdnu, expo, &
                !                              back/1440000.0, status
                !                      endif
                !                   endif
                !                endif
                !                tot1 = 0.0
                !                do ii=1,2400
                !                   tot1 = tot1 + spec(ii)
                !                enddo
                !                print *, 'Total Counts in Spectrum: ', tot1
                !                call ftclos(lun,status)
             elseif (icanflag .ge. 0) then

                !  CCD in anomolous state, augment the observation spectrum
                !  Set the CalDB file name

                ii = 1
                do while (xmmd(ii) .ne. ' ')
                   db_fi(ii) = xmmd(ii)
                   ii = ii + 1
                enddo
                do iii=ii,132
                   db_fi(iii) = ' '
                enddo
                print *,'CalDB Directory: ', db_fi
                print *,'mosnam ', mosnam
                print *,'qpb_fi ', qpb_fi
                qpb_fi(5) = mosnam
                do iii=0,20
                   db_fi(ii+iii) = qpb_fi(iii+1)
                enddo
                print *,'CalDB QPB File: ', db_fi

                c_val(1) = rate
                c_val(2) = hard
                c_val(3) = revl
                print *, 'CalDB File: ', db_file
                call aug_get_exp(db_file,c_val,aug_scal, &
                     aug_time,pflag,aug_spec,aug_expo, &
                     mosnum,i,obsid,prefix)
                if((mosnum .eq. 1) .and. (i .eq. 4) .and. &
                     (revl .ge. 2383)) then
                   rate = rmocfori
                endif

                do ichan=1,nchan
                   spec(ichan) = spec(ichan) + aug_spec(ichan)
                enddo
                expo = expo + aug_expo
                print *, expo, back/1440000.0
             else
                print 2000, i
2000            format(' CCD #',i1,' strongly anomalous, ' &
                     'reprocess without it')
                print *, ' Hardness Ratio: ', hard
                !                        stop 
             endif

             !  Normalize the spectrum

             aug_rate = 0.0
             do ichan=plc_al(1),plc_al(2)
                aug_rate = aug_rate + spec(ichan)
                !                        print *, ichan, aug_rate, spec(ichan)
             enddo
             print *, 'rate, aug_rate, expo:', &
                  rate, aug_rate, expo
             aug_rate = aug_rate/expo

             aug_hrate = 0.0
             do ichan=plc_h1(1),plc_h1(2)
                aug_hrate = aug_hrate + spec(ichan)
             enddo
             do ichan=plc_h2(1),plc_h2(2)
                aug_hrate = aug_hrate + spec(ichan)
             enddo
             do ichan=plc_h3(1),plc_h3(2)
                aug_hrate = aug_hrate + spec(ichan)
             enddo
             aug_hrate = aug_hrate/expo

             print *, 'rate, aug_rate, hrate, aug_hrate: ', &
                  rate, aug_rate, hrate, aug_hrate
             print *, 'rate/aug_rate, hrate/aug_hrate: ', &
                  rate/aug_rate, hrate/aug_hrate

             !                    rate = hrate
             !                    aug_rate = aug_hrate

             do ichan=1,nchan
                spec(ichan) = spec(ichan)*rate/aug_rate
             enddo
             tot1 = 0.0
             tot2 = 0.0
             do ichan=plc_al(1),plc_al(2)
                tot1 = tot1 + spec(ichan)
             enddo
             print *, 'tot1 aug_rate rate tot1/expo expo:', &
                  tot1, aug_rate, rate, tot1/expo, expo

             tot1 = 0.0
             tot2 = 0.0
             do ichan=1,nchan
                uncr(ichan) = sqrt(spec(ichan))
                tot1 = tot1 + spec(ichan)
                oc_spec(ichan,i) = spec(ichan)/back/expo
                tot2 = tot2 + oc_spec(ichan,i)
                oc_uncr(ichan,i) = uncr(ichan)/back/expo
             enddo
             print *, i, back, expo, rate, aug_rate, tot1, tot2
             oc_area(i) = back
             reg_exp(i) = expo
          else
             do ichan=1,nchan
                oc_spec(ichan,i) = 0.0
                oc_uncr(ichan,i) = 0.0
             enddo
             oc_area(i) = 0.0
             if(reg_area(i) .ne.  0.0) then
                errr = 1
                do ichan=1,nchan
                   reg_spec(ichan,i) = 0.0
                   reg_uncr(ichan,i) = 0.0
                enddo
                reg_area(i) = 0.0
                print *, 'missing file: ',spec_name

                !  Will not kill but will not return, we will try to recover

             endif
          endif
       endif
    enddo
    print *, ' '
    print *, 'Augmented exposure and backscale from the ', &
         'corners of the observation data'
    do i=2,7
       if(ccd(i) .eq. 1) then
          print 4010, i, reg_exp(i), oc_area(i)/1440000.0 
       endif
    enddo

    !  Do the center chip if necessary

    if((reg_area(1) .ne. 0.0) .and. (ccd(1) .eq. 1)) then

       !  The center chip is active
       !    create a mask for normalizing the center chips spectrum
       !    determine how many similar chips have data

       print *, 'Process the observation data from the center chip'
       n_real = n_sims
       do i=1,n_sims
          if(oc_area(sim_chips(i)) .eq. 0.0) then
             n_real = n_real - 1
             sim_chips(i) = 0
          endif
       enddo

       if(n_real .eq. 0) then

          !  This is really an error from which we can not recover

          errr = 2
          print *, 'No corner data for chip 1 model'
          stop
       else
          n_sims = n_real
          it = 0
          do i=1,4
             if(sim_chips(i) .ne. 0) then
                it = it + 1
                sim_chips(it) = sim_chips(i)
             endif
          enddo
       endif
       print *, 'N_SIMS: ', n_sims
       print *, (sim_chips(i),i=1,n_sims)

       !  Sum over the similar chips

       do i=1,n_sims
          do ichan=1,nchan
             oc1(ichan) = oc1(ichan) + &
                  oc_spec(ichan,sim_chips(i))* &
                  oc_mask(ichan,sim_chips(i))
             uc1(ichan) = uc1(ichan) + &
                  (oc_uncr(ichan,sim_chips(i))**2)* &
                  oc_mask(ichan,sim_chips(i))
             mk1(ichan) = mk1(ichan) + &
                  oc_mask(ichan,sim_chips(i))
          enddo
       enddo

       !  Check to see whether there are any data

       itest = 0
       do ichan=1,nchan
          if(mk1(ichan) .ne. 0) itest = 1
       enddo
       if(itest .eq. 0)then
          errr = 2
          print *, 'No data to create a spectrum for chip 1'
          stop
       endif

       do ichan=1,nchan
          if(mk1(ichan) .ne. 0.0) then
             oc1(ichan) = oc1(ichan)/mk1(ichan)
             uc1(ichan) = sqrt(uc1(ichan))/mk1(ichan)
          else
             oc1(ichan) = 0.0
             uc1(ichan) = 0.0
          endif
          oc_spec(ichan,1) = oc1(ichan)
          oc_uncr(ichan,1) = uc1(ichan)
          !                print *, oc_spec(ichan,1)
       enddo
       do i=1,7
          if(ccd(i) .eq. 1) then
             tot = 0.0
             do ichan=band(1),band(2)
                tot = tot + oc_spec(ichan,i)
                !                        if(i .eq. 1) print *, ichan, band(1), band(2), &
                !                                tot, oc_spec(ichan,i)
             enddo
             print *, 'Chip',i,' observation corner total: ', tot
          endif
       enddo
    endif

    !  Read FWC corner spectra

    !  The fc data are the FWC data from the corners

    print *, ' '
    print *, 'Process the FWC corner data'
    if(diag .ge. 2) print 4000
    do i=2,7
       if(ccd(i) .eq. 1) then
          write(spec_name,3030) prefix, i
3030      format('mos',a5,'-',i1,'fc.pi')
          call read_pha (spec_name,2400,nchan,channel,spec,uncr, &
               qual,grup,expo,back,area,revl,obsidf,mode,err)
          if(diag .ge. 1) print *, spec_name
          if(diag .ge. 2) print 4005, nchan,expo,back/1440000.0, &
               area,revl,err
          if(err .ne. 1) then

             !  Normalize the spectrum

             do ichan=1,nchan
                fc_spec(ichan,i) = spec(ichan)/back/expo
                fc_uncr(ichan,i) = uncr(ichan)/back/expo
             enddo
             fc_area(i) = back
             fc_exp(i) = expo
          else
             if((reg_area(i) .ne. 0.0) .or. &
                  (oc_area(i) .ne. 0)) then
                errr=1
                print *, 'missing file: ',spec_name, &
                     ' in mos_back'
                stop
             endif
          endif
       endif
    enddo
    print *, 'Exposure and backscale from the corners ', &
         'of the FWC data'
    do i=2,7
       if(ccd(i) .eq. 1) then
          print 4010, i, fc_exp(i), fc_area(i)/1440000.0
       endif
    enddo

    !  Create a corner spectrum for the center chip

    if(ccd(1) .eq. 1) then
       do i=1,n_sims
          do ichan=1,nchan
             fc1(ichan) = fc1(ichan) + &
                  fc_spec(ichan,sim_chips(i))* &
                  oc_mask(ichan,sim_chips(i))
          enddo
       enddo

       t = 0.0
       do ichan=1,nchan
          if(mk1(ichan) .ne. 0) then
             fc1(ichan) = fc1(ichan)/mk1(ichan)
          endif
          fc_spec(ichan,1) = fc1(ichan)
          t = t + fc_spec(ichan,1)
       enddo
       if(t .eq. 0.0) then
          errr = 2
          print *, 'No counts for the pseudo CCD1 corner spec'
          stop
       endif
    endif

    !  Create the ratio of region/corner for FWC data

    do i=2,7
       if((reg_area(i) .ne. 0.0) .and. (ccd(i) .eq. 1)) then
          do ichan=40,nchan
             i1 = ichan - smo_fact
             if(i1 .lt. 40) i1 = 40
             i2 = ichan + smo_fact
             if(i2 .gt. nchan) i2 = nchan
             t1 = 0.0
             t2 = 0.0
             it = 0
             do ii=i1,i2
                t1 = t1 + reg_spec(ii,i)
                t2 = t2 + fc_spec(ii,i)
                it = it + 1
             enddo
             if(it .gt. 0) then
                temp_top(ichan) = t1/it
                temp_bot(ichan) = t2/it
             else
                temp_top(ichan) = 0.0
                temp_bot(ichan) = 0.0
             endif
          enddo
          do ichan=1,nchan
             if(temp_bot(ichan) .ne. 0) then
                reg_mask(ichan,i) = 1.0
                reg_rati(ichan,i) = &
                     temp_top(ichan)/temp_bot(ichan)
             else
                reg_mask(ichan,i) = 0.0
                reg_rati(ichan,i) = 0.0
             endif
          enddo
       endif
    enddo
    if(diag .ge. 3) then
       print *, 'TEMP_TOP: ', temp_top
       print *, 'TEMP_BOT: ', temp_bot
    endif

    !  Single out chip 1 for special treatment again

    if((reg_area(1) .ne. 0.0) .and. (ccd(1) .eq. 1)) then

       !  Does the spectrum have gold gaps?

       itest = 1
       ites = 0
       do ichan=plc_al(1),plc_al(2)
          !                write(21,*) ichan, oc_spec(ichan,1)
          if(oc_spec(ichan,1) .eq. 0.0) then
             itest = 0
             ites = ites + 1
          endif
       enddo
       if((itest .eq. 1) .or. (ites .lt. 10)) then
          do ichan=40,nchan
             i1 = ichan - smo_fact
             if(i1 .lt. 40) i1 = 40
             i2 = ichan + smo_fact
             if(i2 .gt. nchan) i2 = nchan
             t1 = 0.0
             t2 = 0.0
             it = 0
             do ii=i1,i2
                t1 = t1 + reg_spec(ii,1)
                t2 = t2 + fc_spec(ii,1)
                it = it + 1
             enddo
             if(it .gt. 0) then
                temp_top(ichan) = t1/it
                temp_bot(ichan) = t2/it
             else
                temp_top(ichan) = 0.0
                temp_bot(ichan) = 0.0
             endif
          enddo
          do ichan=1,nchan
             if(temp_bot(ichan) .ne. 0) then
                reg_mask(ichan,1) = 1.0
                reg_rati(ichan,1) = &
                     temp_top(ichan)/temp_bot(ichan)
             else
                reg_mask(ichan,1) = 0.0
                reg_rati(ichan,1) = 0.0
             endif
          enddo
       else
          print 2994, (oc_spec(ichan,1),ichan=1,2400)
2994      format(8(e10.3))
          errr = 3
          print *, itest, ites, plc_al(1), plc_al(2)
          print *, 'You are scrod, your chip 1 background ', &
               'has gold gaps - no CCD#3 or CCD#7 data'
          print *, itest, ites, plc_al(1), plc_al(2)
          stop
       endif
    endif

    !  Print out some diagnostic information

    if(diag .ge. 1) then
       print *, 'Backscale for the FWC corner ', &
            'and FWC region data'
       do i=1,7
          print 4015, i, fc_area(i)/1440000.0,  &
               reg_area(i)/1440000.0
4015      format(1x,'Chip ',i1,' backscale =',f7.1, &
               ' backscale =',f7.1)
       enddo
    endif
    t1 = 0.0
    t2 = 0.0
    do i=1,7
       t1 = t1 + fc_area(i)
       t2 = t2 + reg_area(i)
    enddo
    if(diag .ge. 1) then
       print *, 'FC_AREA, REG_AREA, RATIO: ', t1, t2, t1/(t1 + t2)
    endif

    !  Create the masks to remove strong instrumental lines
    !  Remove the Al-Si line from all spectra

    do ichan=plc_alsi(1),plc_alsi(2)
       do i=1,7
          oc_mask(ichan,i) = 0
       enddo
    enddo

    !  Sum the background over all active chips and make an image

    if(diag .ge. 2) then
       write(fname,3032) prefix
3032   format('mos',a5,'-ratio-indiv.qdp')
       open(86,form='formatted',status='unknown',file=fname)
       write(86,*) 'viewport 0.2 0.2 0.95 0.9'
       write(86,*) 'font roman'
       write(86,*) 'cs 1.5'
       write(86,*) 'log y'
       write(86,*) 'r y 1.0 1.e8'
       write(86,*) 'la y Ratio'
       write(86,*) 'la x Channel'

       write(fname,3033) prefix
3033   format('mos',a5,'-aug-indiv.qdp')
       open(87,form='formatted',status='unknown',file=fname)
       write(87,*) 'viewport 0.2 0.2 0.95 0.9'
       write(87,*) 'read serr 2'
       write(87,*) 'font roman'
       write(87,*) 'cs 1.5'
       write(87,*) 'log y'
       write(87,*) 'r y 1.e-2 1.e6'
       write(87,*) 'la y Scaled Counts'
       write(87,*) 'la x Channel'

       write(fname,3034) prefix
3034   format('mos',a5,'-back-indiv.qdp')
       open(88,form='formatted',status='unknown',file=fname)
       write(88,*) 'viewport 0.2 0.2 0.95 0.9'
       write(88,*) 'read serr 2'
       write(88,*) 'font roman'
       write(88,*) 'cs 1.5'
       write(88,*) 'log y'
       write(88,*) 'r y 0.005 0.5e7'
       write(88,*) 'la y Scaled Counts'
       write(88,*) 'la x Channel'

       write(fname,3035) prefix
3035   format('mos',a5,'-back-accum.qdp')
       open(89,form='formatted',status='unknown',file=fname)
       write(89,*) 'viewport 0.2 0.2 0.95 0.9'
       write(89,*) 'read serr 2'
       write(89,*) 'font roman'
       write(89,*) 'cs 1.5'
       write(89,*) 'log y'
       write(89,*) 'r y 1.e-2 1.e6'
       write(89,*) 'la y Scaled Counts'
       write(89,*) 'la x Channel'
    endif
    do ichan=1,2400
       spec(ichan) = 0.0
       uncr(ichan) = 0.0
    enddo
    do i=1,7
       if((reg_area(i) .ne. 0) .and. (ccd(i) .eq. 1)) then
          print 2995, i
2995      format('Add in CCD#',i1,' background spectrum')
          do ichan=1,nchan
             if((reg_mask(ichan,i) .ne. 0.0) .and. &
                  (oc_mask(ichan,i) .ne. 0.0)) then
                spec(ichan) = spec(ichan) + oc_spec(ichan,i)* &
                     reg_rati(ichan,i)*reg_area(i)
                uncr(ichan) = uncr(ichan) + (oc_uncr(ichan,i)* &
                     reg_rati(ichan,i)*reg_area(i))**2
                mask(ichan) = mask(ichan) + reg_area(i)
             endif
             if((diag .ge. 2) .and. (mask(ichan) .gt. 0.0)) then
                write(86,*) ichan, &
                     (10.0**i)*reg_rati(ichan,i)
                write(87,*) ichan, &
                     1.e10*(10.0**i)*oc_spec(ichan,i), &
                     1.e10*(10.0**i)*oc_uncr(ichan,i)
                write(88,*) ichan, &
                     1.e3*(10.0**i)*oc_spec(ichan,i)* &
                     reg_rati(ichan,i)*reg_area(i), &
                     1.e3*(10.0**i)*sqrt((oc_uncr(ichan,i)* &
                     reg_rati(ichan,i)*reg_area(i))**2)
                write(89,*) ichan, &
                     1.e10*(10.0**i)* &
                     spec(ichan)/mask(ichan), &
                     1.e10*(10.0**i)* &
                     sqrt(uncr(ichan))/mask(ichan)
             endif
          enddo
          tmodcounts = 0.0
          tot1 = 0.0
          tot2 = 0.0
          do ichan=band(1),band(2)
             tmodcounts = tmodcounts + oc_spec(ichan,i)* &
                  reg_rati(ichan,i)*reg_area_obs(i)* &
                  reg_exp_obs(i)
             tot1 = tot1 + oc_spec(ichan,i)
             tot2 = tot2 + reg_rati(ichan,i)
          enddo
          if(diag .ge. 2) then
             print *, i, tot1, tot2, reg_area(i), &
                  reg_area_obs(i), reg_exp_obs(i)
          endif

          !  Read in the fwc image

          if(eh .gt. 0) then
             if(el .ge. 10000) then
                write(im_name,3040) prefix, i, el, eh
3040            format('mos',a5,'-im',i1,'-',i5,'-',i5,'.fits')
             elseif(el .ge. 1000) then
                if(eh .ge. 10000) then
                   write(im_name,3041) prefix, i, el, eh
3041               format('mos',a5,'-im',i1,'-',i4,'-', &
                        i5,'.fits')
                else
                   write(im_name,3042) prefix, i, el, eh
3042               format('mos',a5,'-im',i1,'-',i4,'-', &
                        i4,'.fits')
                endif
             else
                if(eh .ge. 10000) then
                   write(im_name,3043) prefix, i, el, eh
3043               format('mos',a5,'-im',i1,'-',i3,'-', &
                        i5,'.fits')
                elseif(eh .ge. 1000) then
                   write(im_name,3044) prefix, i, el, eh
3044               format('mos',a5,'-im',i1,'-',i3,'-', &
                        i4,'.fits')
                else
                   write(im_name,3045) prefix, i, el, eh
3045               format('mos',a5,'-im',i1,'-',i3,'-', &
                        i3,'.fits')
                endif
             endif
             print *, im_name
             call retfit(im_name,image,ini4, &
                  1,780,780,ctype,crval,cdelt,crpix,status)
             if(status .eq. 0) then

                !  Check for bright pixels


                tot = 0.0
                ntot = 0.0
                do j=1,780
                   do jj=1,780
                      if(image(j,jj) .gt. 0) then
                         tot = tot + image(j,jj)
                         ntot = ntot + 1.0
                      endif
                   enddo
                enddo
                if(ntot .gt. 0.0) then
                   avtot = tot/ntot
                   print *, 'Average non-zero value of FWC ', &
                        'image: ', avtot
                   tot = 5.0*avtot
                   do j=1,780
                      do jj=1,780
                         if(image(j,jj) .gt. tot) then
                            image(j,jj) = avtot
                         endif
                      enddo
                   enddo

                   tot = 0.0
                   do j=1,780
                      do jj=1,780
                         tot = tot + image(j,jj)
                      enddo
                   enddo
                   print *, 'FWC image scaling for chip: ', i, &
                        tmodcounts, tot, tmodcounts/tot
                   do j=1,780
                      do jj=1,780
                         outim(j,jj) = outim(j,jj) + &
                              image(j,jj)*tmodcounts/tot
                      enddo
                   enddo
                endif
             endif
          endif
       else
          print 2996, i
2996      format('CCD#',i1,' not included in background spectrum')
       endif
       if(diag .ge. 2) then
          write(86,*) 'no no no'
          write(87,*) 'no no no'
          write(88,*) 'no no no'
          write(89,*) 'no no no'
       endif
    enddo
    if(diag .ge. 2) then
       close(86) 
       close(87) 
       close(88) 
       close(89)
    endif

    if(diag .gt. 1) then
       print *, 'Exposure for the FWC and observation data'
       do i=1,7
          if(ccd(i) .eq. 1) then
             print 4025, i,  reg_exp_fwc(i), reg_exp_obs(i)
4025         format(1x,'Chip ',i1, ' exposure = ',f12.1, &
                  ' exposure =',f9.1)
          endif
       enddo
    endif

    !  Search for and eliminate hot pixels

    if(eh .gt. 0) then
       if(ipix .eq. 1) then
          tot1 = 0.0
          tot2 = 0.0
          do i=1,780
             do ii=1,780
                if(outim(i,ii) .gt. 0.0) then
                   tot1 = tot1 + outim(i,ii)
                   tot2 = tot2 + 1.0
                endif
             enddo
          enddo
          tot1 = 2.0*tot1/tot2
          do i=1,780
             do ii=1,780
                if(outim(i,ii) .gt. tot1) then
                   outim(i,ii) = tot1
                endif
             enddo
          enddo
          tot1 = 0.0
          tot2 = 0.0
       endif
    endif

    !  Write out some more diagnostic information

    if(diag .ge. 2) then
       write(fname,3055) prefix
3055   format('mos',a5,'-back.qdp')
       open(89,form='formatted',status='unknown',file=fname)
       write(89,*) 'viewport 0.2 0.2 0.95 0.9'
       write(89,*) 'read serr 2'
       write(89,*) 'font roman'
       write(89,*) 'cs 1.5'
       write(89,*) 'log y'
       write(89,*) 'la y Model Background Counts'
       write(89,*) 'la x Channel'
    endif
    do ichan=1,nchan
       if(mask(ichan) .ne. 0.0) then
          spec(ichan) = spec(ichan)/mask(ichan)
          uncr(ichan) = sqrt(uncr(ichan))/mask(ichan)
          if(diag .ge. 2) then
             write(89,*) ichan, spec(ichan), uncr(ichan)
          endif
       else
          spec(ichan) = 0.0
          uncr(ichan) = 0.0
       endif
    enddo
    if(diag .ge. 2) then
       close(89)
    endif

    !  Read in the observation image in detector coordinates

    !  First create the file name


    if(eh .gt. 0) then
       if(el .ge. 10000) then
          write(im_name,3060) prefix, el, eh
3060      format('mos',a5,'-obj-im-det-',i5,'-',i5,'.fits')
       elseif(el .ge. 1000) then
          if(eh .ge. 10000) then
             write(im_name,3061) prefix, el, eh
3061         format('mos',a5,'-obj-im-det-',i4,'-',i5,'.fits')
          else
             write(im_name,3062) prefix, el, eh
3062         format('mos',a5,'-obj-im-det-',i4,'-',i4,'.fits')
          endif
       else
          if(eh .ge. 10000) then
             write(im_name,3063) prefix, el, eh
3063         format('mos',a5,'-obj-im-det-',i3,'-',i5,'.fits')
          elseif(eh .ge. 1000) then
             write(im_name,3064) prefix, el, eh
3064         format('mos',a5,'-obj-im-det-',i3,'-',i4,'.fits')
          else
             write(im_name,3065) prefix, el, eh
3065         format('mos',a5,'-obj-im-det-',i3,'-',i3,'.fits')
          endif
       endif

       !  Get an unused lun

       call ftgiou(luna,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'XMM-BACK POSITION 1'
          print *, 'STATUS FTGIOU =', status
       endif

       call retfit(im_name,image,ini4, &
            1,780,780,ctype,crval,cdelt,crpix,status)
       if(status .ne. 0) then
          print *, 'Cant find observation image in detector ', &
               'coords'
          print *, im_name
          stop
       endif

       tot1 = 0.0
       tot2 = 0.0
       do j=1,780
          do jj=1,780
             tot1 = tot1 + image(j,jj)
             tot2 = tot2 + outim(j,jj)
          enddo
       enddo
       print *,'Total observation counts in band ',tot1
       print *,'Total background counts in band ',tot2

       !  Write out the background image

       !  First get an unused lun

       call ftgiou(luna,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'XMM-BACK POSITION 2'
          print *, 'STATUS FTGIOU =', status
       endif
       call ftgiou(lunb,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'XMM-BACK POSITION 3'
          print *, 'STATUS FTGIOU =', status
       endif

       !  Create the output file name

       if(el .ge. 10000) then
          write(imo_name,3080) prefix, el, eh
3080      format('mos',a5,'-back-im-det-',i5,'-',i5,'.fits')
       elseif(el .ge. 1000) then
          if(eh .ge. 10000) then
             write(imo_name,3081) prefix, el, eh
3081         format('mos',a5,'-back-im-det-',i4,'-',i5,'.fits')
          else
             write(imo_name,3082) prefix, el, eh
3082         format('mos',a5,'-back-im-det-',i4,'-',i4,'.fits')
          endif
       else
          if(eh .ge. 10000) then
             write(imo_name,3083) prefix, el, eh
3083         format('mos',a5,'-back-im-det-',i3,'-',i5,'.fits')
          elseif(eh .ge. 1000) then
             write(imo_name,3084) prefix, el, eh
3084         format('mos',a5,'-back-im-det-',i3,'-',i4,'.fits')
          else
             write(imo_name,3085) prefix, el, eh
3085         format('mos',a5,'-back-im-det-',i3,'-',i3,'.fits')
          endif
       endif

       !  Open the old image FITS file

       call ftopen (luna,im_name,0,blockn,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'XMM-BACK POSITION 4a'
          print *, 'STATUS FTOPEN =', status
          print *, im_name
       endif

       !  Check to see if there is an existing file

       call ftopen (lunb,imo_name,0,blockn,status)
       if(status .eq.  0) then
          print *, 'File exists with output file name', imo_name
          if(clobber) then
             print *, 'Clobber the file and continue'
             call ftdelt(lunb,status)
          else
             print *, 'XMM-BACK POSITION 4b'
             print *, 'Will not clobber file'
             stop
          endif
       endif
       status = 0

       !  Create the output FITS file using the original file as a template

       call ftinit (lunb,imo_name,2880,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'XMM-BACK POSITION 5'
          print *, 'STATUS FTINIT =', status
          print *, imo_name
       endif

       call ftcphd (luna,lunb,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'XMM-BACK POSITION 6'
          print *, 'STATUS FTCPHD =', status
       endif

       bitpix = -32
       naxis = 2
       naxes(1) = 780
       naxes(2) = 780
       pcount = 0
       gcount = 1
       group = 1
       call ftpdef (lunb,bitpix,naxis,naxes,pcount, &
            gcount,.FALSE.,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'XMM-BACK POSITION 7'
          print *, 'STATUS FTPDEF =', status
       endif
       call ftmkyj(lunb,'BITPIX',bitpix,'Bitpix',status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'XMM-BACK POSITION 8'
          print *, 'STATUS FTMKYJ =', status
       endif
       if(err .eq. 0) then

          !  Write the output image

          call ftp2de (lunb,group,naxes(1),naxes(1),naxes(2), &
               outim,status)
          if((status .ne. 0) .and. (err .eq. 0)) then
             err = 1
             print *, 'STATUS FTP2DE =', status
             print *, 'XMM-BACK POSITION 9'
             print *, 'Could not write the background image'
          endif

          !  Close the output file

          call ftclos(lunb,status)
          if((status .ne. 0) .and. (err .eq. 0)) then
             err = 1
             print *, 'STATUS FTCLOS =', status
             print *, 'XMM-BACK POSITION 10'
             print *, 'Could not close the output file'
          endif
       endif
       call ftfiou (luna,status)
       call ftfiou (lunb,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTFIOU =', status
          print *, 'XMM-BACK POSITION 11'
          print *, 'Could not close the output file'
       endif
    endif

    !  Bridge the Al-Si gap
    !    Set a number of line-free windows
    !    Extract the data from those windows
    !    Fit a simple polynomial
    !    Use to interpolate over the Al-Si gap

    !  The following two lines need to be converted to energies

    n = 0
    do i=1,nels
       do ichan=mine(i),maxe(i)
          if((ener(ichan) .gt. 0.1) .and. &
               (spec(ichan) .gt. 0.0)) then
             n = n + 1
             tempe(n) = alog(ener(ichan))
             temps(n) = alog(spec(ichan))
             tempu(n) = 0.2
          endif
       enddo
    enddo

    nord = 5
    call poly_fit(tempe,temps,tempu,n,nord,coef)
    if(diag .ge. 2) then
       write(fname,3105) prefix
3105   format('mos',a5,'-bridge-fit.qdp')
       open(89,form='formatted',status='unknown',file=fname)
       write(89,*) 'viewport 0.2 0.2 0.95 0.9'
       write(89,*) 'read serr 2'
       write(89,*) 'font roman'
       write(89,*) 'cs 1.5'
       write(89,*) 'la y Arbitrary'
       write(89,*) 'la x Log(E) (keV)'
    endif
    do ichan=1,nchan
       do i=0,nord
          modl(ichan) = modl(ichan) + &
               coef(i+1)*(alog(ener(ichan))**i)
       enddo
       if((diag .ge. 2) .and. (ener(ichan) .gt. 0.1)) then
          write(89,*) alog(ener(ichan)), alog(spec(ichan)), &
               0.05*(alog(spec(ichan))), modl(ichan)
       endif
    enddo

    if(diag .ge. 2) then
       close(89)
    endif

    !  Finish the bridge

    do ichan = plc_alsi(1),plc_alsi(2)
       fillin(ichan) = exp(modl(ichan))
       spec(ichan) = spec(ichan) + fillin(ichan)
    enddo
    ymin = 1.e8
    ymax = 0
    do ichan = 7,nchan
       if ((obj_cnts(ichan)-stat(ichan) .lt. ymin) .and. &
            (obj_cnts(ichan)-stat(ichan) .gt. 1.0e-5)) &
            ymin = obj_cnts(ichan)-stat(ichan)
       if (obj_cnts(ichan)+stat(ichan) .gt. ymax) &
            ymax = obj_cnts(ichan)+stat(ichan)
       if ((obj_expo*obj_back*(spec(ichan)-uncr(ichan)) .lt. ymin) &
            .and. (obj_expo*obj_back*(spec(ichan)-uncr(ichan))  &
            .gt. 1.0e-5)) &
            ymin = obj_expo*obj_back*(spec(ichan)-uncr(ichan))
       if (obj_expo*obj_back*(spec(ichan)+uncr(ichan)) .gt. ymax) &
            ymax = obj_expo*obj_back*(spec(ichan)+uncr(ichan))
    enddo
    open(89,form='formatted',status='unknown',file=spec_file)
    write(89,*) 'viewport 0.2 0.2 0.9 0.85'
    write(89,*) 'la f'
    write(title,3110) obsid, prefix
3110 format('"',i10,' -- MOS',a5,'"')
    write(89,*) 'la t ', title
    write(89,*) 'la x Energy (keV)'
    write(89,*) 'la y Counts'
    i = 0.75*ymax
    write(89,*) 'la 1 p 3 ', i, ' just left ', &
         '"Observed (red), Particle Background (green)"'
    write(89,*) 'log y'
    write(89,*) 'r x 0 12'
    write(89,*) 'r y ', ymin, ymax
    write(89,*) 'read serr 2'
    write(89,*) 'read serr 3'
    write(89,*) 'read serr'
    write(89,*) 'font roman'
    write(89,*) 'cs 1.8'
    print *, obj_back
    do ichan = 1,nchan
       write(89,*) ener(ichan), obj_cnts(ichan), stat(ichan), &
            obj_expo*obj_back*spec(ichan), &
            obj_expo*obj_back*uncr(ichan)
    enddo
    close(89)

    !  Get two unused luna

    call ftgiou(luna,status)
    if((status .ne. 0) .and. (err .eq. 0)) then
       err = 1
       print *, 'STATUS FTGIOU =', status
       print *, 'XMM-BACK POSITION 12'
    endif
    call ftgiou(lunb,status)
    if((status .ne. 0) .and. (err .eq. 0)) then
       err = 1
       print *, 'STATUS FTGIOU =', status
       print *, 'XMM-BACK POSITION 13'
    endif

    !  Open the spectrum FITS file

    call ftopen (luna,obj_file,0,blockn,status)
    if((status .ne. 0) .and. (err .eq. 0)) then
       err = 1
       print *, 'STATUS FTOPEN =', status
       print *, 'XMM-BACK POSITION 14'
       print *, spec_name
    endif

    !  Create the output FITS file using the original file as a template

    write(back_file,3100) prefix
3100 format('mos',a5,'-back.pi')
    call ftopen (lunb,back_file,0,blockn,status)
    if(status .ne. 0) then
       if(status .ne. 105) then
          status = 0
          call ftdelt(lunb,status)
          if (status .ne. 0) print *, 'STATUS = ', status
       endif
       status = 0
       call fttplt (lunb,back_file,obj_file,status)
       if(status .ne. 0) then
          print *, 'XMM-BACK POSITION 15a, STATUS = ', status
          print *, back_file
          print *, obj_file
          stop
       endif
    else
       print *, 'Background spectrum file already exists: ', &
            back_file
       if(clobber) then
          print *, 'Clobber the file and continue'
          status = 0
          call ftdelt(lunb,status)
          call fttplt (lunb,back_file,obj_file,status)
          if(status .ne. 0) then
             print *, 'XMM-BACK POSITION 15b, STATUS = ', &
                  status
          endif
       else
          print *, 'Will not clobber file'
          stop
       endif
    endif

    if(err .eq. 0) then

       !  Move to the input file spectrum extension

       call ftmahd (lunb,2,hdutype,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTMHD =', status
          print *, 'XMM-BACK POSITION 16'
          print *, 'Could not find the spectrum extension'
       endif

       !  Check to make sure that we have the spectrum extension

       call ftghbn (lunb,10,nchan,tfields,ttype,tform, &
            tunit,extname,varidat,status)
       if(extname .ne. 'SPECTRUM') then
          print *, 'XMM-BACK POSITION 17'
          print *, extname
          print *, 'This was supposed to be a spectrum ext'
          stop
       endif

       !  Get rid of extra extensions
       !  Move to the next extension

       call ftmrhd (lunb,1,hdutype,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTMHD =', status
          print *, 'XMM-BACK POSITION 18'
          print *, 'No more extensions'
          status = 0

          !  Move to the input file spectrum extension

          call ftmahd (lunb,2,hdutype,status)
          if((status .ne. 0) .and. (err .eq. 0)) then
             err = 1
             print *, 'STATUS FTMHD =', status
             print *, 'XMM-BACK POSITION 19'
             print *, 'Could not find the spectrum extension'
          endif
       endif

       !  Check to make sure that we have the spectrum extension

       call ftghbn (lunb,10,nchan,tfields,ttype,tform, &
            tunit,extname,varidat,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTGHBN =', status
          print *, 'XMM-BACK POSITION 20'
          print *, 'Could not read extension info'
       endif

       if(extname .ne. 'SPECTRUM') then
          do while (extname .ne. 'SPECTRUM')
             call ftdhdu (lunb,hdutype,status)
             if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTDHDU =', status
                print *, 'XMM-BACK POSITION 21'
                print *, 'Could not delete an extension'
             endif
             call ftghbn (lunb,10,nchan,tfields,ttype,tform, &
                  tunit,extname,varidat,status)
             if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTGHBN =', status
                print *, 'XMM-BACK POSITION 22'
                print *, 'Could not read extension info'
             endif
          enddo
       endif

       !  Close the input lun

       call ftclos(luna,status)

       !  Write the output spectrum

       if(chisqfl) then ! why obj_expo for chisq and expo for not?? BHP
          call ftukye(lunb,'EXPOSURE',obj_expo,4, &
               'Background exposure',status)
       else
          call ftukye(lunb,'EXPOSURE',expo,4, &
               'Background exposure',status)
       endif
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTUKYE =', status
          print *, 'XMM-BACK POSITION 23'
          print *, 'Could not write the exposure keyword'
       endif
       call ftukye(lunb,'BACKSCAL',obj_back,4, &
            'Scaling factor for background',status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTUKYE =', status
          print *, 'XMM-BACK POSITION 24'
          print *, 'Could not write the backscale keyword'
       endif

       !  Delete the existing column information

       call ftdcol(lunb,2,status)
       call ftdcol(lunb,1,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTUKYE =', status
          print *, 'XMM-BACK POSITION 25'
          print *, 'Could not delete the existing column info'
       endif

       !  Create the new columns

       if(chisqfl) then
          call fticol(lunb,1,'CHANNEL','1J',status)
          call fticol(lunb,2,'COUNTS','1E',status)
          call fticol(lunb,3,'STAT_ERR','1E',status)
       else
          call fticol(lunb,1,'CHANNEL','1J',status)
          call fticol(lunb,2,'COUNTS','1J',status)
       endif
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTUKYE =', status
          print *, 'XMM-BACK POSITION 26'
          print *, 'Could not the new columns'
       endif

       !  Update a few keywords

       call ftukys(lunb,'TUNIT1','channel', &
            'physical unit of field',status)
       call ftukys(lunb,'TUNIT2','count', &
            'physical unit of field',status)
       if(chisqfl) then
          call ftukys(lunb,'TUNIT3','count', &
               'physical unit of field',status)
          call ftukyl(lunb,'POISSERR',.FALSE., &
               'physical unit of field',status)
       else
          !                call ftdkey(lunb,'STAT_ERR',status)
          call ftukyl(lunb,'POISSERR',.TRUE., &
               'physical unit of field',status)
       endif
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTUKYE =', status
          print *, 'XMM-BACK POSITION 27'
          print *, 'Could not write the various keywords'
       endif

       !  Prepare the spectrum data

       if(chisqfl) then
          do ichan=1,nchan
             chan(ichan) = ichan - 1
             spec(ichan) = spec(ichan)*obj_expo*obj_back
             uncr(ichan) = uncr(ichan)*obj_expo*obj_back
          enddo
       else
          do ichan=1,nchan
             chan(ichan) = ichan - 1
             ispec(ichan) = nint(spec(ichan)*expo*obj_back)
          enddo
       endif

       !  Write the column data

       if(chisqfl) then
          call ftpclj(lunb,1,1,1,nchan,chan,status)
          call ftpcle(lunb,2,1,1,nchan,spec,status)
          call ftpcle(lunb,3,1,1,nchan,uncr,status)
       else
          call ftpclj(lunb,1,1,1,nchan,chan,status)
          call ftpclj(lunb,2,1,1,nchan,ispec,status)
       endif
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTUKYE =', status
          print *, 'XMM-BACK POSITION 28'
          print *, 'Could not write the column information'
       endif

       !  Close the output file

       call ftclos(lunb,status)
       if((status .ne. 0) .and. (err .eq. 0)) then
          err = 1
          print *, 'STATUS FTUKYE =', status
          print *, 'XMM-BACK POSITION 29'
          print *, 'Could not close the output file'
       endif

    endif
    call ftfiou (luna,status)
    call ftfiou (lunb,status)

    RETURN

  end subroutine mos_back

end module mos_back_mod
