!+
! Module: sp_partial_mod
!
! Authors:  Steven L. Snowden - Goddard Space Flight Center (NASA)
!
! Description: Extended Source Analysis Software
!
! Comments:
!
! $Id: sp_partial_mod.f90,v 1.2 2009/02/20 13:27:08 aibarra Exp $
!-
!
!  Version 5.0: 7 November 2012
!
!  Returns the average SP flux and BACKSCAL keyword for a region
!
!  sp_partial uses the results from a spectral fit of a limited
!  region of the FOV to better determine the contribution of
!  SP contamination.  This is to avoid the effect of a bright
!  extended region skewing the SP component fit.  It uses the
!  SP templates from the limited region and the full region plus
!  the fitted SP normalization from the limited region to find
!  the SP normalization for the full FOV.
!
!  Input:
!    CalDB data directory
!    MOS number
!    full region image file name
!    full region spectrum file name
!    selected region image file name
!    selected region spectrum file name
!    selected region SP normalization
!
!    Example: sp-partial /path/qpb 1 full-image.fits full-spec.fits
!                   sel-image.fits sel-spec.fits norm
!
! Usage:
!   data directory
!   MOS number
!   full region image file name
!   full region spectrum file name
!   selected region image file name
!   selected region spectrum file name
!   selected region SP normalization
! 
!   Example: 
!     sp-partial /path/qpb 1 full-image.fits 
!                     full-spec.fits sel-image.fits 
!                     sel-spec.fits norm

module sp_partial_mod

  use dal
  use param
  use errorHandling
  use subsn_mod

  implicit none

  private

  public sp_partial

contains

  !---------------------------------------------------------------------
  subroutine sp_partial
  !---------------------------------------------------------------------
        character(len=132) :: xmmd(132)
        character(len=60) :: fullimage, fullspec, &
            regionimage, regionspec
        character(len=132) :: xmmdir
        integer :: mosnum
        real :: backfull, backsel, cfull, csel, rnorm, rrnorm

        equivalence (xmmdir,xmmd)

        xmmdir = stringParameter("caldb")
        mosnum = intParameter("detector")
        fullimage = stringParameter("fullimage")
        fullspec = stringParameter("fullspec")
        regionimage = stringParameter("regionimage")
        regionspec = stringParameter("regionspec")
        rnorm = realParameter("rnorm")

        print *, 'xmmdir: ', xmmdir
        print *, 'mosnum: ', mosnum

        call spscale(xmmdir,xmmd,mosnum,fullimage, &
             fullspec,backfull,cfull)

        call spscale(xmmdir,xmmd,mosnum,regionimage, &
             regionspec,backsel,csel)
        
!  Determine the old normalization

        rrnorm = rnorm*(backfull/backsel)*(cfull/csel)
        print *, 'Scaled Normalization: ', rrnorm
        
        RETURN

      end subroutine sp_partial

    end module sp_partial_mod
