module subsn_mod

  use dal
  use errorHandling
  use genl_mod  
  implicit none
  
contains

        subroutine ANGSEP(DLONG1,DLAT1,DLONG2,DLAT2,DANG,IERR)
!
!C    Calculates the angular separation between two celestial bodies
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:48
!
!A  author : KOD        date: 06-SEP-1989 16:40
!U  update : KOD        date: 20-MAR-1990 15:45
!U  update : KOD        date: 24-OCT-1991 13:57  (sin --> dsin, ...)
!
!T  status: tested
!
!   general description
!G
!G  Subroutine to calculate the angular separation between two
!G  celestial bodies. The 'latitude' is defined as to have
!G  0 degrees at the equator and +90 degrees at the north pole.
!G  Special care is taken for small angles, according to
!G    R.W.Sinnott, 'Sky and Telescope', August 1984, p.159.
!
!   call_var.          type I/O description
!P
!P  DLONG1              R8  I   'longitude 1' (e.g. R.A.; radians)
!P  DLAT1               R8  I   'latitude  1' (e.g. Dec.; radians)
!P
!P  DLONG2              R8  I   'longitude 2' (e.g. R.A.; radians)
!P  DLAT2               R8  I   'latitude  2' (e.g. Dec.; radians)
!P
!P  DANG                R8    O angular separation       (radians)
!P
!P  IERR                I4    O = 0 no error
!P                              = 1 warning
!P                              = 2 error
!P                              = 3 severe error
!
!   include_block name          description
!I  R$COMMON:CGENL.CMN          general parameter common block
!I  R$INCLUDE:IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
!R  WRERR               SR      formats error code in standard form

        IMPLICIT NONE
!        INCLUDE 'cgenl.cmn'
!        INCLUDE 'igenl.inc'

        DOUBLE PRECISION :: DLONG1, DLAT1, DLONG2, DLAT2, DANG, &
            D1, D2, DR
        INTEGER :: IERR
        CHARACTER(len=8) :: RTNAME = 'ANGSEP'
        CHARACTER(len=80) :: LOG

        IERR = 0

        if (max(abs(DLONG1),abs(DLONG2)) .gt. D2PI) then
            IERR = max(IERR,1)
            LOG = 'Unusual value for the longitude'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if (max(abs(DLAT1),abs(DLAT2)) .gt. DPID2) then
            IERR = max(IERR,2)
            LOG = 'Error in input parameters'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if (IERR .lt. 2) then
            D1 = dsin ( 0.5D0 * ( DLAT1 -  DLAT2 ) )
            D2 = dsin ( 0.5D0 * (DLONG1 - DLONG2 ) )
            DR = D1 * D1 + D2 * D2 * dcos(DLAT1) * dcos(DLAT2)
            DANG = 2.D0 * datan2 ( dsqrt(DR), dsqrt(1.D0-DR) )
        endif

        return
        end subroutine angsep


        subroutine aug_get_exp (db_file,c_val,c_dst,c_tim, &
            p_flg,aug,aug_exp,idet,ichip,obsid,prefix)

        implicit none

        integer :: blockn, err, hdutype, &
            i, ichip, icol, idate, idet, iexpo, &
            ifilt, ihard, ii, ikeep, iname, iobid, &
            iobjt, iodd, irate, irevl, ispec, itd, &
            itime, iurate, iuhard, lun, lunp, n, &
            nchan, ncols, nobs, obsid, ord(15000), &
            p_flg, revl(15000), tfields, varidat
        integer :: iplot = 0
        integer :: status = 0
        real :: aug(2400), aug_exp, ave_rate, c_dst(3), cum_exp, &
            c_tim, c_val(3), d1, d2, ddist(15000), dist(15000), &
            energ(2400), expo(15000), &
            hard(15000), hardmax, hardmin, hdist(15000), muhard, &
            murate, rate(15000), ratemax, ratemin, rdist(15000), &
            spec(2400,15000), td, tdist(15000), temphard, time(15000),  &
            tr, tspec(2400), uhard(15000), urate(15000)

        double precision :: date(15000)

        character(len=5) :: prefix
        character(len=6) :: filt(15000)
        character(len=8) :: colnam(23), tform(23), ttype(23), tunit(23)
        character(len=11) :: test
        character(len=13) :: obid(15000)
        character(len=15) :: objt(15000)
        character(len=17) :: name(15000)
        character(len=20) :: extname
        character(len=24) :: title
        character(len=80) :: augfile, db_file
        logical anyf

!        save iplot, lunp

!        print *, db_file,c_val,c_dst,c_tim,p_flg,
!     +      aug_exp,idet,ichip,obsid,prefix
!
!  if(n_params(0) LT 7)then begin
!    print,"get_aug,db_file,r_lim,h_lim,d_lim,c_val,c_dst,c_tim,p_flg,aug,aug_exp"
!    print,"Purpose:to create a mean corner spectrum"
!    print,"  Given a rate, hardness, and date,"
!    print,"  finds the closest c_tim seconds of exposure"
!    print,"  in that 3-D space"
!    print,"Inputs:"
!    print,"  db_file:database file of corner spectra"
!    print,"  c_val:central [rate,hardness,date]"
!    print,"  c_dst:distance scaling in those directions"
!    print,"    as factor of mean uncertainty in those directions"
!    print,"    suggest:[0.00323,0.358,1.0]"
!    print,"  c_tim:how many seconds of exposure desired for spectrum"
!    print,"  p_flg:1 if informative plot to be made"
!    print,"Outputs:"
!    print,"  aug:the augmented spectrum"
!    print,"  aug_exp:the amount of exposure time in the augmented spectrum"
!    return
!  endif

        err = 0

!  If selected, make a plot, first set up the plot

        if((p_flg .eq. 1) .and. (iplot .eq. 0)) then
            iplot = 1

!  Get an unused lun

            status = 0
            call ftgiou(lunp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_EXP POSITION 01'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,2999) prefix
 2999       format('mos',a5,'-aug.qdp')
            open(lunp,form='formatted',status='unknown', &
                    file=augfile)
            write(lunp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
 3000       format('"',i10,' -- MOS',a5,'"')
            write(lunp,*) ' la t ', title
            write(lunp,*) ' read serr 1'
            write(lunp,*) ' read serr 2'
            write(lunp,*) ' read serr 3'
            write(lunp,*) ' read serr 4'
            write(lunp,*) ' read serr 5'
            write(lunp,*) ' read serr 6'
            write(lunp,*) ' read serr 7'
            write(lunp,*) ' read serr 8'
            write(lunp,*) ' read serr 9'
            write(lunp,*) ' read serr 10'
            write(lunp,*) ' read serr 11'
            write(lunp,*) ' read serr 12'
            write(lunp,*) ' read serr 13'
            write(lunp,*) ' read serr 14'
            write(lunp,*) ' read serr 15'
            write(lunp,*) ' read serr 16'
            write(lunp,*) ' read serr 17'
            write(lunp,*) ' read serr 18'
            write(lunp,*) ' read serr 19'
            write(lunp,*) ' font roman'
            write(lunp,*) ' cs 1.5'
            write(lunp,*) ' la f'
            write(lunp,*) ' col 2 on 2'
            write(lunp,*) ' col 1 on 3'
            write(lunp,*) ' col 3 on 4'
            write(lunp,*) ' col 2 on 5'
            write(lunp,*) ' col 1 on 6'
            write(lunp,*) ' col 3 on 7'
            write(lunp,*) ' col 2 on 8'
            write(lunp,*) ' col 1 on 9'
            write(lunp,*) ' col 3 on 10'
            write(lunp,*) ' col 2 on 11'
            write(lunp,*) ' col 1 on 12'
            write(lunp,*) ' col 3 on 13'
            write(lunp,*) ' col 2 on 14'
            write(lunp,*) ' col 1 on 15'
            write(lunp,*) ' col 3 on 16'
            write(lunp,*) ' col 2 on 17'
            write(lunp,*) ' col 1 on 18'
            write(lunp,*) ' col 3 on 19'
            write(lunp,*) ' mark 1 on 2'
            write(lunp,*) ' mark 1 on 3'
            write(lunp,*) ' mark 1 on 4'
            write(lunp,*) ' mark 1 on 5'
            write(lunp,*) ' mark 1 on 6'
            write(lunp,*) ' mark 1 on 7'
            write(lunp,*) ' mark 1 on 8'
            write(lunp,*) ' mark 1 on 9'
            write(lunp,*) ' mark 1 on 10'
            write(lunp,*) ' mark 1 on 11'
            write(lunp,*) ' mark 1 on 12'
            write(lunp,*) ' mark 1 on 13'
            write(lunp,*) ' mark 1 on 14'
            write(lunp,*) ' mark 1 on 15'
            write(lunp,*) ' mark 1 on 16'
            write(lunp,*) ' mark 1 on 17'
            write(lunp,*) ' mark 1 on 18'
            write(lunp,*) ' mark 1 on 19'
            write(lunp,*) ' line off 2'
            write(lunp,*) ' line off 3'
            write(lunp,*) ' line off 4'
            write(lunp,*) ' line off 5'
            write(lunp,*) ' line off 6'
            write(lunp,*) ' line off 7'
            write(lunp,*) ' line off 8'
            write(lunp,*) ' line off 9'
            write(lunp,*) ' line off 10'
            write(lunp,*) ' line off 11'
            write(lunp,*) ' line off 12'
            write(lunp,*) ' line off 13'
            write(lunp,*) ' line off 14'
            write(lunp,*) ' line off 15'
            write(lunp,*) ' line off 16'
            write(lunp,*) ' line off 17'
            write(lunp,*) ' line off 18'
            write(lunp,*) ' line off 19'
            write(lunp,*) ' win 1'
            write(lunp,*) ' yplot 1 2 3 4'
            write(lunp,*) ' loc 0.05 0.48 0.39 1.0'
            write(lunp,*) ' la 11 p 0.06 9.0 just left "CCD #2"'
            write(lunp,*) ' r x1 0.0 0.1'
            write(lunp,*) ' r y1 0.0 10.0'
            write(lunp,*) ' la y Hardness'
            write(lunp,*) ' win 2'
            write(lunp,*) ' yplot 1 5 6 7'
            write(lunp,*) ' loc 0.35 0.48 0.69 1.0'
            write(lunp,*) ' la 12 p 0.06 9.0 just left "CCD #3"'
            write(lunp,*) ' r x2 0.0 0.1'
            write(lunp,*) ' r y2 0.0 10.0'
            write(lunp,*) ' win 3'
            write(lunp,*) ' yplot 1 8 9 10'
            write(lunp,*) ' loc 0.65 0.48 0.99 1.0'
            write(lunp,*) ' la 13 p 0.06 9.0 just left "CCD #4"'
            write(lunp,*) ' r x3 0.0 0.1'
            write(lunp,*) ' r y3 0.0 10.0'
            write(lunp,*) ' win 4'
            write(lunp,*) ' yplot 1 11 12 13'
            write(lunp,*) ' loc 0.05 0.05 0.39 0.57'
            write(lunp,*) ' la 14 p 0.06 9.0 just left "CCD #5"'
            write(lunp,*) ' r x4 0.0 0.1'
            write(lunp,*) ' r y4 0.0 10.0'
            write(lunp,*) ' la y Hardness'
            write(lunp,*) ' la x Rate'
            write(lunp,*) ' win 5'
            write(lunp,*) ' yplot 1 14 15 16'
            write(lunp,*) ' loc 0.35 0.05 0.69 0.57'
            write(lunp,*) ' la 15 p 0.06 9.0 just left "CCD #6"'
            write(lunp,*) ' r x5 0.0 0.1'
            write(lunp,*) ' r y5 0.0 10.0'
            write(lunp,*) ' la x Rate'
            write(lunp,*) ' win 6'
            write(lunp,*) ' yplot 1 17 18 19'
            write(lunp,*) ' loc 0.65 0.05 0.99 0.57'
            write(lunp,*) ' la 16 p 0.06 9.0 just left "CCD #7"'
            write(lunp,*) ' r x6 0.0 0.1'
            write(lunp,*) ' r y6 0.0 10.0'
            write(lunp,*) ' la x Rate'
        endif

!  Get an unused lun for the caldb file

        err = 0
        status = 0
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'AUG_GET_EXP POSITION 02'
            print *, 'STATUS FTGIOU =', status
        endif

!  Open the FITS file

        call ftopen (lun,db_file,0,blockn,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'AUG_GET_EXP POSITION 03'
            print *, 'STATUS FTOPEN =', status
            print *, db_file
        endif

!  Move to the correct extension

        if(err .eq. 0) then
            call ftmahd (lun,ichip,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_EXP POSITION 04'
                print *, 'STATUS FTMAHD =', status
                print *, db_file
                print *, 'Could not find the spectrum extension'
            else

!  Read the FITS file header information

                call ftghbn (lun,23,nchan,tfields,ttype,tform, &
                    tunit,extname,varidat,status)
                print *, 'Total number of Obs in QPB file: ', nchan
                write(test,3010) ichip
 3010           format('QUIES_CHIP',i1)
                if(extname .ne. test) then
                    print *, 'AUG_GET_EXP POSITION 05'
                    print *, extname
                    print *, 'This was supposed to be the QUIES_CHIP', &
                            ichip,' extension'
                    stop
                endif
                if(status .ne. 0) then
                    err = 1
                    print *, 'AUG_GET_EXP POSITION 06'
                    print *, 'STATUS FTGHPR =', status
                    print *, 'Failure on reading the header ', &
                            'information'
                else
                    if(nchan .gt. 15000) then
                        print *, 'Too much data: ', ncols
                        print *, 'Limit selection to 15000'
                        ncols = 15000
                    endif

!  Read in the column names

                    call ftgkns (lun,'TTYPE',1,23,colnam,ncols,status)

!  Extract the channel numbers

                    iobid = 0
                    iname = 0
                    iobjt = 0
                    idate = 0
                    irevl = 0
                    ifilt = 0
                    itime = 0
                    iexpo = 0
                    irate = 0
                    iurate = 0
                    ihard = 0
                    iuhard = 0
                    ispec = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'OBSID') iobid = icol
                        if(colnam(icol) .eq. 'NAME') iname = icol
                        if(colnam(icol) .eq. 'OBJECT') iobjt = icol
                        if(colnam(icol) .eq. 'DATE') idate = icol
                        if(colnam(icol) .eq. 'REVOLUT') irevl = icol
                        if(colnam(icol) .eq. 'FILTER') ifilt = icol
                        if(colnam(icol) .eq. 'ONTIME') itime = icol
                        if(colnam(icol) .eq. 'EXPO') iexpo = icol
                        if(colnam(icol) .eq. 'RATE') irate = icol
                        if(colnam(icol) .eq. 'RATE_ERR') iurate = icol
                        if(colnam(icol) .eq. 'HARD') ihard = icol
                        if(colnam(icol) .eq. 'HARD_ERR') iuhard = icol
                        if(colnam(icol) .eq. 'SPECTRUM') ispec = icol
                    enddo

!        print *, colnam, iobid, iname, iobjt, idate, irevl, ifilt,
!     +       itime, iexpo, irate, iurate, ihard, iuhard, ispec

                    do i=1,nchan
                       if(iobid .ne. 0) &
                            call ftgcvs(lun,iobid,i,1,1,' ', &
                                obid(i),anyf,status)
                        if(iname .ne. 0) &
                            call ftgcvs(lun,iname,i,1,1,' ', &
                                name(i),anyf,status)
                        if(iobjt .ne. 0) &
                            call ftgcvs(lun,iobjt,i,1,1,' ', &
                                objt(i),anyf,status)
                        if(idate .ne. 0) &
                            call ftgcvd(lun,idate,i,1,1,0, &
                                date(i),anyf,status)
                        if(irevl .ne. 0) &
                            call ftgcvj(lun,irevl,i,1,1,0, &
                                revl(i),anyf,status)
                        if(ifilt .ne. 0) &
                            call ftgcvs(lun,ifilt,i,1,1,' ', &
                                filt(i),anyf,status)
                        if(itime .ne. 0) &
                            call ftgcve(lun,itime,i,1,1,0, &
                                time(i),anyf,status)
                        if(iexpo .ne. 0) &
                            call ftgcve(lun,iexpo,i,1,1,0, &
                                expo(i),anyf,status)
                        if(irate .ne. 0) &
                            call ftgcve(lun,irate,i,1,1,0, &
                                rate(i),anyf,status)
                        if(iurate .ne. 0) &
                            call ftgcve(lun,iurate,i,1,1,0, &
                                urate(i),anyf,status)
                        if(ihard .ne. 0) &
                            call ftgcve(lun,ihard,i,1,1,0, &
                                hard(i),anyf,status)
                        if(iuhard .ne. 0) &
                            call ftgcve(lun,iuhard,i,1,1,0, &
                                uhard(i),anyf,status)
                        if(ispec .ne. 0) then
                            call ftgcve(lun,ispec,i,1,2400,0, &
                                    tspec,anyf,status)
                            do ii=1,2400
                                spec(ii,i) = tspec(ii)
                            enddo
                            if((i .eq. nchan) .and. (status .ne. 0)) &
                                    status = 0
                        endif
                    enddo
                    if(status .ne. 0) then
                        print *, 'Missing column in QPB info file'
                        print *, 'STATUS: ', status
                        return
                    endif
                endif
            endif
        endif
        call ftclos (lun,status)

!  first determine if the observation CCD is in an odd state

        iodd = 0
        if(idet .eq. 1) then
            if (ichip .eq. 4) then
                tr = 100.0*c_val(1)
                if(tr .lt. 5.2) then
                    temphard = 0.96
                elseif(tr .lt. 5.2) then
                    temphard = 0.423*tr - 1.24
                else
                    temphard = 0.423*tr - 1.24
                endif
                if(c_val(2) .lt. temphard) iodd = 1
            elseif (ichip .eq. 5) then
                tr = 100.0*c_val(1)
                if(tr .lt. 7.0) then
                    temphard = 1.54
                elseif (tr .lt. 9.0) then
                    temphard = 0.53*tr - 2.17
                else
                    temphard = 2.60
                endif
                if(c_val(2) .lt. temphard) iodd = 1
            else
                if(c_val(2) .lt. 1.5) iodd = 1
            endif
        else
            if (ichip .eq. 2) then
                if(c_val(2) .lt. 1.1) iodd = 1
            elseif (ichip .eq. 5) then
                tr = 100.0*c_val(1)
                if(tr .lt. 10.0) then
                    temphard = 0.15*tr
                elseif (tr .lt. 16.67) then
                    temphard = 0.15*tr
                else
                    temphard = 2.50
                endif
                if(c_val(2) .lt. temphard) iodd = 1
            else
                if(c_val(2) .lt. 1.5) iodd = 1
            endif
        endif

!  Set the normal filtering limits

        d1 = -9.0
        d2 = 0.0
        hardmin = 0.2
        hardmax = 15.0
        ratemin = 0.01
        ratemax = 0.1

!  Sort through the archived observation data

        nobs = 0
        do i=1,nchan
            if((expo(i) .ge. 1000.0) .and. (rate(i) .gt. ratemin) .and. &
                    (rate(i) .lt. ratemax) .and. &
                    (hard(i) .gt. hardmin) .and. &
                    (hard(i) .lt. hardmax) .and. &
                    (filt(i) .ne. 'CalClo')) then

!  Write out all the data for plotting if selected

                if(p_flg .eq. 1) then
                    if(ichip .eq. 2) then
                        write(lunp,2120) rate(i), urate(i), &
                            hard(i), uhard(i), d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2
 2120                   format(1x,2f10.6,2f6.2,34f4.0)
                    elseif(ichip .eq. 3) then
                        write(lunp,2130) rate(i), urate(i), &
                            d1, d2, d1, d2, d1, d2,  &
                            hard(i), uhard(i), d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2
 2130                   format(1x,2f10.6,6f4.0,2f6.2,28f4.0)
                    elseif(ichip .eq. 4) then
                        write(lunp,2140) rate(i), urate(i), &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2,  &
                            hard(i), uhard(i), d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2140                   format(1x,2f10.6,12f4.0,2f6.2,22f4.0)
                    elseif(ichip .eq. 5) then
                        write(lunp,2150) rate(i), urate(i), &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, hard(i), uhard(i), d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2
 2150                   format(1x,2f10.6,18f4.0,2f6.2,16f4.0)
                    elseif(ichip .eq. 6) then
                        write(lunp,2160) rate(i), urate(i), &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            hard(i), uhard(i), d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2
 2160                   format(1x,2f10.6,24f4.0,2f6.2,10f4.0)
                    elseif(ichip .eq. 7) then
                        write(lunp,2170) rate(i), urate(i), &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, &
                            hard(i), uhard(i), d1, d2, d1, d2
 2170                   format(1x,2f10.6,30f4.0,2f6.2,4f4.0)
                    endif
                endif

!  Check to see whether the archived CCD is in an odd state
!  and select whether the data will be kept or not

                ikeep = 1
                if(idet .eq. 1) then
                    if (ichip .eq. 4) then
                        tr = 100.0*c_val(1)
                        if(tr .lt. 5.2) then
                            temphard = 0.96
                        elseif(tr .lt. 5.2) then
                            temphard = 0.423*tr - 1.24
                        else
                            temphard = 0.423*tr - 1.24
                        endif
                        if(iodd .eq. 1) then
                            if(hard(i) .ge. temphard) ikeep = 0
                        else
                            if(hard(i) .lt. temphard) ikeep = 0
                        endif
                    elseif (ichip .eq. 5) then
                        tr = 100.0*c_val(1)
                        if(tr .lt. 7.0) then
                            temphard = 1.54
                        elseif (tr .lt. 9.0) then
                            temphard = 0.53*tr - 2.17
                        else
                            temphard = 2.60
                        endif
                        if(iodd .eq. 1) then
                            if(hard(i) .ge. temphard) ikeep = 0
                        else
                            if(hard(i) .lt. temphard) ikeep = 0
                        endif
                    else
                        if(hard(i) .lt. 1.5) then
                            ikeep = 0
                        endif
                    endif
                endif
                if(idet .eq. 2) then
                    if(ichip .eq. 2) then
                        if(iodd .eq. 1) then
                            if(hard(i)  .ge. 1.1) ikeep = 0
                        else
                            if(hard(i)  .lt. 1.1) ikeep = 0
                        endif
                    elseif (ichip .eq. 5) then
                        tr = 100.0*c_val(1)
                        if(tr .lt. 10.0) then
                            temphard = 0.15*tr
                        elseif (tr .lt. 16.67) then
                            temphard = 0.15*tr
                        else
                            temphard = 2.50
                        endif
                        if(iodd .eq. 1) then
                            if(hard(i)  .ge. temphard) ikeep = 0
                        else
                            if(hard(i)  .lt. temphard) ikeep = 0
                        endif
                    else
                        if (iodd .eq. 1) then
                            if(hard(i)  .ge. 1.5) ikeep = 0
                        else
                            if(hard(i)  .lt. 1.5) ikeep = 0
                        endif
                    endif
                endif
                if(filt(i) .eq. 'CalClo') ikeep = 0

!  Keep the data set so transfer the information

                if(ikeep .eq. 1) then
                    nobs = nobs + 1
                    obid(nobs) = obid(i)
                    name(nobs) = name(i)
                    objt(nobs) = objt(i)
                    date(nobs) = date(i)
                    revl(nobs) = revl(i)
                    filt(nobs) = filt(i)
                    time(nobs) = time(i)
                    expo(nobs) = expo(i)
                    rate(nobs) = rate(i)
                    urate(nobs) = urate(i)
                    hard(nobs) = hard(i)
                    uhard(nobs) = uhard(i)
                    do ii=1,2400
                        spec(ii,nobs) = spec(ii,i)
                    enddo
                endif
            endif
        enddo

        if(nobs .eq. 0) then
            print *, 'CCD in an anomalous state - no', &
                    'appropriate data'
            print *, 'Rerun analysis excluding instrument: MOS',  &
                    idet,'  CCD #', ichip
            stop
        endif

!  Set the energies of the bins

        do i=1,2400
            energ(i) = 0.005*(i - 1)
        enddo

!  Get some temporary values

        murate = 0.0
        muhard = 0.0
        do i=1,nobs
            murate = murate + urate(i)
            muhard = muhard + uhard(i)
        enddo
        murate = murate/nobs
        muhard = muhard/nobs

        print *,'Rate: ',c_val(1),' Rate error: ',murate
        print *,'Hard: ',c_val(2),' Hard error: ',muhard

!  Find the closest c_tim secs

        if(c_tim .eq. 0.0) then
            print *, 'Error: no exposure length provided'
            return
        else
            if((c_dst(1) .eq. 0.0) .and. (c_dst(2) .eq. 0.0) .and. &
                    (c_dst(3) .eq. 0.0)) then
                print *, 'Error: no distance scalings'
                return
            else
                do i=1,nobs
                    if(c_dst(1) .gt. 0.0) then
                        rdist(i) = (rate(i)-c_val(1))/c_dst(1)
                    else
                        rdist(i) = rate(i)*0.0
                    endif
                    if(c_dst(2) .gt. 0.0) then
                        hdist(i) = (hard(i)-c_val(2))/c_dst(2)
                    else
                        hdist(i) = hard(i)*0.0
                    endif
                    if(c_dst(3) .gt. 0.0) then
                        ddist(i) = (revl(i)-c_val(3))/c_dst(3)
                    else
                        ddist(i) = revl(i)*0.0
                    endif
                    dist(i) = sqrt(rdist(i)**2 + hdist(i)**2 + &
                        ddist(i)**2)
                enddo
            endif
            do i=1,nobs
                ord(i) = i
                tdist(i) = dist(i)
            enddo
            do ii=nobs-1,1,-1
                do i=1,ii
                    if(tdist(i) .gt. tdist(i+1)) then
                        td = tdist(i)
                        tdist(i) = tdist(i+1)
                        tdist(i+1) = td
                        itd = ord(i)
                        ord(i) = ord(i+1)
                        ord(i+1) = itd
                    endif
                enddo
            enddo

            cum_exp = 0.0
            n = 0
            ave_rate = 0.0
            do while ((cum_exp .lt. c_tim) .and. (n .lt. nobs))
                n = n + 1
                cum_exp = cum_exp + expo(ord(n))
                ave_rate = ave_rate + rate(ord(n))*expo(ord(n))
                if(p_flg .eq. 1) then
                    if(ichip .eq. 2) then
                        write(lunp,2125) rate(ord(n)), urate(ord(n)), &
                            d1, d2, hard(ord(n)), uhard(ord(n)), &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2
 2125                   format(1x,2f10.6,2f4.0,2f6.2,32f4.0)
                    elseif(ichip .eq. 3) then
                        write(lunp,2135) rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, d1, d2, d1, d2,  &
                            hard(ord(n)), uhard(ord(n)), d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2,  &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2
 2135                   format(1x,2f10.6,8f4.0,2f6.2,26f4.0)
                    elseif(ichip .eq. 4) then
                        write(lunp,2145) rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, &
                            hard(ord(n)), uhard(ord(n)),  &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2145                   format(1x,2f10.6,14f4.0,2f6.2,20f4.0)
                    elseif(ichip .eq. 5) then
                        write(lunp,2155) rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            hard(ord(n)), uhard(ord(n)), &
                            d1, d2, d1, d2, d1, d2, d1, d2, & 
                            d1, d2, d1, d2, d1, d2
 2155                   format(1x,2f10.6,20f4.0,2f6.2,14f4.0)
                    elseif(ichip .eq. 6) then
                        write(lunp,2165) rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, hard(ord(n)), uhard(ord(n)), &
                            d1, d2, d1, d2, d1, d2, d1, d2
 2165                   format(1x,2f10.6,26f4.0,2f6.2,8f4.0)
                    elseif(ichip .eq. 7) then
                        write(lunp,2175) rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2, &
                            d1, d2, d1, d2, d1, d2, d1, d2,  &
                            d1, d2, d1, d2, d1, d2, d1, d2,  &
                            hard(ord(n)), uhard(ord(n)), d1, d2
 2175                   format(1x,2f10.6,32f4.0,2f6.2,2f4.0)
                    endif
                endif
            enddo
            ave_rate = ave_rate/cum_exp
            print *, 'Exposure: ', cum_exp
            print *, 'Exposure weighted count rate: ', ave_rate
            print *, 'Observation count rate: ', c_val(1), ' and ', &
                'ratio: ', ave_rate/c_val(1)

            if(p_flg .eq. 1) then
                if(ichip .eq. 2) then
                    write(lunp,2127) c_val(1), 0.2*c_val(1), &
                        d1, d2, d1, d2, c_val(2), 0.2*c_val(2), &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2
 2127               format(1x,2f10.6,4f4.0,2f6.2,30f4.0)
                elseif(ichip .eq. 3) then
                    write(lunp,2137) c_val(1), 0.2*c_val(1), &
                        d1, d2, d1, d2, d1, d2, d1, d2,  &
                        d1, d2, c_val(2), 0.2*c_val(2), &
                        d1, d2, d1, d2, d1, d2, d1, d2,  &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2
 2137               format(1x,2f10.6,10f4.0,2f6.2,24f4.0)
                elseif(ichip .eq. 4) then
                    write(lunp,2147) c_val(1), 0.2*c_val(1), &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        c_val(2), 0.2*c_val(2),  &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2
 2147               format(1x,2f10.6,16f4.0,2f6.2,18f4.0)
                elseif(ichip .eq. 5) then
                    write(lunp,2157) c_val(1), 0.2*c_val(1), &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2,  &
                        c_val(2), 0.2*c_val(2), &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2
 2157               format(1x,2f10.6,22f4.0,2f6.2,12f4.0)
                elseif(ichip .eq. 6) then
                    write(lunp,2167) c_val(1), 0.2*c_val(1), &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, c_val(2), 0.2*c_val(2), &
                        d1, d2, d1, d2, d1, d2
 2167               format(1x,2f10.6,28f4.0,2f6.2,6f4.0)
                elseif(ichip .eq. 7) then
                    write(lunp,2177) c_val(1), 0.2*c_val(1), &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2, &
                        d1, d2, d1, d2, d1, d2, d1, d2,  &
                        d1, d2, d1, d2, d1, d2, d1, d2,  &
                        d1, d2, c_val(2), 0.2*c_val(2)
 2177               format(1x,2f10.6,34f4.0,2f6.2)
                endif
            endif
        endif

        call ftfiou (lun,status)


!  Actually put together the spectrum

        do ii=1,2400
            aug(ii) = 0.0
        enddo
        aug_exp = 0.
        do i=1,n
            do ii=1,2400
                aug(ii) = aug(ii) + spec(ii,ord(i))
            enddo
            aug_exp = aug_exp + expo(ord(i))
        enddo
        aug_exp = aug_exp*ave_rate/c_val(1)
        call ftclos (lun,status)
        call ftfiou (lun,status)

        return
        end subroutine aug_get_exp



        subroutine aug_get_pnexp (db_file,c_val,c_dst,c_tim, &
            p_flg,aug,aug_exp,ichip, &
            obsid,prefix,modeint,plc_lo,plc_hi)

        implicit none

        integer :: blockn, err, hdutype, i, iback, ichip, icol, idate, &
            iexpo, iexpid, ifilt, ihard1, ihard2, ihard3, ii, ikeep, &
            ikey, imode, iobid, iobjt, &
            irate, irate1, irate2, irate3, &
            irate4, irevl, ispec, ispecoo, isubmode, itd, ictime, &
            ittime, iuhard1, iuhard2, iuhard3, iurate, iurate1, &
            iurate2, iurate3, iurate4, lun, lunp, lunpp, lunppp, &
            lunpppp, n, nb1, nb2, nb3, nchan, ncols, next, ng, nn, &
            nobs, nt, obsid, ord(15000), p_flg, plc_lo(2), plc_hi(2), &
            revl(15000), status, tfields, varidat
        integer :: iplot = 0

        real :: aug(4096), aug_exp, ave_rate, back(15000), &
            c_dst(3), cum_exp, ctime(15000), c_tim, c_val(3), d1, d2, &
            ddist(15000), dist(15000), &
            energ(4096), expmax, expmin, expo(15000), hard1(15000), &
            hard2(15000), hard3(15000), hardmax, hardmin, hdist(15000), &
            muhard, murate, oootsca, ootsca(15000), rate(15000), &
            rate1(15000), rate2(15000), rate3(15000), rate4(15000), &
            ratemax, ratemin, rdist(15000), spec(4096,15000), &
            specoo(4096,15000), td, tdist(15000),  &
            te1, te2, temp, teo1, teo2,  &
            ttime(15000), tootsca,  &
            tspec(4096), tspecoo(4096), uhard1(15000), uhard2(15000),  &
            uhard3(15000), urate(15000), urate1(15000), urate2(15000), & 
            urate3(15000), urate4(15000)
        real :: amscal = 1440000.0
        real :: csc(4,2)
        double precision :: date(15000)

        character(len=1) :: key(15000)
        character(len=4) :: prefix
        character(len=6) :: filt(15000)
        character(len=8) :: tform(36), ttype(36), tunit(36)
        character(len=11) :: colnam(36), extna
        character(len=13) :: obid(15000)
        character(len=15) :: objt(15000)
        character(len=17) :: expid(15000)
        character(len=20) :: extname
        character(len=24) :: title
        character(len=25) :: mode(15000), modein, modeint, submode(15000) 
        character(len=80) :: augfile, db_file
        logical anyf

!        save iplot, lunp, lunpp, lunppp, lunpppp

        status = 0

        csc(1,1) = 3.75
        csc(2,1) = 5.07
        csc(3,1) = 5.05
        csc(4,1) = 4.00
        csc(1,1) = 1.75
        csc(2,2) = 1.75
        csc(3,2) = 1.72
        csc(4,2) = 2.16

!  if(n_params(0) LT 7)then begin
!        print,"get_aug,db_file,r_lim,h_lim,d_lim,c_val,c_dst,c_tim,p_flg,aug,aug_exp"
!    print,"Purpose:to create a mean corner spectrum"
!    print,"  Given a rate, hardness, and date,"
!    print,"  finds the closest c_tim seconds of exposure"
!    print,"  in that 3-D space"
!    print,"Inputs:"
!    print,"  db_file:database file of corner spectra"
!    print,"  c_val:central [rate,hardness,date]"
!    print,"  c_dst:distance scaling in those directions"
!    print,"    as factor of mean uncertainty in those directions"
!    print,"    suggest:[0.00323,0.358,1.0]"
!    print,"  c_tim:how many seconds of exposure desired for spectrum"
!    print,"  p_flg:1 if informative plot to be made"
!    print,"Outputs:"
!    print,"  aug:the augmented spectrum"
!    print,"  aug_exp:the amount of exposure time in the augmented spectrum"
!    return
!  endif

        err = 0
        status = 0

        print 4001, c_val
 4001   format('Count Rate, Hardness, and Revolution: ',f8.6,f8.3,f8.0) 
        print 4002, c_dst
 4002   format('Scaling Factors for Obs. Selection: ',f8.6,f8.3,f8.0)
        if(modeint .eq. 'PrimeFullWindow') then
            print *, 'PN Mode: ', modeint
            modein = modeint
            oootsca = 0.063
        elseif(modeint .eq. 'PrimeFullWindowExtended3 ') then
            print *, 'PN Mode: ', modeint
            modein = 'PrimeFullWindowExten'
            oootsca = 0.0232
        elseif(modeint .eq. 'PrimeFullWindowExtended5 ') then
            print *, 'PN Mode: ', modeint
            modein = 'PrimeFullWindowExten'
            oootsca = 0.0163
        else
            print *, 'AUG_GET_PNEXP POSITION 01'
            print *, 'Bad submode: ', modeint
            stop
        endif
        print *, oootsca
 4005   format('Observation OOT Scale Factor: ',f7.4)

!  If selected, make a plot, first set up the plot

        if((p_flg .eq. 1) .and. (iplot .eq. 0)) then
            iplot = 1

!  Get an unused lun

            status = 0
            call ftgiou(lunp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 02'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,2999) prefix
 2999       format('pn',a4,'-aug.qdp')
            open(lunp,form='formatted',status='unknown', &
                    file=augfile)
            write(lunp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
 3000       format('"',i10,' -- PN',a4,'"')
            write(lunp,*) ' la t ', title
            write(lunp,*) ' read serr 1'
            write(lunp,*) ' read serr 2'
            write(lunp,*) ' read serr 3'
            write(lunp,*) ' read serr 4'
            write(lunp,*) ' read serr 5'
            write(lunp,*) ' read serr 6'
            write(lunp,*) ' read serr 7'
            write(lunp,*) ' read serr 8'
            write(lunp,*) ' read serr 9'
            write(lunp,*) ' font roman'
            write(lunp,*) ' cs 1.5'
            write(lunp,*) ' la f'
            write(lunp,*) ' col 2 on 2'
            write(lunp,*) ' col 1 on 3'
            write(lunp,*) ' col 2 on 4'
            write(lunp,*) ' col 1 on 5'
            write(lunp,*) ' col 2 on 6'
            write(lunp,*) ' col 1 on 7'
            write(lunp,*) ' col 2 on 8'
            write(lunp,*) ' col 1 on 9'
            write(lunp,*) ' mark 1 on 2'
            write(lunp,*) ' mark 1 on 3'
            write(lunp,*) ' mark 1 on 4'
            write(lunp,*) ' mark 1 on 5'
            write(lunp,*) ' mark 1 on 6'
            write(lunp,*) ' mark 1 on 7'
            write(lunp,*) ' mark 1 on 8'
            write(lunp,*) ' mark 1 on 9'
            write(lunp,*) ' line off 2'
            write(lunp,*) ' line off 3'
            write(lunp,*) ' line off 4'
            write(lunp,*) ' line off 5'
            write(lunp,*) ' line off 6'
            write(lunp,*) ' line off 7'
            write(lunp,*) ' line off 8'
            write(lunp,*) ' line off 9'
            write(lunp,*) ' win 1'
            write(lunp,*) ' yplot 1 2 3'
            write(lunp,*) ' loc 0.05 0.45 0.57 1.0'
            write(lunp,*) ' la 11 p 0.00065 2.6 just left "Quad #1"'
            write(lunp,*) ' r x1 0.0005 0.004'
            write(lunp,*) ' r y1 2.0 8.0'
            write(lunp,*) ' la y Hardness'
            write(lunp,*) ' win 2'
            write(lunp,*) ' yplot 1 4 5'
            write(lunp,*) ' loc 0.47 0.45 0.99 1.0'
            write(lunp,*) ' la 12 p 0.00065 2.6 just left "Quad #2"'
            write(lunp,*) ' r x2 0.0005 0.004'
            write(lunp,*) ' r y2 2.0 8.0'
            write(lunp,*) ' win 3'
            write(lunp,*) ' yplot 1 6 7'
            write(lunp,*) ' loc 0.05 0.05 0.57 0.60'
            write(lunp,*) ' la 13 p 0.00065 2.6 just left "Quad #3"'
            write(lunp,*) ' r x3 0.0005 0.004'
            write(lunp,*) ' r y3 2.0 8.0'
            write(lunp,*) ' la y Hardness'
            write(lunp,*) ' la x Rate'
            write(lunp,*) ' win 4'
            write(lunp,*) ' yplot 1 8 9'
            write(lunp,*) ' loc 0.47 0.05 0.99 0.60'
            write(lunp,*) ' la 14 p 0.00065 2.6 just left "Quad #4"'
            write(lunp,*) ' r x4 0.0005 0.004'
            write(lunp,*) ' r y4 2.0 8.0'
            write(lunp,*) ' col off 1'
            write(lunp,*) ' la x Rate'

!  Get another unused lun for the augmented spectra

            status = 0
            call ftgiou(lunpp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 03'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,3001) prefix
 3001       format('pn',a4,'-aug-spec.qdp')
            open(lunpp,form='formatted',status='unknown', &
                    file=augfile)
            write(lunpp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
            write(lunpp,*) ' la t ', title
            write(lunpp,*) ' font roman'
            write(lunpp,*) ' cs 1.5'
            write(lunpp,*) ' la f'
            write(lunpp,*) ' col 1 on 2'
            write(lunpp,*) ' col 1 on 3'
            write(lunpp,*) ' col 1 on 4'
            write(lunpp,*) ' col 1 on 5'
            write(lunpp,*) ' mark 1 on 2'
            write(lunpp,*) ' mark 1 on 3'
            write(lunpp,*) ' mark 1 on 4'
            write(lunpp,*) ' mark 1 on 5'
            write(lunpp,*) ' line off 2'
            write(lunpp,*) ' line off 3'
            write(lunpp,*) ' line off 4'
            write(lunpp,*) ' line off 5'
            write(lunpp,*) ' win 1'
            write(lunpp,*) ' yplot 1 2'
            write(lunpp,*) ' loc 0.05 0.45 0.57 1.0'
            write(lunpp,*) ' la 11 p 4.0 200.0 just left "Quad #1"'
            write(lunpp,*) ' r x1 0.15 15.0'
            write(lunpp,*) ' r y1 0.1 500.0'
            write(lunpp,*) ' la y Counts'
            write(lunpp,*) ' log x on'
            write(lunpp,*) ' log y on'
            write(lunpp,*) ' win 2'
            write(lunpp,*) ' yplot 1 3'
            write(lunpp,*) ' loc 0.47 0.45 0.99 1.0'
            write(lunpp,*) ' la 12 p 4.0 200.0 just left "Quad #2"'
            write(lunpp,*) ' r x2 0.15 15.0'
            write(lunpp,*) ' r y2 0.1 500.0'
            write(lunpp,*) ' log x on'
            write(lunpp,*) ' log y on'
            write(lunpp,*) ' win 3'
            write(lunpp,*) ' yplot 1 4'
            write(lunpp,*) ' loc 0.05 0.05 0.57 0.60'
            write(lunpp,*) ' la 13 p 4.0 200.0 just left "Quad #3"'
            write(lunpp,*) ' r x3 0.15 15.0'
            write(lunpp,*) ' r y3 0.1 500.0'
            write(lunpp,*) ' la y Counts'
            write(lunpp,*) ' la x Energy (keV)'
            write(lunpp,*) ' log x on'
            write(lunpp,*) ' log y on'
            write(lunpp,*) ' win 4'
            write(lunpp,*) ' yplot 1 5'
            write(lunpp,*) ' loc 0.47 0.05 0.99 0.60'
            write(lunpp,*) ' la 14 p 4.0 200.0 just left "Quad #4"'
            write(lunpp,*) ' r x4 0.15 15.0'
            write(lunpp,*) ' r y4 0.1 500.0'
            write(lunpp,*) ' col off 1'
            write(lunpp,*) ' la x Energy (keV)'
            write(lunpp,*) ' log x on'
            write(lunpp,*) ' log y on'

!  Get another unused lun for the selected rate 
!  plotted with revolution

            status = 0
            call ftgiou(lunppp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 03a'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,3002) prefix
 3002       format('pn',a4,'-aug-rev-rate.qdp')
            open(lunppp,form='formatted',status='unknown', &
                    file=augfile)
            write(lunppp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
            write(lunppp,*) ' la t ', title
            write(lunppp,*) ' read serr 2'
            write(lunppp,*) ' read serr 3'
            write(lunppp,*) ' read serr 4'
            write(lunppp,*) ' read serr 5'
            write(lunppp,*) ' read serr 6'
            write(lunppp,*) ' read serr 7'
            write(lunppp,*) ' read serr 8'
            write(lunppp,*) ' read serr 9'
            write(lunppp,*) ' font roman'
            write(lunppp,*) ' cs 1.5'
            write(lunppp,*) ' la f'
            write(lunppp,*) ' col 1 on 2'
            write(lunppp,*) ' col 2 on 3'
            write(lunppp,*) ' col 1 on 4'
            write(lunppp,*) ' col 3 on 5'
            write(lunppp,*) ' col 1 on 6'
            write(lunppp,*) ' col 4 on 7'
            write(lunppp,*) ' col 1 on 8'
            write(lunppp,*) ' col 5 on 9'
            write(lunppp,*) ' mark 1 on 2'
            write(lunppp,*) ' mark 1 on 3'
            write(lunppp,*) ' mark 1 on 4'
            write(lunppp,*) ' mark 1 on 5'
            write(lunppp,*) ' mark 1 on 6'
            write(lunppp,*) ' mark 1 on 7'
            write(lunppp,*) ' mark 1 on 8'
            write(lunppp,*) ' mark 1 on 9'
            write(lunppp,*) ' line off 2'
            write(lunppp,*) ' line off 3'
            write(lunppp,*) ' line off 4'
            write(lunppp,*) ' line off 5'
            write(lunppp,*) ' line off 6'
            write(lunppp,*) ' line off 7'
            write(lunppp,*) ' line off 8'
            write(lunppp,*) ' line off 9'
            write(lunppp,*) ' win 1'
            write(lunppp,*) ' yplot 1 2 3'
            write(lunppp,*) ' loc 0.05 0.45 0.57 1.0'
            write(lunppp,*) ' la 11 p 200.0 0.0036 just left "Quad #1"'
            write(lunppp,*) ' r x1 0.0 3500.0'
            write(lunppp,*) ' r y1 0.001 0.004'
            write(lunppp,*) ' la y Count Rate'
            write(lunppp,*) ' win 2'
            write(lunppp,*) ' yplot 1 4 5'
            write(lunppp,*) ' loc 0.47 0.45 0.99 1.0'
            write(lunppp,*) ' la 12 p 200.0 0.0036 just left "Quad #2"'
            write(lunppp,*) ' r x2 0.0 3500.0'
            write(lunppp,*) ' r y2 0.001 0.004'
            write(lunppp,*) ' win 3'
            write(lunppp,*) ' yplot 1 6 7'
            write(lunppp,*) ' loc 0.05 0.05 0.57 0.60'
            write(lunppp,*) ' la 13 p 200.0 0.0036 just left "Quad #3"'
            write(lunppp,*) ' r x3 0.0 3500.0'
            write(lunppp,*) ' r y3 0.001 0.004'
            write(lunppp,*) ' la y Count Rate'
            write(lunppp,*) ' la x Revolution'
            write(lunppp,*) ' win 4'
            write(lunppp,*) ' yplot 1 8 9'
            write(lunppp,*) ' loc 0.47 0.05 0.99 0.60'
            write(lunppp,*) ' la 14 p 200.0 0.0036 just left "Quad #4"'
            write(lunppp,*) ' r x4 0.0 3500.0'
            write(lunppp,*) ' r y4 0.001 0.004'
            write(lunppp,*) ' col off 1'
            write(lunppp,*) ' la x Revolution'

!  Get another unused lun for the selected rate 
!  plotted with revolution

            status = 0
            call ftgiou(lunpppp,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 03b'
                print *, 'STATUS FTGIOU =', status
            endif
            write(augfile,3003) prefix
 3003       format('pn',a4,'-aug-rev-hard.qdp')
            open(lunpppp,form='formatted',status='unknown', &
                    file=augfile)
            write(lunpppp,*) ' viewport 0.2 0.2 0.9 0.8'
            write(title,3000) obsid, prefix
            write(lunpppp,*) ' la t ', title
            write(lunpppp,*) ' read serr 2'
            write(lunpppp,*) ' read serr 3'
            write(lunpppp,*) ' read serr 4'
            write(lunpppp,*) ' read serr 5'
            write(lunpppp,*) ' read serr 6'
            write(lunpppp,*) ' read serr 7'
            write(lunpppp,*) ' read serr 8'
            write(lunpppp,*) ' read serr 9'
            write(lunpppp,*) ' font roman'
            write(lunpppp,*) ' cs 1.5'
            write(lunpppp,*) ' la f'
            write(lunpppp,*) ' col 1 on 2'
            write(lunpppp,*) ' col 2 on 3'
            write(lunpppp,*) ' col 1 on 4'
            write(lunpppp,*) ' col 3 on 5'
            write(lunpppp,*) ' col 1 on 6'
            write(lunpppp,*) ' col 4 on 7'
            write(lunpppp,*) ' col 1 on 8'
            write(lunpppp,*) ' col 5 on 9'
            write(lunpppp,*) ' mark 1 on 2'
            write(lunpppp,*) ' mark 1 on 3'
            write(lunpppp,*) ' mark 1 on 4'
            write(lunpppp,*) ' mark 1 on 5'
            write(lunpppp,*) ' mark 1 on 6'
            write(lunpppp,*) ' mark 1 on 7'
            write(lunpppp,*) ' mark 1 on 8'
            write(lunpppp,*) ' mark 1 on 9'
            write(lunpppp,*) ' line off 2'
            write(lunpppp,*) ' line off 3'
            write(lunpppp,*) ' line off 4'
            write(lunpppp,*) ' line off 5'
            write(lunpppp,*) ' line off 6'
            write(lunpppp,*) ' line off 7'
            write(lunpppp,*) ' line off 8'
            write(lunpppp,*) ' line off 9'
            write(lunpppp,*) ' win 1'
            write(lunpppp,*) ' yplot 1 2 3'
            write(lunpppp,*) ' loc 0.05 0.45 0.57 1.0'
            write(lunpppp,*) ' la 11 p 200.0 9.0 just left "Quad #1"'
            write(lunpppp,*) ' r x1 0.0 3500.0'
            write(lunpppp,*) ' r y1 0.0 10.0'
            write(lunpppp,*) ' la y Hardness'
            write(lunpppp,*) ' win 2'
            write(lunpppp,*) ' yplot 1 4 5'
            write(lunpppp,*) ' loc 0.47 0.45 0.99 1.0'
            write(lunpppp,*) ' la 12 p 200.0 9.0 just left "Quad #2"'
            write(lunpppp,*) ' r x2 0.0 3500.0'
            write(lunpppp,*) ' r y2 0.0 10.0'
            write(lunpppp,*) ' win 3'
            write(lunpppp,*) ' yplot 1 6 7'
            write(lunpppp,*) ' loc 0.05 0.05 0.57 0.60'
            write(lunpppp,*) ' la 13 p 200.0 9.0 just left "Quad #3"'
            write(lunpppp,*) ' r x3 0.0 3500.0'
            write(lunpppp,*) ' r y3 0.0 10.0'
            write(lunpppp,*) ' la y Hardness'
            write(lunpppp,*) ' la x Revolution'
            write(lunpppp,*) ' win 4'
            write(lunpppp,*) ' yplot 1 8 9'
            write(lunpppp,*) ' loc 0.47 0.05 0.99 0.60'
            write(lunpppp,*) ' la 14 p 200.0 9.0 just left "Quad #4"'
            write(lunpppp,*) ' r x4 0.0 3500.0'
            write(lunpppp,*) ' r y4 0.0 10.0'
            write(lunpppp,*) ' col off 1'
            write(lunpppp,*) ' la x Revolution'
        endif

!  Get an unused lun for the CalDB file

        err = 0
        status = 0
        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'AUG_GET_PNEXP POSITION 04'
            print *, 'STATUS FTGIOU =', status
        endif

!  Open the FITS file

        call ftopen (lun,db_file,0,blockn,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'AUG_GET_PNEXP POSITION 05'
            print *, 'STATUS FTOPEN =', status
            print *, db_file
        endif
        print *, 'db_file: ', db_file

!  Move to the correct extension

        if(err .eq. 0) then
            next = ichip + 1
            call ftmahd (lun,next,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'AUG_GET_PNEXP POSITION 06'
                print *, 'STATUS FTMAHD =', status
                print *, 'Could not find the spectrum extension'
            else

!  Read the FITS file header information

                call ftghbn (lun,36,nchan,tfields,ttype,tform, &
                    tunit,extname,varidat,status)
                write(extna,3011) ichip
 3011           format('QUIES_QUAD',i1)
                if(extname .ne. extna) then
                    print *, extname
                    print *, 'This was supposed to be a QUIES ext ',  &
                            'for quadrant: ', ichip
                    stop
                else
                    print *, 'Quadrant and QPB Extension'
                    print *, ichip, extname
                endif
                if(status .ne. 0) then
                    err = 1
                    print *, 'AUG_GET_PNEXP POSITION 07'
                    print *, 'STATUS FTGHPR =', status
                    print *, 'Failure on reading the header ', &
                            'information'
                else

!  Read in the column names

                    call ftgkns (lun,'TTYPE',1,36,colnam,ncols,status)
                    if(ncols .gt. 15000) then
                        print *, 'Too much data: ', ncols
                        print *, 'Limit selection to 15000'
                        ncols = 15000
                    endif

!  Extract the channel numbers

                    nt = 0
                    ng = 0
                    nb1 = 0
                    nb2 = 0
                    nb3 = 0
                    iobid = 0
                    iexpid = 0
                    iobjt = 0
                    idate = 0
                    irevl = 0
                    imode = 0
                    isubmode = 0
                    ifilt = 0
                    ittime = 0
                    ictime = 0
                    ikey = 0
                    iexpo = 0
                    irate = 0
                    iurate = 0
                    irate1 = 0
                    iurate1 = 0
                    irate2 = 0
                    iurate2 = 0
                    irate3 = 0
                    iurate3 = 0
                    irate4 = 0
                    iurate4 = 0
                    ihard1 = 0
                    iuhard1 = 0
                    ihard2 = 0
                    iuhard2 = 0
                    ihard3 = 0
                    iuhard3 = 0
                    ispec = 0
                    ispecoo = 0
                    iback = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'OBSID') iobid = icol
                        if(colnam(icol) .eq. 'EXP_ID') iexpid = icol
                        if(colnam(icol) .eq. 'OBJECT') iobjt = icol
                        if(colnam(icol) .eq. 'DATE') idate = icol
                        if(colnam(icol) .eq. 'REVOLUT') irevl = icol
                        if(colnam(icol) .eq. 'OBS_MODE') imode = icol
                        if(colnam(icol) .eq. 'SUB_MODE') isubmode = icol
                        if(colnam(icol) .eq. 'FILTER') ifilt = icol
                        if(colnam(icol) .eq. 'TOTAL_TIME') ittime = icol
                        if(colnam(icol) .eq. 'CLEAN_TIME') ictime = icol
                        if(colnam(icol) .eq. 'KEY') ikey = icol
                        if(colnam(icol) .eq. 'EXPO') iexpo = icol
                        if(colnam(icol) .eq. 'RATE') irate = icol
                        if(colnam(icol) .eq. 'RATE_ERR') iurate = icol
                        if(colnam(icol) .eq. 'RATE_1') irate1 = icol
                        if(colnam(icol) .eq. 'RATE_ERR_1')  &
                                iurate1 = icol
                        if(colnam(icol) .eq. 'RATE_2') irate2 = icol
                        if(colnam(icol) .eq. 'RATE_ERR_2')  &
                                iurate2 = icol
                        if(colnam(icol) .eq. 'RATE_3') irate3 = icol
                        if(colnam(icol) .eq. 'RATE_ERR_3')  &
                                iurate3 = icol
                        if(colnam(icol) .eq. 'RATE_4') irate4 = icol
                        if(colnam(icol) .eq. 'RATE_ERR_4')  &
                                iurate4 = icol
                        if(colnam(icol) .eq. 'HARD_1') ihard1 = icol
                        if(colnam(icol) .eq. 'HARD_ERR_1')  &
                                iuhard1 = icol
                        if(colnam(icol) .eq. 'HARD_2') ihard2 = icol
                        if(colnam(icol) .eq. 'HARD_ERR_2')  &
                                iuhard2 = icol
                        if(colnam(icol) .eq. 'HARD_3') ihard3 = icol
                        if(colnam(icol) .eq. 'HARD_ERR_3')  &
                                iuhard3 = icol
                        if(colnam(icol) .eq. 'SPECTRUM') ispec = icol
                        if(colnam(icol) .eq. 'SPECTRUM_OO')  &
                                ispecoo = icol
                        if(colnam(icol) .eq. 'BACKSCAL') iback = icol
                    enddo
!                    print *, colnam
!                    print *, ncols, iobid, iexpid, iobjt, idate, irevl, &
!                         imode, isubmode, ifilt, ittime, ictime, ikey,  &
!                         iexpo, irate,  &
!                         iurate, ihard2, iuhard2, ispec, ispecoo, iback
!                    stop
                    nn = 1
                    do i=1,nchan
                        nt = nt + 1
                        if(ikey .ne. 0) &
                            call ftgcvs(lun,ikey,i,1,1,' ', &
                                key(nn),anyf,status)
                        if((key(nn) .eq. 'G') .or.  &
                                (key(nn) .eq. ' ') .or.  &
                                (key(nn) .eq. 'O')) then
                            if(iobid .ne. 0) &
                                call ftgcvs(lun,iobid,i,1,1,' ', &
                                    obid(nn),anyf,status)
                            if(iexpid .ne. 0) &
                                call ftgcvs(lun,iexpid,i,1,1,' ', &
                                    expid(nn),anyf,status)
                            if(iobjt .ne. 0) &
                                call ftgcvs(lun,iobjt,i,1,1,' ', &
                                    objt(nn),anyf,status)
                            if(idate .ne. 0) &
                                call ftgcvd(lun,idate,i,1,1,0, &
                                    date(nn),anyf,status)
                            if(irevl .ne. 0) &
                                call ftgcvj(lun,irevl,i,1,1,0, &
                                    revl(nn),anyf,status)
                                if(anyf) revl(nn) = 0
                            if(ifilt .ne. 0) &
                                call ftgcvs(lun,ifilt,i,1,1,' ', &
                                    filt(nn),anyf,status)
                            if(ictime .ne. 0) &
                                call ftgcve(lun,ictime,i,1,1,0, &
                                    ctime(nn),anyf,status)
                                if(anyf) ctime(nn) = 0.0
                            if(ittime .ne. 0) &
                                call ftgcve(lun,ittime,i,1,1,0, &
                                    ttime(nn),anyf,status)
                                if(anyf) ttime(nn) = 0.0
                            if(iexpo .ne. 0) &
                                call ftgcve(lun,iexpo,i,1,1,0, &
                                    expo(nn),anyf,status)
                            if(iback .ne. 0)  &
                                call ftgcve(lun,iback,i,1,1,0, &
                                    back(nn),anyf,status)
                            if(imode .ne. 0) &
                                call ftgcvs(lun,imode,i,1,1,' ', &
                                    mode(nn),anyf,status)
                            if(isubmode .ne. 0) &
                                call ftgcvs(lun,isubmode,i,1,1,' ', &
                                    submode(nn),anyf,status)
                            if(irate .ne. 0) &
                                call ftgcve(lun,irate,i,1,1,0, &
                                    rate(nn),anyf,status)
                                if(anyf) rate(nn) = 0.0
                            if(iurate .ne. 0) &
                                call ftgcve(lun,iurate,i,1,1,0, &
                                    urate(nn),anyf,status)
                                if(anyf) urate(nn) = 0.0
                            if(irate1 .ne. 0)  &
                                call ftgcve(lun,irate1,i,1,1,0, &
                                    rate1(nn),anyf,status)
                                if(anyf) rate1(nn) = 0.0
                            if(iurate1 .ne. 0)  &
                                call ftgcve(lun,iurate1,i,1,1,0, &
                                    urate1(nn),anyf,status)
                                if(anyf) urate1(nn) = 0.0
                            if(irate2 .ne. 0)  &
                                call ftgcve(lun,irate2,i,1,1,0, &
                                    rate2(nn),anyf,status)
                                if(anyf) rate2(nn) = 0.0
                            if(iurate2 .ne. 0)  &
                                call ftgcve(lun,iurate2,i,1,1,0, &
                                    urate2(nn),anyf,status)
                                if(anyf) urate2(nn) = 0.0
                            if(irate3 .ne. 0)  &
                                call ftgcve(lun,irate3,i,1,1,0, &
                                    rate3(nn),anyf,status)
                                if(anyf) rate3(nn) = 0.0
                            if(iurate3 .ne. 0)  &
                                call ftgcve(lun,iurate3,i,1,1,0, &
                                    urate3(nn),anyf,status)
                                if(anyf) urate3(nn) = 0.0
                            if(irate4 .ne. 0)  &
                                call ftgcve(lun,irate4,i,1,1,0, &
                                    rate4(nn),anyf,status)
                                if(anyf) rate4(nn) = 0.0
                            if(iurate4 .ne. 0)  &
                                call ftgcve(lun,iurate4,i,1,1,0, &
                                    urate4(nn),anyf,status)
                                if(anyf) urate4(nn) = 0.0
                            if(ihard1 .ne. 0) &
                                call ftgcve(lun,ihard1,i,1,1,0, &
                                    hard1(nn),anyf,status)
                                if(anyf)  hard1(nn) = 0.0
                            if(iuhard1 .ne. 0) &
                                call ftgcve(lun,iuhard1,i,1,1,0, &
                                    uhard1(nn),anyf,status)
                                if(anyf)  uhard1(nn) = 0.0
                            if(ihard2 .ne. 0) &
                                call ftgcve(lun,ihard2,i,1,1,0, &
                                    hard2(nn),anyf,status)
                                if(anyf) hard2(nn) = 0.0
                            if(iuhard2 .ne. 0) &
                                call ftgcve(lun,iuhard2,i,1,1,0, &
                                    uhard2(nn),anyf,status)
                                if(anyf) uhard2(nn) = 0.0
                            if(ihard3 .ne. 0) &
                                call ftgcve(lun,ihard3,i,1,1,0, &
                                    hard3(nn),anyf,status)
                                if(anyf) hard3(nn) = 0.0
                            if(iuhard3 .ne. 0) &
                                call ftgcve(lun,iuhard3,i,1,1,0, &
                                    uhard3(nn),anyf,status)
                                if(anyf) uhard3(nn) = 0.0

!  Extract the spectrum and normalize by exposure and area

                            if (expo(nn) .gt. 100.0) then
                                if(ispec .ne. 0) then
                                    call ftgcve(lun,ispec,i,1,4096,0, &
                                            tspec,anyf,status)
                                    te1 = 0.0
                                    do ii=plc_lo(1),plc_lo(2)
                                        te1 = te1 + tspec(ii)
                                    enddo
                                    te2 = 0.0
                                    do ii=plc_hi(1),plc_hi(2)
                                        te2 = te2 + tspec(ii)
                                    enddo
                                    do ii=1,4096
                                        spec(ii,nn) = amscal*tspec(ii) &
                                                /(expo(nn)*back(nn))
                                    enddo
                                    if((i .eq. nchan) .and.  &
                                            (status .ne. 0)) status = 0
                                endif
                                rate(nn) = 1.0e6*rate(nn)
                                urate(nn) = 1.0e6*urate(nn)

!                                write(94,*) ichip, i, nn, revl(nn),  &
!                                        expo(nn), ttime(nn),  &
!                                        rate(nn), hard2(nn) 

!  Extract the OOT spectrum and normalize by exposure and area

                                if(ispecoo .ne. 0) then
                                    call ftgcve(lun,ispecoo,i,1,4096, &
                                            0,tspecoo,anyf,status)
                                    teo1 = 0.0
                                    do ii=plc_lo(1),plc_lo(2)
                                        teo1 = teo1 + tspecoo(ii)
                                    enddo
                                    teo2 = 0.0
                                    do ii=plc_hi(1),plc_hi(2)
                                        teo2 = teo2 + tspecoo(ii)
                                    enddo
                                    do ii=1,4096
                                        if(ii .lt. 600) then
                                            temp = 5.0 - 1.3* &
                                                    (0.005*ii - 0.7)
                                        elseif(ii .lt. 1400) then
                                            temp = 2.0 - 0.125* &
                                                    (0.005*ii - 3.0)
                                        else
                                            temp = 1.5
                                        endif
                                        specoo(ii,nn) = amscal* &
                                                tspecoo(ii)* &
                                                temp/(expo(nn)*back(nn))
                                    enddo
                                    if((i .eq. nchan) .and.  &
                                            (status .ne. 0)) status = 0
                                endif

!  Set the OOT scale factor

                                if(submode(nn) .eq.  &
                                        '        PrimeFullWindow') then
                                    tootsca = 0.063
                                elseif(submode(nn) .eq.  &
                                        'PrimeFullWindow') then
                                    tootsca = 0.063
                                elseif(submode(nn) .eq.  &
                                        'PrimeFullWindowExtended') then
                                    tootsca = 0.0232

!  Full frame extended 5 is well hidden

                                elseif(submode(nn) .eq. &
                                        'PrimeFullWindowExtended') then
                                    tootsca = 0.0163
                                else
                                    print *, 'AUG_GET_PNEXP POSITION 08'
                                    print *, 'Bad submode: ', &
                                            submode(nn)
                                endif
                                ootsca(nn) = tootsca

!  Find the normal and OOT counts in the low and high spectral bands

!                                if(nn .eq. 10) then 
!                                    print *, nn, te1, teo1, te2, teo2
!                                endif
                                te1 = 0.0
                                teo1 = 0.0
                                do ii=plc_lo(1),plc_lo(2)
                                    te1 = te1 + spec(ii,nn)
                                    teo1 = teo1 + specoo(ii,nn)
                                enddo
                                te2 = 0.0
                                teo2 = 0.0
                                do ii=plc_hi(1),plc_hi(2)
                                    te2 = te2 + spec(ii,nn)
                                    teo2 = teo2 + specoo(ii,nn)
                                enddo
                                temp = rate(nn)
                                rate(nn) = te1 + te2 - &
                                        tootsca*(teo1 + teo2)
!                                print *, nn, temp, rate(nn), &
!                                        temp/rate(nn)
                                temp = hard2(nn)
                                hard2(nn) = (te2 - tootsca*teo2)/ &
                                        (te1 - tootsca*teo1)
!                                print *, nn, temp, hard2(nn), &
!                                        temp/hard2(nn)
                                nn = nn + 1
                            endif
                        endif
                    enddo
                    nn = nn - 1
                    if(status .ne. 0) then
                        print *, 'AUG_GET_PNEXP POSITION 09'
                        print *, 'Missing column in FWC info file'
                        print *, 'STATUS: ', status
                        return
                    endif
                endif
            endif
        endif
!        print *, 'ng, nb: ', ng, nb
        call ftclos (lun,status)

!  Set the normal filtering limits

        d1 = -9.0
        d2 = 0.0
        hardmin = 3.0
        hardmax = 7.0
        ratemin = 1.0e-3
        ratemax = 3.0e-3
        expmin = 5000.0
        expmax = 40000.0

!  Sort through the archived observation data collecting
!  the specifics on possible observations that can be used
!  for the augmentation.

        nobs = 0
        do i=1,nn
!            print 3999, p_flg, ichip, expo(i), rate(i), ratemin, &
!                ratemax, hard(i), hardmin, hardmax, submode(i), modein
! 3999       format(2i4,f8.1,3f8.3,3f8.1,3x,2a25)
!            print *, submode(i), modein

            if((expo(i) .gt. 0.0) .and. (ttime(i) .gt. 0.0)) then
              if((expo(i) .ge. expmin) .and.  &
                    (expo(i) .lt. expmax) .and. &
                    (rate(i) .gt. ratemin) .and.  &
                    (rate(i) .lt. ratemax) .and.  &
                    (hard2(i) .gt. hardmin) .and. &
                    (hard2(i) .lt. hardmax) .and. &
                    (expo(i) .gt. 0.8*ttime(i))) then

!                write(94,*) ichip, i, revl(i), expo(i), ttime(i),  &
!                        rate(i), hard2(i) 

!  Write out all the data for plotting if selected by the 
!  rate, exposure, and hardness criteria

                if(p_flg .eq. 1) then
                    if(ichip .eq. 1) then
                        write(lunpppp,2820) revl(i), &
                            hard2(i), uhard2(i), d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2820                   format(1x,i6,2f6.2,14f4.0)
                        write(lunppp,2720) revl(i), &
                            rate(i), urate(i), d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2720                   format(1x,i6,2f10.6,14f4.0)
                        write(lunp,2120) rate(i), urate(i), &
                            hard2(i), uhard2(i), d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2120                   format(1x,2f10.6,2f6.2,22f4.0)
                    elseif(ichip .eq. 2) then
                        write(lunpppp,2830) revl(i),  &
                            d1, d2, d1, d2,  &
                            hard2(i), uhard2(i), d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2830                   format(1x,i6,4f4.0,2f6.2,10f4.0)
                        write(lunppp,2730) revl(i),  &
                            d1, d2, d1, d2,  &
                            rate(i), urate(i), d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2730                   format(1x,i6,4f4.0,2f10.6,10f4.0)
                        write(lunp,2130) rate(i), urate(i), &
                            d1, d2, d1, d2, &
                            hard2(i), uhard2(i), d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2130                   format(1x,2f10.6,4f4.0,2f6.2,18f4.0)
                    elseif(ichip .eq. 3) then
                        write(lunpppp,2840) revl(i),  &
                            d1, d2, d1, d2,  &
                            d1, d2, d1, d2, &
                            hard2(i), uhard2(i), d1, d2, &
                            d1, d2, d1, d2
 2840                   format(1x,i6,8f4.0,2f6.2,6f4.0)
                        write(lunppp,2740) revl(i),  &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2,  &
                            rate(i), urate(i), d1, d2, &
                            d1, d2, d1, d2
 2740                   format(1x,i6,8f4.0,2f10.6,6f4.0)
                        write(lunp,2140) rate(i), urate(i), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            hard2(i), uhard2(i), d1, d2, &
                            d1, d2, d1, d2
 2140                   format(1x,2f10.6,8f4.0,2f6.2,14f4.0)
                    elseif(ichip .eq. 4) then
                        write(lunpppp,2850) revl(i),  &
                            d1, d2, d1, d2,  &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            hard2(i), uhard2(i), d1, d2
 2850                   format(1x,i6,12f4.0,2f6.2,2f4.0)
                        write(lunppp,2750) revl(i),  &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2,  &
                            d1, d2, d1, d2, &
                            rate(i), urate(i), d1, d2
 2750                   format(1x,i6,12f4.0,2f10.6,2f4.0)
                        write(lunp,2150) rate(i), urate(i), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            hard2(i), uhard2(i), d1, d2
 2150                   format(1x,2f10.6,12f4.0,2f6.2,10f4.0)
                    endif
                endif

!  Check to see whether the observation CCD is in an odd state
!  and select whether the data will be kept or not

                ikeep = 1

!  Keep the data set so transfer the information
 
                if(ikeep .eq. 1) then
                    nobs = nobs + 1
                    obid(nobs) = obid(i)
                    expid(nobs) = expid(i)
                    objt(nobs) = objt(i)
                    date(nobs) = date(i)
                    revl(nobs) = revl(i)
                    filt(nobs) = filt(i)
                    mode(nobs) = mode(i)
                    submode(nobs) = submode(i)
                    ttime(nobs) = ttime(i)
                    ctime(nobs) = ctime(i)
                    back(nobs) = back(i)
                    expo(nobs) = expo(i)
                    rate(nobs) = rate(i)
                    urate(nobs) = urate(i)
                    hard2(nobs) = hard2(i)
                    uhard2(nobs) = uhard2(i)
!            print *, (spec(ii,i),ii=40,50)
                    do ii=1,4096
                        spec(ii,nobs) = spec(ii,i)
                        specoo(ii,nobs) = specoo(ii,i)
                    enddo
                endif
              else
                  nb2 = nb2 + 1
              endif
            else
                nb3 = nb3 + 1
            endif
        enddo
!  Set the energies of the bins

        do i=1,4096
            energ(i) = 0.005*(i - 1)
        enddo

!  Get some temporary values for the average rate and hardness
!  errors for the corner data

        murate = 0.0
        muhard = 0.0
        do i=1,nobs
            murate = murate + urate(i)
            muhard = muhard + uhard2(i)
        enddo
        murate = murate/nobs
        muhard = muhard/nobs

!  Find the closest c_tim secs

        if(c_tim .eq. 0.0) then
            print *, 'AUG_GET_PNEXP POSITION 11'
            print *, 'No exposure length provided'
            stop
        else
            if((c_dst(1) .eq. 0.0) .and. (c_dst(2) .eq. 0.0) .and. &
                    (c_dst(3) .eq. 0.0)) then
                print *, 'AUG_GET_PNEXP POSITION 12'
                print *, 'No distance scalings'
                stop
            else

!  First, set the distance in parameter space

                do i=1,nobs
                    if(c_dst(1) .ne. 0.0) then
                        rdist(i) = (rate(i)-c_val(1))/c_dst(1)
                    else
                        rdist(i) = 0.0
                    endif
                    if(c_dst(2) .ne. 0.0) then
                        hdist(i) = (hard2(i)-c_val(2))/c_dst(2)
                    else
                        hdist(i) = 0.0
                    endif
                    if(c_dst(3) .ne. 0.0) then
                        ddist(i) = (revl(i)-c_val(3))/c_dst(3)
                    else
                        ddist(i) = 0.0
                    endif
                    dist(i) = sqrt(rdist(i)**2 + hdist(i)**2 + &
                        ddist(i)**2)
                enddo
            endif

!  Sort through the CalDB data to order them by the distance
!  from the observation values.

            do i=1,nobs
                ord(i) = i
                tdist(i) = dist(i)
            enddo
            do ii=nobs-1,1,-1
                do i=1,ii
                    if(tdist(i) .gt. tdist(i+1)) then
                        td = tdist(i)
                        tdist(i) = tdist(i+1)
                        tdist(i+1) = td
                        itd = ord(i)
                        ord(i) = ord(i+1)
                        ord(i+1) = itd
                    endif
                enddo
            enddo

!  Now accumulate spectra until a limiting exposure is reached

            cum_exp = 0.0
            n = 0
            ave_rate = 0.0
            do while ((cum_exp .lt. c_tim) .and. (n .lt. nobs))
                n = n + 1
                cum_exp = cum_exp + expo(ord(n))
                ave_rate = ave_rate + rate(ord(n))*expo(ord(n))
                if(p_flg .eq. 1) then
                    if(ichip .eq. 1) then
                        write(lunpppp,2825) revl(ord(n)), &
                            d1, d2, hard2(ord(n)), uhard2(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2825                   format(1x,i6,2f4.0,2f6.2,12f4.0)
                        write(lunppp,2725) revl(ord(n)), &
                            d1, d2, rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2725                   format(1x,i6,2f4.0,2f10.6,12f4.0)
                        write(lunp,2125) rate(ord(n)), urate(ord(n)), &
                            d1, d2, hard2(ord(n)), uhard2(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2125                   format(1x,2f10.6,2f4.0,2f6.2,20f4.0)
                    elseif(ichip .eq. 2) then
                        write(lunpppp,2835) revl(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, hard2(ord(n)), uhard2(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2835                   format(1x,i6,6f4.0,2f6.2,8f4.0)
                        write(lunppp,2735) revl(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2735                   format(1x,i6,6f4.0,2f10.6,8f4.0)
                        write(lunp,2135) rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, hard2(ord(n)), uhard2(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2
 2135                   format(1x,2f10.6,6f4.0,2f6.2,16f4.0)
                    elseif(ichip .eq. 3) then
                        write(lunpppp,2845) revl(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, hard2(ord(n)), uhard2(ord(n)), &
                            d1, d2, d1, d2
 2845                   format(1x,i6,10f4.0,2f6.2,4f4.0)
                        write(lunppp,2745) revl(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2
 2745                   format(1x,i6,10f4.0,2f10.6,4f4.0)
                        write(lunp,2145) rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, hard2(ord(n)), uhard2(ord(n)), &
                            d1, d2, d1, d2
 2145                   format(1x,2f10.6,10f4.0,2f6.2,12f4.0)
                    elseif(ichip .eq. 4) then
                        write(lunpppp,2855) revl(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, hard2(ord(n)), uhard2(ord(n))
 2855                   format(1x,i6,14f4.0,2f6.2)
                        write(lunppp,2755) revl(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, rate(ord(n)), urate(ord(n))
 2755                   format(1x,i6,14f4.0,2f10.6)
                        write(lunp,2155) rate(ord(n)), urate(ord(n)), &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, d1, d2, &
                            d1, d2, hard2(ord(n)), uhard2(ord(n))
 2155                   format(1x,2f10.6,14f4.0,2f6.2,8f4.0)
                    endif
                endif
            enddo
            ave_rate = ave_rate/cum_exp
            print 4010, cum_exp
 4010       format('Exposure: ',f10.1)
            print 4020, ave_rate
 4020       format('Exposure weighted count rate: ',f10.6)
            print 4030, c_val(1)
 4030       format('Observation count rate: ',f10.6)
            print 4035, c_val(2)
 4035       format('Observation hardness ratio: ',f10.6)
            nb1 = nobs - n
            print *, 'Total Obs, Good Observations, Selected Obs, ', &
                    'Not Selected Obs: ', nt, nn, n, nb1
            print *, 'Out of Range Obs, Bad Exposure Obs: ', &
                    nb2, nb3
        endif

        call ftfiou (lun,status)

!  Actually put together the spectrum

        do ii=1,4096
            aug(ii) = 0.0
        enddo
        aug_exp = 0.
        do i=1,n
            do ii=1,4096
                aug(ii) = aug(ii) + (spec(ii,ord(i)) - &
                        ootsca(ord(i))*specoo(ii,ord(i)))*expo(ord(i))
            enddo
            aug_exp = aug_exp + expo(ord(i))
        enddo
        do ii=1,4096
            aug(ii) = aug(ii)/aug_exp
        enddo

        te1 = 0.0
        te2 = 0.0
        do i=plc_lo(1),plc_lo(2)
            te1 = te1 + aug(i) 
        enddo
        do i=plc_hi(1),plc_hi(2)
            te2 = te2 + aug(i) 
        enddo

        print 4040, te1+te2
 4040   format('Augmented count rate: ',f10.6)
        print 4050, te2/te1
 4050   format('Augmented hardness ratio: ', f8.3)
        print 4060, aug_exp
 4060   format('Augmented exposure: ',f10.1)
        if(p_flg .eq. 1) then
            do i=1,4096
                if(ichip .eq. 1) then
                    write(lunpp,2225) energ(i), 1.0e6*aug(i), &
                            d1, d1, d1 
 2225               format(1x,f10.3,f10.5,3f4.0)
                elseif(ichip .eq. 2) then
                    write(lunpp,2235) energ(i), d1, 1.0e6*aug(i), &
                            d1, d1
 2235               format(1x,f10.3,f4.0,f10.5,2f4.0)
                elseif(ichip .eq. 3) then
                    write(lunpp,2245) energ(i), d1, d1, &
                            1.0e6*aug(i), d1
 2245               format(1x,f10.3,2f4.0,f10.5,f4.0)
                elseif(ichip .eq. 4) then
                    write(lunpp,2255) energ(i), d1, d1, d1, &
                            1.0e6*aug(i)
 2255               format(1x,f10.3,3f4.0,f10.5)
                endif
            enddo
        endif
        call ftclos (lun,status)
        call ftfiou (lun,status)

        return
        end subroutine aug_get_pnexp



        SUBROUTINE DCVSRT(COVAR,NCVM,MA,LISTA,MFIT)

!C  Repacks the covarience matrix for MQMLMN
!
!************************ FFORM VERSION 1.0 ************  1-MAR-88 14:40
!
!A  author : Press et al.      date:
!U  update : J. Bloch          date:
!U  update : SLS               date:  1-MAR-1988 14:39
!
!T  status: not tested
!
!   general description
!G  DCVSRT is the REAL*8 version of routine COVSRT found in "Numerical
!G  Recipes" by Press et al. See p 515 for details. The only difference
!G  is that array COVAR is REAL*8 instead of single precision.  Given
!G  the covarience matrix COVAR of a fit MFIT of MA total parameters,
!G  and there ordering LISTA, repack the covarience matrix to the true
!G  order of the parameters.  Elements associated with fixed parameters
!G  will be zero.  NCVM is the physical dimension of COVAR
!
!   call_var.          type I/O description
!P  MA                  I4  I   total number of parameters
!P  MFIT                I4  I   number of active parameters
!P  NCVM                I4      dimension of COVAR array
!P  COVAR               R8  I/O covarience matrix
!P  LISTA               I4  I   position vector

        IMPLICIT NONE

        INTEGER :: I, J, MA, MFIT, LISTA(MFIT), NCVM
        DOUBLE PRECISION :: COVAR(NCVM,NCVM), SWAP

!  Zero all elements below the diagonal

        DO 12 J=1,MA-1
            DO 11 I=J+1,MA
                COVAR(I,J) = 0.
   11       ENDDO
   12   ENDDO

!  Repack off-diagonal elements of fit into correct locations
!  below the diagonal

        DO 14 I=1,MFIT-1
            DO 13 J=I+1,MFIT
                IF(LISTA(J) .GT. LISTA(I)) THEN
                    COVAR(LISTA(J),LISTA(I)) = COVAR(I,J)
                ELSE
                    COVAR(LISTA(I),LISTA(J)) = COVAR(I,J)
                ENDIF
   13       ENDDO
   14   ENDDO

!  Temporarily store origial elements in top row, and zero the diagonal

        SWAP = COVAR(1,1)

        DO 15 J=1,MA
            COVAR(1,J) = COVAR(J,J)
            COVAR(J,J) = 0.
   15   ENDDO

        COVAR(LISTA(1),LISTA(1)) = SWAP

!  Now sort the elements into proper order on diagonal

        DO 16 J=2,MFIT
            COVAR(LISTA(J),LISTA(J)) = COVAR(1,J)
   16   ENDDO

!  Finally, fill in above the diagonal by symmetry

        DO 18 J=2,MA
            DO 17 I=1,J-1
                COVAR(I,J) = COVAR(J,I)
   17       ENDDO
   18   ENDDO

        RETURN
        END subroutine dcvsrt


        SUBROUTINE DGAUSJ(A,N,NP,B,M,MP,IERR)

!C  Numerical Routines linear equation solution routine
!
!************************ FFORM VERSION 1.0 ************  1-MAR-88 15:03
!
!A  author : Press et al.      date:
!U  update : J. Bloch          date:  6-SEP-1987
!U  update : SLS               date:  1-MAR-1988 14:42
!
!T  status: not tested
!
!   general description
!G  Linear equation solution by Gauss-Jordan elimination.  A is an
!G  input matrix of N x N elements, stored in an array of physical
!G  dimensions NP x NP.  B is an input matrix of N x M containing the
!G  M right-hand side vectors, stored in an array of physical
!G  dimensions NP x MP.  On output, A is replaced by its matrix
!G  inverse, and B is replaced by the corresponding set of solution
!G  vectors.
!G  Modified by J. Bloch to be a double precision matrix version of the
!G  subroutine GAUSSJ in the book NUMERICAL RECIPES by Press et al.
!
!   call_var.          type I/O description
!P  A                   R8  I/O Updated REAL*8 Matrix which will be
!P                              replaced by it's inverse.
!P  N                   I4  I   actual order of matrix A
!P  NP                  I4      actual dimenstions of array A.
!P  B                   R8  I/O Updated array containing M right hand
!P                                  side vectors. Upon return, B
!P                                  contains the solution vectors.
!P  M                   I4  I   number of right hand side vectors.
!P  MP                  I4  I   actual second dimension size of B.
!P  IERR                I4  I/O = 0 no errors
!P                              = 3 singular matrix

        IMPLICIT NONE

!       PARAMETER NMAX=200

        INTEGER :: I, ICOL, IERR, IROW, J, K, L, LL, M, MP, N, NP
        INTEGER :: IPIV(200), INDXR(200), INDXC(200)

        DOUBLE PRECISION :: A(NP,NP), B(NP,MP), BIG, DUM, PIVINV

        IERR = 0
        DO 11 J=1,N
            IPIV(J) = 0
   11   ENDDO

!  Start the main loop over the columns to be reduced

        I = 0
        DO WHILE ((I .LT. N) .AND. (IERR .EQ. 0))
            I = I + 1
            BIG = 0.D0

!  Start the outer loop of the search for the pivot element

            J = 0
            DO WHILE ((J .LT. N) .AND. (IERR .EQ. 0))
                J = J + 1
                IF(IPIV(J) .NE. 1) THEN
                    K = 0
                    DO WHILE ((K .LT. N) .AND. (IERR .EQ. 0))
                        K = K + 1
                        IF (IPIV(K) .EQ. 0) THEN
                            IF (ABS(A(J,K)) .GE. BIG) THEN
                                BIG = DABS(A(J,K))
                                IROW = J
                                ICOL = K
                            ENDIF
                        ELSE IF (IPIV(K) .GT. 1) THEN
                            IERR = 3
                            PRINT *, 'DGAUSJ, singular matrix, pos 1'
                        ENDIF
                    ENDDO
                ENDIF
            ENDDO
            IF(IERR .EQ. 0) THEN
                IPIV(ICOL) = IPIV(ICOL) + 1

!  We now have the pivot element, so we interchange rows, if needed,
!  to put the pivot element on the diagonal.  The columns are not
!  physically interchanged, only relabeled: INDX(I), the column of the
!  Ith pivot element, is the Ith column that is reduced, while INDXR(I)
!  is the row in which that pivot element was origially located.  If
!  INDXR(I) =/ INDXC(I) there is an implied column interchange.  With
!  this form of bookkeeping, the solution B's will end up in the correct
!  order, and the inverse matrix will be scrambled by columns.

                IF (IROW .NE. ICOL) THEN
                    DO 14 L=1,N
                        DUM = A(IROW,L)
                        A(IROW,L) = A(ICOL,L)
                        A(ICOL,L) = DUM
   14               ENDDO

                    DO 15 L=1,M
                        DUM = B(IROW,L)
                        B(IROW,L) = B(ICOL,L)
                        B(ICOL,L) = DUM
   15               ENDDO
                ENDIF

!  We are now ready to divide the pivot row by the pivot element
!  located at IROW and ICOL

                INDXR(I) = IROW
                INDXC(I) = ICOL
                IF (A(ICOL,ICOL) .EQ. 0.) THEN
                    IERR = 3
                    PRINT *, 'DGAUSJ, singular matrix, pos 2'
                ELSE
                    PIVINV = 1.D0/A(ICOL,ICOL)
                    A(ICOL,ICOL) = 1.D0
                    DO 16 L=1,N
                        A(ICOL,L) = A(ICOL,L)*PIVINV
   16               ENDDO

                    DO 17 L=1,M
                        B(ICOL,L) = B(ICOL,L)*PIVINV
   17               ENDDO

!  Next we reduce the rows, except for the pivot row

                    DO 21 LL=1,N
                        IF(LL .NE. ICOL)THEN
                            DUM = A(LL,ICOL)
                            A(LL,ICOL) = 0.
                            DO 18 L=1,N
                                A(LL,L) = A(LL,L) - A(ICOL,L)*DUM
   18                       ENDDO
                            DO 19 L=1,M
                                B(LL,L) = B(LL,L) - B(ICOL,L)*DUM
   19                       ENDDO
                        ENDIF
   21               ENDDO
                ENDIF
            ENDIF
   22   ENDDO

!  This is the end of the main loop over the columns of the reduction.
!  Now unscramble the solution in view of the column interchanges.
!  We do this by interchanging pairs of columns in the order that the
!  permutation was built up.

        IF(IERR .EQ. 0) THEN
            DO 24 L=N,1,-1
                IF(INDXR(L).NE.INDXC(L))THEN
                    DO 23 K=1,N
                        DUM = A(K,INDXR(L))
                        A(K,INDXR(L)) = A(K,INDXC(L))
                        A(K,INDXC(L)) = DUM
   23               ENDDO
                ENDIF
   24       ENDDO
        ENDIF

        RETURN
        END subroutine dgausj


        subroutine DRADRS(DRAD,ODRADRS)

!C    'Resets' radians to the interval  0 .. 2 * pi
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 16:58
!
!A  author : KOD        date: 19-JUL-1989 18:31
!U  update : KOD        date: 29-MAR-1990 15:08
!
!T  status: tested
!
!   general description
!G  'Resets' radians to the interval  0  .. 2 * pi.
!
!   call_var.          type I/O description
!P  DRAD                R8  I   radians (may be negative)
!P  ODRADRS             R8    O radians inside the interval  0 .. 2 * pi
!
!   include_block name          description
!I  CGENL.CMN          general parameter common block
!I  IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
!R  RESVAL              SR      'resets' periodic variables
!R  ERRSTC              SR      continue error stack for one level

        IMPLICIT NONE
!        INCLUDE 'cgenl.cmn'
!        INCLUDE 'igenl.inc'

        DOUBLE PRECISION :: DRAD, DXNEW, ODRADRS
        DOUBLE PRECISION :: DXMIN = 0.D0
        DOUBLE PRECISION :: DXMAX
        DOUBLE PRECISION :: DXOLD
        INTEGER :: ISTAT = 0
        INTEGER :: IERR = 0
        CHARACTER(len=8) :: RTNAME = 'DRADRS'

        DXMAX = D2PI
        DXOLD = DRAD
        CALL RESVAL(DXMIN,DXMAX,DXOLD,DXNEW,ISTAT)
        if (ISTAT.gt.1) then
            IERR = max(IERR,ISTAT)
            call ERRSTC(RTNAME,10)
        endif

        if (IERR .lt. 2) ODRADRS = DXNEW

        RETURN
        end subroutine DRADRS


        SUBROUTINE ERRSTC(RTNAME,NUM)

        INTEGER NUM
        CHARACTER*8 RTNAME

        Print *, 'Error in Routine: ', rtname
        Print *, 'Error Number: ', num

        RETURN
        end subroutine errstc


        subroutine EQUECL(DJD,LMEAN,LDIR,DLOLD,DBOLD,DLNEW,DBNEW,IERR)

!C    Converts coordinates between equatorial and ecliptical system
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:02
!
!A  author : KOD        date: 19-JUL-1989 18:31
!U  update : KOD        date: 29-MAR-1990 15:13
!
!T  status: tested
!
!   general description
!G  Subroutine to convert coordinates between the equatorial and the
!G  ecliptical system (LII,BII).
!G
!G  To convert the TRUE positions, use the MEAN obliquity of the
!G  ecliptic (normal case);
!G  for the APPARENT positions (affected by aberration and nutation)
!G  use the TRUE obliquity.
!G  If the equatorial coordinates refer to a standard equinox
!G  (1950, 2000, ...), the obliquity of the ecliptic for this
!G  epoch should be used.
!G
!G  (Jean Meeus, 'Astronomical Formulae for Calculators',
!G                third edition, 1985, p.43-44)
!
!   call_var.          type I/O description
!P  DJD                 R8  I   Julian Day for the conversion
!P
!P  LMEAN               L4  I   mean or true obliquity of the ecliptic:
!P                                LMEAN = .true.  : mean obliquity
!P                                LMEAN = .false. : true obliquity
!P
!P  LDIR                L4  I   transformation direction:
!P                                LDIR = .true.  : equ --> ecl
!P                                LDIR = .false. : equ <-- ecl
!P
!P  DLOLD               R8  I   old longitude  (radians)
!P  DBOLD               R8  I   old latitude   (radians)
!P
!P  DLNEW               R8    O new longitude  (radians)
!P  DBNEW               R8    O new latitude   (radians)
!P
!P  IERR                I4    O = 0 no error
!P                              = 1 warning
!P                              = 2 error
!P                              = 3 severe error
!
!   include_block name          description
!I  CGENL.CMN          general parameter common block
!I  IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
!R  EULER1              SR      general coordinate transformation
!R  JDEPS1              SR      calculates mean/true obl. of the ecl.
!R  WRERR               SR      formats error code in standard form
!R  ERRSTC              SR      continue error stack for one level
!
        IMPLICIT NONE
!        INCLUDE 'cgenl.cmn'
!        INCLUDE 'igenl.inc'

        DOUBLE PRECISION :: DJD, DLOLD, DBOLD, DLNEW, DBNEW,&
            D1, D2, D3, DEPS, DPHI1, DTHETA1, DPHI2, DTHETA2, DJ
        INTEGER :: IERR
        INTEGER :: ISTAT = 0
        LOGICAL :: LMEAN, LDIR, LMN
        CHARACTER(len=8) :: RTNAME = 'EQUECL'
        CHARACTER(len=80) :: LOG

        IERR = 0

        if ((DLOLD .lt. 0.D0) .or. (DLOLD .GT. D2PI)) then
            IERR = 1
            LOG = 'Warning: unusual value for DLOLD'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if (dabs(DBOLD) .gt. DPID2) then
            IERR = max(IERR,2)
            LOG = 'Error in input parameter DBOLD'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if ((DJD .lt. 0.D0) .or. (DJD .gt. 3.D6)) then
            IERR = max(IERR,2)
            LOG = 'Error in the input parameter DJD'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if  (IERR.lt.2)  then

            DJ = DJD
            LMN = LMEAN
            call JDEPS1(DJ,LMN,DEPS,ISTAT)

            if (ISTAT.gt.1) then
                IERR = max(IERR,ISTAT)
                call ERRSTC(RTNAME,10)
            endif
        endif

        if  (IERR.lt.2)  then

            D1 = 0.D0
            D2 = DEPS
            D3 = 0.D0

            DPHI1   = DLOLD
            DTHETA1 = DPID2 - DBOLD

            if (LDIR) then
                call EULER1(D1,D2,D3,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
            else
                call EULER1(-D3,-D2,-D1,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
            endif

            if (ISTAT.gt.1) then
                IERR = max(IERR,ISTAT)
                call ERRSTC(RTNAME,20)
            else
                DLNEW = DPHI2
                DBNEW = DPID2 - DTHETA2
            endif
        endif

        return
        end subroutine equecl


        subroutine EQUGAL(DJD,LDIR,DLOLD,DBOLD,DLNEW,DBNEW,IERR)
!
!    Converts coordinates between equatorial and new galactic system
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:02
!
!A  author : KOD        date: 19-JUL-1989 19:45
!U  update : KOD        date: 20-MAR-1990 14:56
!
!T  status: tested
!
!   general description
!G  Subroutine to convert coordinates between the equatorial and the
!G  new galactic system (LII,BII).
!G  Parameters extracted from:
!G     F.C.Bertiau, 'Programmes for pocket calculators HP-67
!G                   and HP-97 in the field of theoretical and
!G                   observational astronomy',
!G                   1977 Leuven University Press, 'ASTR. COOR. 2'
!
!   call_var.          type I/O description
!P  DJD                 R8  I   Julian Day for the conversion
!P
!P  LDIR                L4  I   transformation direction:
!P                                LDIR = .true.  : equ --> gal II
!P                                LDIR = .false. : equ <-- gal II
!P
!P  DLOLD               R8  I   old longitude  (radians)
!P  DBOLD               R8  I   old latitude   (radians)
!P
!P  DLNEW               R8    O new longitude  (radians)
!P  DBNEW               R8    O new latitude   (radians)
!P
!P  IERR                I4    O = 0 no error
!P                              = 1 warning
!P                              = 2 error
!P                              = 3 severe error
!
!   include_block name          description
!I  CGENL.CMN          general parameter common block
!I  IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
!R  EULER1              SR      general coordinate transformation
!R  WRERR               SR      formats error code in standard form
!R  ERRSTC              SR      continue error stack for one level

        IMPLICIT NONE
  
!        INCLUDE 'cgenl.cmn'
!        INCLUDE 'igenl.inc'

        DOUBLE PRECISION :: DJD, DLOLD, DBOLD, DLNEW, DBNEW, &
            D1, D2, D3, DPHI1, DTHETA1, DPHI2, DTHETA2, DT
!  julian day for 1950.01.01 / 00:00:00
        double precision :: DJD50 = 2433282.5D0
        double precision :: D11 = 4.92619181375D0
        double precision :: D21 = 1.09257611174D0
        double precision :: D31 = 5.70722665400D0
        double precision :: D12 = 5.82362268000D-7
        double precision :: D22 = 2.59857718339D-7
        double precision :: D32 = 0.64223753852D-7
        INTEGER :: IERR
        INTEGER :: ISTAT = 0
        LOGICAL :: LDIR
        CHARACTER(len=8) :: RTNAME = 'EQUGAL'
        CHARACTER*80  LOG

        IERR = 0

        if ((DLOLD .lt. 0.D0) .or. (DLOLD .GT. D2PI)) then
            IERR = 1
            LOG = 'Warning: unusual value for DLOLD'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if (dabs(DBOLD) .gt. DPID2) then
            IERR = max(IERR,2)
            LOG = 'Error in input parameter DBOLD'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if (IERR .lt. 2) then
            DT = DJD - DJD50
            ISTAT = 0
            if (dabs(DT).gt.5.D4) ISTAT = 1
            if (dabs(DT).gt.5.D6) ISTAT = 2
            if (ISTAT.gt.1) then
                IERR = max(IERR,ISTAT)
                LOG = 'Error in input parameter DJD'
                call WRERR(RTNAME,IERR,LOG)
            endif
        endif

        if (IERR .lt. 2)  then

            D1 = D11 + D12 * DT
            D2 = D21 + D22 * DT
            D3 = D31 + D32 * DT

            DPHI1   = DLOLD
            DTHETA1 = DPID2 - DBOLD

!            print *, LDIR,D1,D2,D3,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT
            if (LDIR) then
                call EULER1(D1,D2,D3,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
            else
                call EULER1(-D3,-D2,-D1,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT)
            endif
!            print *, LDIR,D1,D2,D3,DPHI1,DTHETA1,DPHI2,DTHETA2,ISTAT

            if (ISTAT .gt. 1) then
                IERR = max(IERR,ISTAT)
                call ERRSTC(RTNAME,10)
            else
                DLNEW = DPHI2
                DBNEW = DPID2 - DTHETA2
            endif

        endif

        RETURN
        END subroutine equgal


        subroutine EULER1(D1,D2,D3,DPH1,DTH1,DPH2,DTH2,IERR)

!C    General transformation of spherical coordinates
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:03
!
!A  author : KOD        date: 15-JUL-1989 18:31
!U  update : KOD        date: 20-MAR-1990 14:57
!
!T  status: tested
!
!   general description
!G
!G  Subroutine to convert spherical coordinates between coordinate
!G  systems which are rotated with respect to each other.
!G
!G  This subroutine converts the azimuthal and polar angles
!G  (DPH1,DTH1), given in mathematical notation and in radians,
!G  into the corresponding angles with respect to a coordinate
!G  system which orientation is given by the EULER angles (D1,D2,D3)
!G  in the following way:
!G
!G  The original cartesian coordinate system (X0,Y0,Z0) is rotated
!G  in three steps:
!G
!GT   1) rotation around the Z0-axis by the angle D1  --->  (X1,Y1,Z1)
!GT   2) rotation around the X1-axis by the angle D2  --->  (X2,Y2,Z2)
!GT   3) rotation around the Z2-axis by the angle D3  --->  (X3,Y3,Z3)
!G
!G  The calculation is accelerated by using differences of angles.
!
!   call_var.          type I/O description
!P  D1                  R8  I   EULER angle 1   (radians, -2 pi .. 2 pi)
!P  D2                  R8  I   EULER angle 2   (radians, -2 pi .. 2 pi)
!P  D3                  R8  I   EULER angle 3   (radians, -2 pi .. 2 pi)
!P
!P  DPH1                R8  I   original  azimuthal angle (rad, 0..2 pi)
!P  DTH1                R8  I   original    polar   angle (rad, 0..pi)
!P
!P  DPH2                R8    O resulting azimuthal angle (rad, 0..2 pi)
!P  DTH2                R8    O resulting   polar   angle (rad, 0..pi)
!P
!P  IERR                I4    O = 0 no error
!P                              = 1 warning
!P                              = 2 error
!P                              = 3 severe error
!
!   include_block name          description
!I  CGENL.CMN          general parameter common block
!I  IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description
!R  REPOL1              SR      cartesian to polar coordinate conversion
!R  DRADRS              R8      'resets' radians
!R  WRERR               SR      formats error code in standard form


        IMPLICIT NONE


!        INCLUDE 'cgenl.cmn'
!        INCLUDE 'igenl.inc'

        DOUBLE PRECISION :: D1, D2, D3, DPH1, DPH2, DTH1, DTH2, &
            DST, DCT, DH, DS1, DC1, DS2, DC2, DX, DY, DZ, &
            DR, DPHI, DTHETA, IDRADRS, ODRADRS
        INTEGER :: IERR
        CHARACTER(len=8) :: RTNAME = 'EULER1'
        CHARACTER(len=80) :: LOG

        IERR = 0

        if ((max(dabs(D1),dabs(D2),dabs(D3)) .gt. D2PI) .or. &
                (DPH1 .lt. 0.D0)) then
            IERR = max(IERR,1)
            LOG = 'Warning: Unusual input parameters'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if ((DTH1 .lt. 0.D0) .or. (DTH1 .gt. DPI)) then
            IERR = max(IERR,2)
            LOG = 'Error in input parameters'
            call WRERR(RTNAME,IERR,LOG)
        else

            DST = sin(DTH1)
            DCT = cos(DTH1)

            DH  = DPH1 - D1
            DS1 = sin(DH)
            DC1 = cos(DH)

            DS2 = sin(D2)
            DC2 = cos(D2)

            DX = DST*DC1
            DY = DST*DS1*DC2 + DCT*DS2
            DZ =-DST*DS1*DS2 + DCT*DC2

            call REPOL1(DX,DY,DZ,DR,DPHI,DTHETA)

            if (DABS(DR-1.D0).gt.1.E-6) then
                IERR = max(IERR,2)
            else
                IDRADRS = DPHI - D3
                call DRADRS(IDRADRS,ODRADRS)
                DPH2 = ODRADRS
                DTH2 = DTHETA
            endif

        endif

        RETURN
        end subroutine euler1

        subroutine gammlncal(gammln,xx)
        implicit none
        real gammln
        double precision :: cof(6)=(/76.18009173D0,-86.50532033D0,24.01409822D0, &
            -1.231739516D0,.120858003D-2,-.536382D-5/) 
        double precision :: fpf=5.5, half=0.5, one=1.0, ser, &
            stp=2.50662827465D0, tmp, x, xx
        integer :: j
        x = xx - one
        tmp = x + fpf
        tmp = (x + half)*log(tmp) - tmp
        ser = one
        do j=1,6
            x = x + one
            ser = ser + cof(j)/x
        enddo
        gammln = tmp + log(stp*ser)
        return
        end subroutine gammlncal



        SUBROUTINE GFUNCS (TEMPX,ALOC,YMODEL,DYDA,DY2DA2, &
            LISTA,MFIT,MA)

!C  Calculates Gaussian function distribution and partial derivatives
!
!************************ FFORM VERSION 1.0 ************ 21-APR-88 08:24
!
!A  author : SLS               date: 17-August-2005 15:00
!
!T  status: not tested
!
!   general description
!G  PFUNCS is the model value subroutine for gauss_fit for use with the
!G  fitting subroutine MQM.FOR.
!
!   call_var.          type I/O description
!P  TEMPX               R4  I/? X value.
!P  ALOC                R4  I/? array of model values.
!P                              A(1) = normalization
!P                              A(2) = width
!P                              A(3) = peak point
!P  YMODEL              R4      value of the function.
!P  DYDA                R4  I/O array of first partial derivatives
!P                              w.r.t. each parameter. (Only the
!P                              parameters actually being fit are
!P                              computed.)
!P  DY2DA2              R4  I/O array of second partial derivatives
!P                              w.r.t. the parameters.  (Only those
!P                              parameters being fit will be computed.)
!P  LISTA               I4  I   list of parameters being fitted.
!P  MFIT                I4  I   number of parameters being fit.
!P  MA                  I4  I   number of parameters in model.

        IMPLICIT NONE

!        SAVE

        INTEGER :: I, MFIT, LISTA(MFIT), MA
        REAL :: ALOC(MA), DY2DA2(10,10), DYDA(10), TEMPX, YMODEL
        DOUBLE PRECISION :: A(10), CHAN, G, YMOD

        DO I=1,10
            A(I) = DBLE(ALOC(I))
        ENDDO
        CHAN = DBLE(TEMPX)

!  Compute the model value

        G   = exp(-A(2)*(CHAN - A(3))*(CHAN - A(3)))
        YMOD = A(1)*G
        YMODEL = SNGL(YMOD)

!  Compute values on intervals to prepare for computation
!  of numerical derivatives.

        DYDA(1) = G
        DYDA(2) = -A(1)*(CHAN - A(3))*(CHAN - A(3))*G
        DYDA(3) = 2.0*A(1)*A(2)*(CHAN - A(3))*G

        DY2DA2(1,1) = 0.0
        DY2DA2(1,2) = -(CHAN - A(3))*(CHAN - A(3))*G
        DY2DA2(1,3) = 2.0*A(2)*(CHAN - A(3))*G
        DY2DA2(2,1) = -(CHAN - A(3))*(CHAN - A(3))*G
        DY2DA2(2,2) = A(1)*(CHAN - A(3))*(CHAN - A(3))* &
                (CHAN - A(3))*(CHAN - A(3))*G
        DY2DA2(2,3) = 2.0*A(1)*(CHAN - A(3))* &
                (1.0 -  A(2)*(CHAN - A(3))*(CHAN - A(3)))*G
        DY2DA2(3,1) = 2.0*A(2)*(CHAN - A(3))*G
        DY2DA2(3,2) = 2.0*A(1)*(CHAN - A(3))* &
                (1.0 -  A(2)*(CHAN - A(3))*(CHAN - A(3)))*G
        DY2DA2(3,3) = 2.0*A(1)*A(2)* &
                (2.0*A(2)*(CHAN - A(3))*(CHAN - A(3)) - 1.0)*G

        RETURN
        END subroutine gfuncs



        subroutine hard_scale (xmmdir,alpha,idet,filt,islow, &
            ishigh,fscal)

        implicit none

        integer :: bitpix, blockn, gcount, idet, i, ii, &
            iii, islow, ishigh, lun, naxes(3), naxis, pcount
        integer :: err = 0
        integer :: iflag = 0
        integer :: status = 0
        real :: alpha, fscal, fscarr(240,240,9), fscarrh(240,240,9), &
            fscarrm(240,240,9), fscarrs(240,240,9), ts, th
        character (len=1) :: db_fi(132), sp_fi(25)
        character (len=4) :: filt
        character (len=25) :: sp_file, sp_files, sp_filem, sp_fileh
        character (len=132) :: db_file, xmmdir
        logical anyf, extend, simple

        equivalence (db_file,db_fi)
        equivalence (sp_file,sp_fi)

        if(iflag .eq. 0) then

!  Get an unused lun

            call ftgiou(lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 1'
                print *, 'STATUS FTGIOU =', status
                stop
            endif

!  Set the name

            db_file = xmmdir
            sp_files = '/scale-factors-s.fits.gz'
            sp_filem = '/scale-factors-m.fits.gz'
            sp_fileh = '/scale-factors-h.fits.gz'

!  Read in the soft data

            sp_file = sp_files
            ii = 1
            do while (db_fi(ii) .ne. ' ')
                ii = ii + 1
            enddo
            do iii=0,24
                db_fi(ii+iii) = sp_fi(iii+1)
            enddo

!  Open the FITS file for the scale factors

            call ftopen(lun,db_file,0,blockn,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 2'
                print *, 'STATUS FTOPEN =', status
                print *, db_file
            endif

!  Read the FITS file header information

            call ftghpr (lun,10,simple,bitpix,naxis,naxes, &
                            pcount,gcount,extend,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 3'
                print *, 'STATUS FTGHPR =', status
                print *, db_file
                stop
            endif

!  Read in the file

            call ftg3de (lun,0,0,naxes(1),naxes(2),naxes(1), &
                    naxes(2),naxes(3),fscarrs,anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 4'
                print *, 'STATUS FTG2DE =', status
                print *, db_file
            endif

!  Close the FITS file

            call ftclos(lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 5'
                print *, 'STATUS FTCLOS =', status
                print *, db_file
            endif

!  Read in the medium spectrum scale factor array

            sp_file = sp_filem
            do iii=0,24
                db_fi(ii+iii) = sp_fi(iii+1)
            enddo

!  Open the FITS file for the scale factors

            call ftopen(lun,db_file,0,blockn,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 6'
                print *, 'STATUS FTOPEN =', status
                print *, db_file
            endif

!  Read the FITS file header information

            call ftghpr (lun,10,simple,bitpix,naxis,naxes, &
                            pcount,gcount,extend,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 7'
                print *, 'STATUS FTGHPR =', status
                print *, db_file
                stop
            endif

!  Read in the file

            call ftg3de (lun,0,0,naxes(1),naxes(2),naxes(1),naxes(2), &
                    naxes(3),fscarrm,anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 8'
                print *, 'STATUS FTG2DE =', status
                print *, db_file
            endif

!  Close the FITS file

            call ftclos(lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 9'
                print *, 'STATUS FTCLOS =', status
                print *, db_file
            endif

!  Read in the hard spectrum scale factor array

            sp_file = sp_fileh
            do iii=0,24
                db_fi(ii+iii) = sp_fi(iii+1)
            enddo

!  Open the FITS file for the scale factors

            call ftopen(lun,db_file,0,blockn,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 10'
                print *, 'STATUS FTOPEN =', status
                print *, db_file
            endif

!  Read the FITS file header information

            call ftghpr (lun,10,simple,bitpix,naxis,naxes, &
                            pcount,gcount,extend,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 11'
                print *, 'STATUS FTGHPR =', status
                print *, db_file
                stop
            endif

!  Read in the file

            call ftg3de (lun,0,0,naxes(1),naxes(2),naxes(1),naxes(2), &
                    naxes(3),fscarrh,anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 12'
                print *, 'STATUS FTG2DE =', status
                print *, db_file
            endif

!  Close the FITS file

            call ftclos(lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 13'
                print *, 'STATUS FTCLOS =', status
                print *, db_file
            endif

!  Clean up

            call ftfiou (lun,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'HARD_SCALE POSITION 14'
                print *, 'STATUS FTFIOU =', status
                print *, db_file
            endif

            if((alpha .lt. 1.0) .or. (alpha .gt. 2.4)) then
                print *, 'HARDSCALE Error'
                print *, 'Selected value for alpha out of range'
                stop
            elseif(alpha .eq. 1.0) then
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = fscarrh(i,ii,iii)
                        enddo
                    enddo
                enddo
            elseif(alpha .eq. 1.7) then
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = fscarrm(i,ii,iii)
                        enddo
                    enddo
                enddo
            elseif(alpha .eq. 2.4) then
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = fscarrs(i,ii,iii)
                        enddo
                    enddo
                enddo
            elseif(alpha .lt. 1.7) then
                ts = (1.7 - alpha)/0.7
                th = (alpha - 1.0)/0.7
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = &
                                    ts*fscarrm(i,ii,iii) + &
                                    th*fscarrh(i,ii,iii)
                        enddo
                    enddo
                enddo
            else
                ts = (2.4 - alpha)/0.7
                th = (alpha - 1.7)/0.7
                do i=1,240
                    do ii=1,240
                        do iii=1,9
                            fscarr(i,ii,iii) = &
                                    ts*fscarrs(i,ii,iii) + &
                                    th*fscarrm(i,ii,iii)
                        enddo
                    enddo
                enddo
            endif
        endif

!  Now evaluate the scale factor

        if(filt .eq. 'Thin') then
            if(idet .eq. 1) then
                fscal = fscarr(islow,ishigh,1)
            elseif(idet .eq. 2) then
                fscal = fscarr(islow,ishigh,4)
            else
                fscal = fscarr(islow,ishigh,7)
            endif
        elseif(filt .eq. 'Medi') then
            if(idet .eq. 1) then
                fscal = fscarr(islow,ishigh,2)
            elseif(idet .eq. 2) then
                fscal = 1.0
            else
                fscal = fscarr(islow,ishigh,8)
            endif
        elseif(filt .eq. 'Thic') then
            if(idet .eq. 1) then
                fscal = fscarr(islow,ishigh,3)
            elseif(idet .eq. 2) then
                fscal = fscarr(islow,ishigh,6)
            else
                fscal = fscarr(islow,ishigh,9)
            endif
        endif

        return
        end subroutine hard_scale



      subroutine JDEPS1(DJD,LMEAN,DEPS,IERR)
!
!C    Calculates the MEAN/TRUE obliquity of the ecl. for the Jul. Date
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:05
!
!A  author : KOD        date: 29-AUG-1989 17:22
!U  update : KOD        date: 29-MAR-1990 15:53
!
!T  status: tested
!
!   general description
!G  Subroutine to calculate the MEAN/TRUE obliquity of the ecliptic
!G  for a given Julian Date, according to
!G     Jean Meeus, 'Astronomical Formulae for Calculators',
!G                 third edition, 1985, p.44,69,70,81.
!G
!G  The parameters for calculating the mean obliquity have been
!G  taken from 'THE ASTRONOMICAL ALMANACH', 1984, S21.
!G
!G  For APPARENT positions (affected by aberration and nutation)
!G  use the TRUE obliquity, otherwise the MEAN obliquity.
!
!   call_var.          type I/O description
!P  DJD                 R8  I   Julian Date
!P
!P  LMEAN               L4  I   mean or true obliquity of the ecliptic:
!P                                LMEAN = .true.  : mean obliquity
!P                                LMEAN = .false. : true obliquity
!P
!P  DEPS                R8    O obliquity of the ecliptic  (radians)
!P
!P  IERR                I4    O = 0 no error
!P                              = 1 warning
!P                              = 2 error
!P                              = 3 severe error
!
!   include_block name          description
!I  CGENL.CMN          general parameter common block
!
!   routines_called    type     description
!R  JDNUT1              SR      calculates the nutation parameters
!R  WRERR               SR      formats error code in standard form
!R  ERRSTC              SR      continue error stack for one level

        IMPLICIT NONE
!        INCLUDE 'cgenl.cmn'

        DOUBLE PRECISION :: DJD, DEPS, DT, DP, DE, DJ
        DOUBLE PRECISION :: D1 =  0.40909280422D0
        DOUBLE PRECISION :: D2 = -2.26965525D-4
        DOUBLE PRECISION :: D3 = -2.86040D-9
        DOUBLE PRECISION :: D4 =  8.78967D-9
        DOUBLE PRECISION :: DJD0 = 2451545.D0
        DOUBLE PRECISION :: DYEAR =  36525.D0
        INTEGER :: IERR
        INTEGER :: ISTAT = 0
        LOGICAL :: LMEAN
        CHARACTER(len=8) :: RTNAME = 'JDEPS1'
        CHARACTER(len=80) :: LOG

        IERR = 0

        if ((DJD .lt. 0.D0) .or. (DJD .gt. 3.D6)) then
            IERR = 2
            LOG = 'Error in the input parameter DJD'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if (IERR .lt. 2) then
            DT = (DJD - DJD0) / DYEAR
            DEPS = D1 + ( D2 + ( D3 + D4 * DT ) * DT ) * DT

            if (.not.LMEAN) then
                DJ = DJD
                call JDNUT1(DJ,DP,DE,ISTAT)

                if  (ISTAT.gt.1) then
                    IERR = max(IERR,ISTAT)
                    call ERRSTC(RTNAME,10)
                else
                    DEPS = DEPS + DE
                endif
            endif
        endif

        return
        end subroutine jdeps1

      subroutine JDNUT1(DJD,DP,DE,IERR)
!
!C    Calculates the nutation in longitude and in obliquity from the JD
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:06
!
!A  author : KOD        date: 29.08L-1989 14:45
!U  update : KOD        date: 20-MAR-1990 15:04
!
!T  status: tested
!
!   general description
!G  Subroutine to calculate the nutation in longitude and the
!G  nutation in obliquity which are needed for the calculation
!G  of the APPARENT place of a star and for that of the APPARENT
!G  sidereal time, according to
!G     Jean Meeus, 'Astronomical Formulae for Calculators',
!G                 third edition, 1985, p.69-70 .
!
!   call_var.          type I/O description
!P  DJD                 R8  I   Julian Date
!P  DP                  R8    O nutation in longitude (radians)
!P  DE                  R8    O nutation in obliquity (radians)
!P  IERR                I4    O = 0 no error
!P                              = 1 warning
!P                              = 2 error
!P                              = 3 severe error
!
!   include_block name          description
!I  CGENL.CMN          general parameter common block
!
!   routines_called    type     description
!R  WRERR               SR      formats error code in standard form

        IMPLICIT NONE
!      INCLUDE 'cgenl.cmn'

        DOUBLE PRECISION :: DJD, DP, DE, DT, DLS, DLM, &
                    DMS, DMM, DOM, DH, DLS2, DLM2, DOM2
        DOUBLE PRECISION :: DJD0 = 2415020.D0
        DOUBLE PRECISION :: DYEAR =  36525.D0
        DOUBLE PRECISION :: DF = 4.848136811D-6
        INTEGER     IERR
        CHARACTER(len=8) :: RTNAME = 'JDNUT1'
        CHARACTER(len=80) :: LOG

        IERR = 0

        if ((DJD .lt. 0.D0) .or. (DJD .gt. 3.D6)) then
            IERR = 2
            LOG = 'Error in the input parameter DJD'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if (IERR .lt. 2) then
            DT = (DJD - DJD0) / DYEAR
            DLS = 4.8816283D0 + (628.3319506D0 + 5.2883D-6 * DT) * DT
            DLM = 4.7199672D0 + (8399.709144D0 - 1.9775D-5 * DT) * DT
            DMS = 6.2565830D0 + (628.3019466D0 - 2.6180D-6 * DT) * DT
            DMM = 5.1680002D0 + (8328.691104D0 + 1.6043D-4 * DT) * DT
            DOM = 4.5236020D0 - (33.75714610D0 + 3.6268D-5 * DT) * DT

            DLS2 = 2.D0 * DLS
            DLM2 = 2.D0 * DLM
            DOM2 = 2.D0 * DOM

            DH = - (17.2327D0 + 0.01737D0 * DT ) * sin(DOM) &
               - ( 1.2729D0 + 0.00013D0 * DT ) * sin(DLS2) &
               + 0.2088D0 * sin(DOM2) - 0.2037D0 * sin(DLM2) &
               + ( 0.1261D0 - 0.00031D0 * DT) * sin(DMS) &
               + 0.0675 * sin(DMM) &
               - (0.0497D0 - 0.00012D0 * DT) * sin(DLS2+DMS) &
               - 0.0342D0 * sin(DLM2-DOM) &
               - 0.0261D0 * sin(DLM2+DMM) &
               + 0.0214D0 * sin(DLS2-DMS) &
               - 0.0149D0 * sin(DLS2-DLM2+DMM) &
               + 0.0124D0 * sin(DLS2-DOM) &
               + 0.0114D0 * sin(DLM2-DMM)

            DP = DH * DF

            DH =   ( 9.2100D0 + 0.00091D0 * DT ) * cos(DOM) &
               + ( 0.5522D0 - 0.00029D0 * DT ) * cos(DLS2) &
               - 0.0904D0 * cos(DOM2) &
               + 0.0884D0 * cos(DLM2) &
               + 0.0216D0 * cos(DLS2+DMS) &
               + 0.0183D0 * cos(DLM2-DOM) &
               + 0.0113D0 * cos(DLM2+DMM) &
               - 0.0093D0 * cos(DLS2-DMS) &
               - 0.0066D0 * cos(DLS2-DOM)
              DE = DH * DF
        endif

        return
        end subroutine jdnut1


        SUBROUTINE MQM (FTYPE,TEMPX,TEMPY,TEMPU,NDATA,A,MA, &
            LISTA,MFIT,COVAR,ALPHA,NCA,CHISQ,ALAMDA,IERR)

!C  Maximum likelyhood fitting routine
!
!************************ FFORM VERSION 1.0 ************  2-MAR-88 09:16
!
!A  author : J. Bloch          date:  4-SEP-1987
!U  update : J. Bloch          date:  6-SEP-1987
!U  update : SLS               date:  2-MAR-1988 09:25
!
!T  status: not tested
!
!   general description
!G  MQMLMN was derived from MRQMIN in the book "Numerical Recipes" by
!G  Press et al. It finds the best parameter fit using the marquardt
!G  method applied to the Poisson Maximum Likelihood Estimator, as
!G  described by Cash in the 1979 March 15 Astrophysical Journal. The
!G  SIG array has been removed because it is not needed.
!
!   call_var.          type I/O description
!P  FTYPE               I4      = 1 Polynomial function (PFUNCS)
!P                              = 2 Gaussian function (GFUNCS)
!P  IERR                I4  I/O = 0 no error
!P                              = 2 Bad return from DGAUSJ
!P                              = 3
!P  TEMPX               R4  ?/? independent variable data values.
!P  TEMPY               R4  ?/? dependent variable data values.
!P  TEMPU               R4  ?/? dependent variable uncertainty values.
!P  NDATA               I4      number of data points to fit.
!P  A                   R4  I/O model parameter values.
!P  MA                  I4  I/? number of model parameter values.
!P  LISTA               I4  I/O list of parameters
!P                                  in array A actually being varied.
!P  MFIT                I4  I/? number of parameters actually being fit.
!P  COVAR               R8  I/O covariance matrix and work
!P                                  array used durint the fit.
!P  ALPHA               R8  I/O matrix ALPHA used in the Marquard method
!P                                  (See Press et al.,pp. 521-528)
!P  NCA                 I4  I/? Actual dimensions of ALPHA and COVAR.
!P  CHISQ               R4  I/O Output Maximum Likelihood Estimator.
!P  ALAMDA              R4  I/O Updated REAL iteration control variable.
!P                              Must be set to  < 0.0 on first call
!P                              of MQMLMN. If a step fails, ALMADA grows
!P                              by a factor of 10. On final call, set
!P                              ALMADA = 0.0 so curvature matrix is
!P                              returned.
!P  PFUNCS              R4  ?/? Input name of model function, with call
!P                                  sequence:
!P                                  CALL PFUNCS(X1,A,YMOD,DYDA,DY2DA2,MA)
!P                                      With:
!P                                      X1 = Independent variable at
!P                                          which to evaluate the model.
!P                                      A(MA) = Array of model params.
!P                                      YMOD = Output model value at X1.
!P                                      DYDA(MA) = Vector of first
!P                                           derivatives of the model
!P                                           w.r.t. the parameters.
!P                                      DY2DA2(20,20)= Matrix of partial
!P                                           second derivatives of the
!P                                           model w.r.t the parameters.
!P                                      LISTA = list of parameters being
!P                                           fitted.
!P                                      MFIT  = number of parameters being
!P                                           fit.
!P                                      MA = Input INTEGER number of
!P                                           parameters in the model.
!
!   include_block name          description
!
!   routines_called    type     description
!R  DCVSRT              SR      Repacks the covarience matrix for MQM
!R  DGAUSJ              SR      Numerical Routines linear equation
!R                              solution routine
!R  MRQ_CB              SR      Computes matrix elements for MQM

        IMPLICIT NONE

!        SAVE

        INTEGER :: MA
        INTEGER :: FTYPE, IERR, IERR1, IHIT, J, K, KK, LISTA(MA), &
            MFIT, NCA, NDATA

        REAL :: A(MA), ALAMDA, ATRY(10), CHISQ, OCHISQ, &
            TEMPX(NDATA), TEMPY(NDATA), TEMPU(NDATA)

        DOUBLE PRECISION :: ALPHA(NCA,NCA), COVAR(NCA,NCA),  &
            BETA(10), DA(10)

        IERR = 0

!  Do some initializations

        IF(ALAMDA .LT. 0.) THEN
            KK = MFIT + 1

!  Check to see if LISTA contains a proper permutation of the coeffs.

            J = 0
            DO WHILE ((J .LT. MA) .AND. (IERR .EQ. 0))
                J = J + 1
                IHIT = 0
                DO 11 K=1,MFIT
                    IF(LISTA(K) .EQ. J) IHIT = IHIT + 1
   11           ENDDO
                IF(IHIT .EQ. 0) THEN
                    LISTA(KK) = J
                    KK = KK + 1
                ELSE IF (IHIT .GT. 1) THEN
                    IERR = 3
                    PRINT *, 'MQM: Improper permutation ', &
                        'in LISTA, 1'
                ENDIF
            ENDDO

            IF(KK .NE. (MA+1)) THEN
                 IERR = 3
                 PRINT *, 'MQM: Improper permutation in LISTA, 2'
            ENDIF

            IF(IERR .EQ. 0) THEN
                ALAMDA = 0.001
                CALL MRQ (FTYPE,TEMPX,TEMPY,TEMPU,NDATA,A,MA, &
                    LISTA,MFIT,ALPHA,BETA,NCA,CHISQ,IERR1)
                IF(IERR1 .EQ. 3) THEN
                    IERR = 3
                    PRINT *, 'MQM ==> Bad return from MRQ #1'
                ELSE
                    OCHISQ = CHISQ
                    DO 13 J=1,MA
                        ATRY(J) = A(J)
   13               ENDDO
                ENDIF
            ENDIF
        ENDIF

!  Alter linearized fitting matrix by augmenting diagonal elements

        IF(IERR .EQ. 0) THEN
            DO 15 J=1,MFIT
                DO 14 K=1,MFIT
                    COVAR(J,K) = ALPHA(J,K)
   14           ENDDO
                COVAR(J,J) = ALPHA(J,J)*(1.0D0 + DBLE(ALAMDA))
                DA(J) = BETA(J)
   15       ENDDO

!  Get the matrix solution

            CALL DGAUSJ(COVAR,MFIT,NCA,DA,1,1,IERR1)
            IF(IERR1 .EQ. 3) THEN
                IERR = 3
                PRINT *, 'MQM ==> Bad return from DGAUSJ'
                RETURN
            ELSE

!  If converged evaluate covarience matrix with ALAMDA = 0

                IF(ALAMDA .EQ. 0.) THEN
                    CALL DCVSRT(COVAR,NCA,MA,LISTA,MFIT)
                ELSE

!  Check to see if the trial succeeded

                    DO 16 J=1,MFIT
                        ATRY(LISTA(J)) = A(LISTA(J)) + SNGL(DA(J))
   16               ENDDO

                    CALL MRQ (FTYPE,TEMPX,TEMPY,TEMPU,NDATA, &
                        ATRY,MA,LISTA,MFIT,COVAR,DA,NCA,CHISQ, &
                        IERR1)
                    IF(IERR1 .EQ. 3) THEN
                        IERR = 3
                        PRINT *,'MQM ==> Bad return ', &
                                'from MRQ #2'
                    ELSE
                        IF(CHISQ .LT. OCHISQ) THEN

!  Success, accept the new solution

                            ALAMDA = 0.1*ALAMDA
                            OCHISQ = CHISQ
                            DO 18 J=1,MFIT
                                DO 17 K=1,MFIT
                                    ALPHA(J,K) = COVAR(J,K)
   17                           ENDDO
                                BETA(J) = DA(J)
                                A(LISTA(J)) = ATRY(LISTA(J))
   18                       ENDDO
                        ELSE

!  Failure, increase ALAMDA and return

                            ALAMDA = 10.*ALAMDA
                            CHISQ = OCHISQ
                        ENDIF
                    ENDIF
                ENDIF
            ENDIF
        ENDIF

        RETURN
        END subroutine mqm



         SUBROUTINE MRQ (FTYPE,TEMPX,TEMPY,TEMPU,NDATA,A,MA, &
             LISTA,MFIT,ALPHA,BETA,NALP,CHISQ,IERR)
 
!C  Computes matrix elements for MQM
!
!************************ FFORM VERSION 1.0 ************ DD-MMM-YY HH:MM
!
!A  author : J. Bloch          date:  4-SEP-1987
!U  update : J. Bloch          date:  6-SEP-1987
!U  update : SLS               date:  2-MAR-1988 09:23
!
!T  status: not tested
!
!   general description:
!G  MRQ was derived from MRQCOF in the book "Numerical Recipes"
!G  by Press et al. It computes the matrix elements needed for
!G  the marquardt method applied to the Poisson Maximum Likelihood
!G  Estimator, as described by Cash in the 1979 March 15
!G  Astrophysical Journal. The SIG array has been removed because
!G  it is not needed.
!
!   call_var.          type I/O description
!P  FTYPE               I4      = 1 Polynomial function (PFUNCS)
!P                              = 2 Gaussian function (GFUNCS)
!P  IERR                I4    O = 0 no error
!P                              = 3 illegal model value (model < 0.)
!P  TEMPX               R4  ?/? array of independent var. data values.
!P  TEMPY               R4  ?/? array of dependent variable data values.
!P  TEMPU               R4  ?/? array of dependent variable uncertainty values.
!P  NDATA               I4      number of data points to fit.
!P  A                   R4  I/O array of model parameter values.
!P  MA                  I4  I/? number of model parameter values.
!P  LISTA               I4  I/O array list of parameters
!P                                  in array A actually being varied.
!P  MFIT                I4  I/? number of parameters actually being fit.
!P  COVAR               R8  I/O covariance matrix and work
!P                                  array used durint the fit.
!P  ALPHA               R8  I/O matrix ALPHA used in the Marquard
!P                              method.  (See Press et al., pp. 521-528)
!P  BETA                        vector beta used in the Marquardt
!P                              method.  (See Press et al. pp. 521-528)
!P  NALP                        actual dimensions of ALPHA and BETA.
!P  CHISQ               R4  I/O Output Maximum Likelihood Estimator.
!
!   include_block_name          description
!I  R$COMMON:CGENL.CMN          general parameter common block
!
!   routines_called    type     description
!R  HFLAG               R       output flag handling routine
!R  WRFLAG              SR      writes to FLAG output stream
!R  PFUNCS              SR      input name of model function, with call
!R                                  sequence:
!R                              CALL PFUNCS(X1,A,YMOD,DYDA,DY2DA2,LISTA,
!R                                          MFIT,MA)
!R                                  With:
!R                                    X1 = Independent variable at
!R                                        which to evaluate the model.
!R                                    A(MA) = Array of model parameters.
!R                                    YMOD = Output model value at X1.
!R                                    DYDA(MA) = Vector of first
!R                                         derivatives of the model
!R                                         w.r.t. the parameters.
!R                                    DY2DA2(20,20) = Matrix of partial
!R                                         second derivatives of the
!R                                         model w.r.t the parameters.
!R                                    LISTA(MFIT) = Same as above.
!R                                    MFIT = Same as above.
!R                                    MA = Input INTEGER number of
!R                                         parameters in the model

        IMPLICIT NONE

!        PARAMETER MMAX=10

        INTEGER :: MFIT
        INTEGER :: FTYPE, I, IERR, J, K, KK, LISTA(MFIT),  &
            MA, NALP, NDATA

        REAL :: A(MA), CHISQ, DY, DYDA(10), D2YDA2(10,10), &
                TEMPX(NDATA), TEMPY(NDATA), SIG2I, TEMPU(NDATA), &
                WT, YMOD

        DOUBLE PRECISION :: ALPHA(NALP,NALP), BETA(MA)

        IERR = 0

!  Initialize (symmetric) ALPHA and BETA

        DO 12 J=1,MFIT
            DO 11 K=1,J
                ALPHA(J,K) = 0.
   11       ENDDO
            BETA(J) = 0.
   12   ENDDO

        CHISQ = 0.0

!  Summation loop over all data

        I = 0
        DO WHILE ((I .LT. NDATA) .AND. (IERR .EQ. 0))
            I = I + 1
            IF(FTYPE .EQ. 1) THEN
                CALL PFUNCS(TEMPX(I),A,YMOD,DYDA, &
                        D2YDA2,LISTA,MFIT,MA)
            ELSEIF(FTYPE .EQ. 2) THEN
                CALL GFUNCS(TEMPX(I),A,YMOD,DYDA, &
                        D2YDA2,LISTA,MFIT,MA)
            ELSE
                PRINT *, 'Illegal model type'
            ENDIF
            IF(YMOD .LT. -1.0D200) THEN
                PRINT *,'MRQ: Illegal model value =', YMOD
                PRINT *,'               TEMPX(I) =', TEMPX(I)
                PRINT *,'               TEMPY(I) =', TEMPY(I)
                PRINT *,'               TEMPU(I) =', TEMPU(I)
                PRINT *,'                      A =', &
                        (A(KK),KK=1,MA)
                IERR = 3
            ELSE
                SIG2I = 1./(TEMPU(I)*TEMPU(I))
                DY = TEMPY(I) - YMOD

                DO 14 J=1,MFIT
                    WT = SIG2I*DYDA(LISTA(J))
                    DO 13 K=1,J
                        ALPHA(J,K) = ALPHA(J,K) + WT*DYDA(LISTA(K))
   13               ENDDO
                    BETA(J) = BETA(J) + DY*WT
   14           ENDDO
                CHISQ = CHISQ + DY*DY*SIG2I
!                print *, tempx(i), tempy(i), ymod, chisq
            ENDIF
        ENDDO

        if(ierr .eq. 0) then

!  Fill in the symmetric side

            do j=2,mfit
                do k=1,j-1
                    alpha(k,j) = alpha(j,k)
                enddo
            enddo
        endif

        return
        end subroutine mrq



        SUBROUTINE PFUNCS (TEMPX,ALOC,YMODEL,DYDA,DY2DA2, &
            LISTA,MFIT,MA)

!C  Calculates polynomial function distribution and partial derivatives
!
!************************ FFORM VERSION 1.0 ************ 21-APR-88 08:24
!
!A  author : SLS               date: 17-August-2005 15:00
!
!T  status: not tested
!
!   general description
!G  PFUNCS is the model value subroutine for poly_fit for use with the
!G  fitting subroutine MQMLMN.FOR.
!
!   call_var.          type I/O description
!P  TEMPX               R4  I/? X value.
!P  ALOC                R4  I/? array of model values.
!P                              A(1) = constant
!P                              A(2) = linear
!P                              A(3) = quadratic
!P                              A(4) - A(10) third to ninth order
!P  YMODEL              R4      value of the polynomial.
!P  DYDA                R4  I/O array of first partial derivatives
!P                              w.r.t. each parameter. (Only the
!P                              parameters actually being fit are
!P                              computed.)
!P  DY2DA2              R4  I/O array of second partial derivatives
!P                              w.r.t. the parameters.  (Only those
!P                              parameters being fit will be computed.)
!P  LISTA               I4  I   list of parameters being fitted.
!P  MFIT                I4  I   number of parameters being fit.
!P  MA                  I4  I   number of parameters in model.
!P                              (Should be <= 10)

        IMPLICIT NONE

!        SAVE

        INTEGER I, II, MFIT, LISTA(MFIT), MA

        REAL ALOC(MA), DY2DA2(10,10), DYDA(10), TEMPX, YMODEL

        DOUBLE PRECISION A(10), DTIME, YMOD

        DO I=1,10
            A(I) = DBLE(ALOC(I))
        ENDDO
        DTIME = DBLE(TEMPX)

!  Compute the model value

        YMOD = A(1) + A(2)*DTIME + A(3)*DTIME*DTIME + &
            A(4)*DTIME*DTIME*DTIME + &
            A(5)*DTIME*DTIME*DTIME*DTIME + &
            A(6)*DTIME*DTIME*DTIME*DTIME*DTIME + &
            A(7)*DTIME*DTIME*DTIME*DTIME*DTIME*DTIME + &
            A(8)*DTIME*DTIME*DTIME*DTIME*DTIME*DTIME*DTIME
        YMODEL = SNGL(YMOD)

!  Compute values on intervals to prepare for computation
!  of numerical derivatives.

        DYDA(1) = 1
        DYDA(2) = DTIME
        DYDA(3) = DTIME*DTIME
        DYDA(4) = DTIME*DTIME*DTIME
        DYDA(5) = DTIME*DTIME*DTIME*DTIME
        DYDA(6) = DTIME*DTIME*DTIME*DTIME*DTIME
        DYDA(7) = DTIME*DTIME*DTIME*DTIME*DTIME*DTIME
        DYDA(8) = DTIME*DTIME*DTIME*DTIME*DTIME*DTIME*DTIME
        DYDA(9) = 0.0
        DYDA(10) = 0.0

        DO I=1,10
            DO II=1,10
                DY2DA2(I,II) = 0
            ENDDO
        ENDDO

        RETURN
        END subroutine pfuncs


!        subroutine poidevcal (poidev,xm,idum)
!
!        implicit none

! !       real :: alxm, g, sq, t
 !       real :: gammln, poidev, ran, y
 !       double precision :: em, emt, xm, xmt
 !       real :: pi = 3.141592654
 !       real :: oldm = -1.0
 !       integer :: idum

 !       if (xm .lt. 12.0) then
 !           if (xm .ne. oldm) then
 !               oldm = xm
 !               g = exp(-xm)
 !           endif
 !           em = -1.0
 !           t = 1.0
 !           do while (t .gt. g)
 !               em = em + 1.0
 !               call random (ran,idum)
 !               t = t*ran
 !               if(ran .gt. 1) stop
 !           enddo
 !       else
 !           if (xm .ne. oldm) then
 !               oldm = xm
 !               sq = sqrt(2.0*xm)
 !               alxm = alog(oldm)
 !               xmt = xm + 1.0
 !               call gammlncal(gammln,xmt)
 !               g = xm*alxm - gammln
 !           endif
 !           em = -1.0
 !           do while ((em .lt. 0.0) .or. (ran .gt. t))
 !               call random (ran,idum)
 !               y = tan(pi*ran)
 !               em = sq*y + xm
 !               if(em .ge. 0.0) then
 !                   em = int(em)
  !                  emt = em + 1.0
 !                   call gammlncal(gammln,emt)
 !                   t = 0.9*(1.0 + y*y)*exp(em*alxm - gammln - g)
 !                   call random (ran,idum)
 !               endif
  !          enddo
  !      endif
!
 !       poidev = em
!
 !       return
  !      end subroutine poidevcal

        subroutine poly_fit(tempx,tempy,tempu,n,np,ap)

        implicit none

        integer :: i, ierr, iflag, iup, iuplim, &
            lista(10), maxitr, n, np, nppo
        integer :: ftype = 1

        real :: a(10), alamda, almlim, ap(10), chisq, deltch, &
            i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, it, &
            ochisq, olamda, t, tempx(n), tempy(n), tempu(n)
        real :: ainit(10) = (/1.0, -0.1, 0.03, -0.01, 0.003, &
               -0.0001, 0.00003, -0.00001, 0.000003, -0.000001/)
        double precision :: alpha(10,10), covar(10,10)
        character(len=1) :: fix(10) = (/' ', ' ', ' ', ' ', &
           ' ', ' ', ' ',' ', ' ', ' '/)
        character(len=15) :: pshort(10) = (/'I0', 'I1', 'I2', 'I3', &
           'I4', 'I5', 'I6','I7', 'I8', 'I9'/)
        character(len=80) :: pname(10)

        common /modpar/ i0, i1, i2, i3, i4, i5, i6, i7, i8, i9

        equivalence(a(1),i0)

        ierr = 0

        do i=1,10
            a(i) = ainit(i)
        enddo

        it = 0.0
        t = 0.0
        do i=1,n
            if(tempy(i) .ne. 0.0) then
                t = t + tempy(i)
                it = it + 1
            endif
        enddo
        a(1) = t/it

        nppo = np + 1
        do i=0,np
            lista(i+1) = i + 1
            fix(i+1) = '*'
            write(pname(i+1),3000) i
 3000       format('I',i1)
        enddo
        if(np+2 .lt. 10) then
            do i=np+2,9
                lista(i+1) = 0
                fix(i+1) = ' '
                pname(i+1) = ' '
                a(i) = 0.0
            enddo
        endif

!  *********************************************************************
!  *                                                                   *
!  *                       Main Fit Begins Here                        *
!  *                                                                   *
!  *********************************************************************

        maxitr = 100
        almlim = 1.E3
        iuplim = 10

!        call report(-1,' Initial Model Value Settings:',a,fix)

        alamda = -1.0
        ochisq = 0.0
        iup = 0
        i = 1
        iflag = 0

        do while ((iflag .eq. 0) .and. (i .lt. maxitr))
            i = i + 1
            olamda = alamda
            call mqm (ftype,tempx,tempy,tempu,n,a,10,lista,nppo, &
                covar,alpha,10,chisq,alamda,ierr)
            if(ierr .eq. 3) then
                print *, 'Bad return from MQM'
                print *, 'Program stopped in POLY_FIT'
                print *, 'Position 01'
                stop
            endif

            deltch = abs(ochisq - chisq)

!  Check for various termination conditions

            if((alamda .lt. olamda) .and. (deltch .lt. 0.1)) &
                    iflag = 1
            if(iflag .eq. 0) then
                if(alamda .gt. olamda) iup = iup + 1
                if(alamda .lt. olamda) iup = 0
                if(iup .gt. iuplim) iflag = 1
                if(alamda .gt. almlim) iup = iup + 1

!                write(*,*)
!                write(*,*) 'ITER =',i,'  ALAMDA =',alamda, &
!                        '  M.L.E. =',chisq
!                call report(-1,'Intermediate step..',a,fix)
            endif
        enddo

!        write(*,*)
!        call report(-1,' Final Model Values: (*=Floating)',a,fix)
        write(*,1010) ' Np =',np,' N =',n,' M.L.E. =',chisq
 1010   format(/,1x,a,i2,a,i4,a,g12.4,a,g14.7)

!  Finish off the fit to get the covarience matrix
 
        alamda = 0.0
        call mqm (ftype,tempx,tempy,tempu,n,a,10,lista,nppo, &
                covar,alpha,10,chisq,alamda,ierr)
        if(ierr .eq. 3) then
            print *, 'Bad return from MQM'
            print *, 'Program stopped in POLY_FIT'
            print *, 'Position 02'
            stop
        endif

!  Write out final values and one sigma uncertainties

        print *, 'Final parameter values and the formal standard errors'
        print *

        do i=1,nppo
            ap(i) = a(i)
            write(6,2100) pshort(lista(i)),a(lista(i)), &
                    sngl(dsqrt(dabs(covar(lista(i),lista(i)))))
 2100       format(1x,a15,f13.5,' +/-',f11.5)
        enddo
 
        return
        end subroutine poly_fit


        subroutine putfit(infile,rarray,i4array,iflag, &
            units,comm,isize1,isize2,bzero,bscale,ctype,crval,cdelt, &
            crpix,clobber,status)

!  INFILE    file name of output file
!  RARRAY    REAL*4 array for output
!  I4ARRAY   INTEGER*4 array for output
!  IFLAG     flag for input file type selection
!            1 => R*4 array
!            2 => I*4 array
!  UNITS     character string showing the units of the array
!  COMM      comment character string
!  ISIZE     array (square) dimension
!  BZERO     offset value for array
!  BSCALE    slope value for array
!  CRVAL1    reference coordinate for first axis (RA)
!  CDELT1    pixel size for first axis
!  CRVAL2    reference coordinate for second axis (Dec)
!  CDELT2    pixel size for second axis
!  CLOBBER   1 = overwrite an existing file
!            0 = don't
!  STATUS    FITSIO status flag

        implicit none

        integer :: isize1, isize2
        real :: bzero, bscale, cdelt1, cdelt2, &
            crpix1, crpix2, crval1, crval2, rarray(isize1,isize2)
        double precision :: cdelt(2), crpix(2), crval(2)
        integer :: i4array(isize1,isize2)
        character(len=5) :: com
        character(len=8) :: ctype(2), units
        character(len=60) :: comm, comment
        character(len=132) :: infile
        integer :: bitpix, blockn, fpixel, iflag, &
            gcount, group, naxes(10), naxis, lun, pcount, status
        logical :: clobber, extend, simple

        equivalence (comment,com)

        integer :: err = 0

        status = 0

!  Get an unused lun

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'POSITION 31'
        endif
        comment = comm

!  Write out the array

        write(errstr,*) "PUTFIT file name: ", trim(infile)
        call message(AppMsg,Noisy)

        simple = .TRUE.
        naxis = 2
        naxes(1) = isize1
        naxes(2) = isize2
        pcount = 0
        gcount = 1
        extend = .FALSE.
        if(iflag .eq. 1) then
            bitpix = -32
        elseif(iflag .eq. 2) then
            bitpix = 16
        elseif(iflag .eq. 3) then
            bitpix = 32
        endif
        group = 1
        fpixel = 1

!  Check to see if there is an existing file

        call ftopen (lun,infile,0,blockn,status)
        if(status .eq.  0) then
            print *, 'File exists with output file name'
            if(clobber) then
                print *, 'Clobber the file and continue'
                call ftdelt(lun,status)
            else
                print *, 'Will not clobber file'
                print *, 'XMM-BACK-IMAGE POSITION 0'
                stop
            endif
        endif
        status = 0

!  Open the output file

        call ftinit(lun,infile,2880,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTINIT =', status
            print *, 'POSITION 1'
            print *, infile
        endif

!  Set the output file header

        call ftphpr(lun,simple,bitpix,naxis,naxes,pcount,gcount, &
                    extend,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTPHPR =', status
            print *, 'POSITION 2'
            print *, infile
        endif

!  Write some useful header keywords

        call ftpcom(lun,comment,status)
        call ftpkys(lun,'CONTENT','IMAGE','  ',status)
        call ftpkys(lun,'TELESCOP','XMM','mission name',status)
        call ftpkys(lun,'INSTRUME','EPIC','instrument name',status)
        call ftpkys(lun,'OBS_MODE','POINTING', &
                'obs mode: POINTING,SLEW, OR SCAN',status)
        call ftpkys(lun,'RADECSYS','FK5', &
                'Equatorial system reference',status)
        call ftpkyj(lun,'EQUINOX',2000,'Equinox',status)
        call ftpkyf(lun,'BZERO',bzero,4,' ',status)
        call ftpkyf(lun,'BSCALE',bscale,8,' ',status)
        call ftpkys(lun,'BUNIT',units,'Units of data',status)
        crpix1 = crpix(1)
        call ftpkyf(lun,'CRPIX1',crpix1,4,'Reference pixel',status)
        call ftpkys(lun,'CTYPE1',ctype(1),'Projection',status)
        crval1 = crval(1)
        call ftpkyf(lun,'CRVAL1',crval1,4,'Central longitude',status)
        cdelt1 = cdelt(1)
        call ftpkyf(lun,'CDELT1',cdelt1,8,'Pixel size',status)
        call ftpkys(lun,'CUNIT1','deg','Units of coordinate',status)

        crpix2 = crpix(2)
        call ftpkyf(lun,'CRPIX2',crpix2,4,'Reference pixel',status)
        call ftpkys(lun,'CTYPE2',ctype(2),'Projection',status)
        crval2 = crval(2)
        call ftpkyf(lun,'CRVAL2',crval2,4,'Central latitude',status)
        cdelt2 = cdelt(2)
        call ftpkyf(lun,'CDELT2',cdelt2,8,'Pixel size',status)
        call ftpkys(lun,'CUNIT2','deg','Units of coordinate',status)

        call ftpcom(lun, &
        'This file was produced using the XMM-ESAS software written by', &
        status)
        call ftpcom(lun, &
        'S. L. Snowden, and written using the FITSIO package of ', &
        status)
        call ftpcom(lun,'W. D. Pence.',status)

!  Define the file format

        call ftpdef(lun,bitpix,naxis,naxes,pcount,gcount,extend, &
                    status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTPDEF =', status
            print *, 'POSITION 3'
            print *, infile
        endif

!  Write the file

        if(iflag .eq. 1) then
            call ftp2de(lun,group,isize1,isize1,isize2, &
                rarray,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTP2DE =', status
                print *, 'POSITION 4'
                print *, infile
            endif
        elseif(iflag .eq. 2) then
            call ftp2dj(lun,group,isize1,isize1,isize2, &
                i4array,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTP2DJ =', status
                print *, 'POSITION 6'
                print *, infile
            endif
        endif

!  Close the file

        call ftclos(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTOPEN =', status
            print *, 'POSITION 7'
            print *, infile
        endif

!  Clean up

        call ftfiou (lun,status)

        return
        end subroutine putfit



        subroutine random (ran,idum)

        implicit none

        real :: r(97)
        real :: ran
        real :: rm1=3.8580247E-6, rm2=7.4373773E-6
        integer :: m1=259200, ia1=7141, ic1=54773
        integer :: m2=134456, ia2=8121, ic2=28411
        integer :: m3=243000, ia3=4561, ic3=51349
        integer :: iff=0
        integer :: idum, j
        integer :: ix1, ix2, ix3

!        print *, '1 ix1,ix2,ix3: ', ix1,ix2,ix3,iff,idum,m1,m2,m3
        if ((idum .lt. 0) .or. (iff .eq. 0)) then
            iff = 1
            ix1 = mod(ic1 - idum,m1)
            ix1 = mod(ia1*ix1 + ic1,m1)
            ix2 = mod(ix1,m2)
            ix1 = mod(ia1*ix1 + ic1,m1)
            ix3 = mod(ix1,m3)
            do j=1,97
                ix1 = mod(ia1*ix1 + ic1,m1)
                ix2 = mod(ia2*ix2 + ic2,m2)
                r(j) = (float(ix1) + float(ix2)*rm2)*rm1
            enddo
            idum = 1
        endif
!        print *, '2 ix1,ix2,ix3: ', ix1,ix2,ix3
        ix1 = mod(ia1*ix1 + ic1,m1)
        ix2 = mod(ia2*ix2 + ic2,m2)
        ix3 = mod(ia3*ix3 + ic3,m3)
!        print *, '3 ia1,ix1,ic1,m1,ia2,ix2,ic2,m2,ix3: ', &
!              ia1,ix1,ic1,m1,ia2,ix2,ic2,m2,ix3
        j = 1 + (97*ix3)/m3
        if((j .gt. 97) .or. (j .lt. 1)) then
            print *, 'RAN1 failure'
            print *, 'j = ', j
            print *, r
            stop
        endif
        ran = r(j)
        r(j) = (float(ix1) + float(ix2)*rm2)*rm1
!        print *, 'ran: ', ran
!        print *, '4 ix1,ix2,ix3: ', ix1,ix2,ix3,m1,m2,m3
        return
        end subroutine random


        subroutine read_arf (obj_arf,mchan,nchan,arf,emin,emax,err)

        implicit none

        integer :: mchan
        integer :: blockn, err, hdutype, i, iarf, icol, imin, &
            imax, irow, lun, nchan, ncols, status, tfields, varidat
        real :: arf(mchan), emax(mchan), emin(mchan)
        character (len=8) :: colnam(10), tform(10), ttype(10), tunit(10)
        character (len=20) :: extname
        character (len=80) :: obj_arf
        logical :: anyf

        status = 0
        err = 0

!  Get an unused lun

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            print *, 'READ_ARF'
            print *, 'STATUS FTGIOU =', status
            print *, 'POSITION 01'
            err = 3
            return
        ENDIF

!  Open the FITS file

        call ftopen (lun,obj_arf,0,blockn,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'READ_ARF'
            print *, 'STATUS FTOPEN =', status
            print *, 'POSITION 02'
            print *, obj_arf
            err = 3
            return
        endif

!  Read the FITS file header information

        if(err .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'READ_ARF'
                print *, 'STATUS FTMRHD =', status
                print *, 'POSITION 03'
                print *, 'Could not find the SPECRESP extension'
            else

!  Read the FITS file header information

                call ftghbn (lun,10,nchan,tfields,ttype,tform, &
                    tunit,extname,varidat,status)
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, 'READ_ARF'
                    print *, 'STATUS FTGHBN =', status
                    print *, 'POSITION 04'
                    err = 3
                    return
                endif
                if(extname .ne. 'SPECRESP') then
                    print *, extname
                    print *, 'READ_ARF'
                    print *,' POSITION 05'
                    print *, 'This was supposed to be the SPECRESP ext'
                    err = 3
                    return
                endif
                if(mchan .lt. nchan) then
                    print *, 'READ_ARF'
                    print *,' POSITION 06'
                    print *, 'Number of array elements smaller than', &
                            ' number of channels in spectrum'
                    err = 3
                    return
                elseif(status .ne. 0) then
                    err = 1
                    print *, 'READ_ARF'
                    print *, 'STATUS FTGHPR =', status
                    print *, 'POSITION 07'
                    print *, 'Failure on reading the header ', &
                            'information'
                    err = 3
                    return
                else

!  Read in the column names

                    call ftgkns (lun,'TTYPE',1,10,colnam,ncols,status)
                    if(status .ne. 0) then
                        err = 1
                        print *, 'READ_ARF'
                        print *, 'STATUS FTGKNS =', status
                        print *, 'POSITION 08'
                        print *, 'Failure on reading the column names'
                    endif

!  Extract the channel numbers

                    iarf = 0
                    imin = 0
                    imax = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'ENERG_LO') imin = icol
                        if(colnam(icol) .eq. 'ENERG_HI') imax = icol
                        if(colnam(icol) .eq. 'SPECRESP') iarf = icol
                    enddo
                    do irow=1,nchan
                        arf(irow) = 0
                        emin(irow) = 0
                        emax(irow) = 0
                    enddo
                    do i=1,nchan
                        if(iarf .ne. 0) &
                            call ftgcve(lun,iarf,i,1,1,0, &
                                arf(i),anyf,status)
                        if(imin .ne. 0) &
                            call ftgcve(lun,imin,i,1,1,0, &
                                emin(i),anyf,status)
                        if(imax .ne. 0) &
                            call ftgcve(lun,imax,i,1,1,0, &
                                emax(i),anyf,status)
                        if((status .ne. 0) .and. (err .eq. 0)) then
                            err = 1
                            print *, 'READ_ARF'
                            print *, 'STATUS FTGCV =', status
                            print *, 'POSITION 09'
                            print *, 'Failure on reading data'
                        endif
                    enddo
                endif
            endif
            if(status .ne. 0) then
                print *, 'READ_ARF'
                print *, 'STATUS = ', status
                print *, 'POSITION 09'
                print *, 'Failure in READ_RMF getting energy bounds'
            endif
            call ftclos (lun,status)
        endif

!  Clean up

        call ftfiou (lun,status)

        return
        end subroutine read_arf


        subroutine read_pha (spec_name,mchan,nchan,channel,counts, &
                stat,quality,group,exposure,backscal,areascal,revl, &
                obsid,modet,err)

        implicit none

        integer :: mchan
        integer :: blockn, channel(mchan), err, frmtime, group(mchan), &
            hdutype, i, ich, &
            icol, icou, igrp, iqual, irow, lun, nchan, ncols, &
            obsid, quality(mchan), revl, tfields, varidat 
        integer :: status = 0
        real ::areascal, backscal, counts(mchan), exposure, stat(mchan)
        character(len=1) :: mod(25)
        character(len=8) :: colnam(10), tform(10), ttype(10), tunit(10)
        character(len=20) :: extname
        character(len=25) :: mode, modet
        character(len=80) :: com, spec_name
        logical :: anyf

        equivalence (mode,mod)

        err = 0

!  Get an unused lun

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'STATUS FTGIOU =', status
            print *, 'READ_PHA POSITION 01'
        endif

!  Open the FITS file

        call ftopen (lun,spec_name,0,blockn,status)
        if(status .ne. 0) then
            err = 1
            print *, 'STATUS FTOPEN =', status
            print *, 'READ_PHA POSITION 02'
            print *, spec_name
        endif

!  Read in the header information

        call ftgkyj (lun,'OBS_ID',obsid,com,status)
        if(status .ne. 0) then
            print *, 'READ_PHA POSITION 03'
            print *, 'Missing OBS_ID keyword'
            print *, spec_name
            status = 0
            err = 2
        endif

!  Read in the header information

        call ftgkyj (lun,'REVOLUT',revl,com,status)
        if(status .ne. 0) then
            print *, 'READ_PHA POSITION 04'
            print *, 'Missing REVOLUT keyword'
            status = 0
            err = 2
        endif

!  Read in the instrument information

        call ftgkys (lun,'INSTRUME',mode,com,status)
        if(status .ne. 0) then
            print *, 'READ_PHA POSITION 05'
            print *, 'Missing INSTRUME keyword'
            status = 0
            err = 2
        elseif (mode .eq. 'EPN') then

!  Read in the mode information if a PN observation

            call ftgkys (lun,'SUBMODE',mode,com,status)
            if(status .ne. 0) then
                print *, 'READ_PHA POSITION 06'
                print *, 'Missing SUBMODE keyword'
                status = 0
                err = 2
            endif
            call ftgkyj (lun,'FRMTIME',frmtime,com,status)
            if(status .ne. 0) then
                print *, 'READ_PHA POSITION 07'
                print *, 'Missing FRMTIME keyword'
                status = 0
                err = 2
            else
                if(frmtime .lt. 100) then
                    continue
                elseif(frmtime .lt. 210) then
                    mod(24) = '3'
                else
                    mod(24) = '5'
                endif
                mod(25) = ' '
                modet = mode
            endif
        endif

!  Move to the spectrum extension

        if(err .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'STATUS FTMRHD =', status
                print *, 'READ_PHA POSITION 08'
                print *, 'Could not find the spectrum extension'
                print *, spec_name
            else

!  Read the FITS file header information

                call ftghbn (lun,10,nchan,tfields,ttype,tform, &
                    tunit,extname,varidat,status)
                if(extname .ne. 'SPECTRUM') then
                    print *, 'READ_PHA POSITION 09'
                    print *, extname
                    print *, 'This was supposed to be a spectrum ext'
                    stop
                endif
                if(mchan .lt. nchan) then
                    print *, spec_name
                    print *,' 2', mchan, nchan
                    print *, 'Number of array elements smaller than', &
                            ' number of channels in spectrum'
                    err = 3
                    return
                elseif(status .ne. 0) then
                    err = 1
                    print *, 'STATUS FTGHPR =', status
                    print *, 'READ_PHA POSITION 10'
                    print *, 'Failure on reading the header ', &
                            'information'
                else

!  Read in the header information

                    call ftgkye (lun,'BACKSCAL',backscal,com,status)
                    if(status .ne. 0) then
                        print *, 'READ_PHA POSITION 11'
                        print *, 'Missing BACKSCAL keyword'
                        status = 0
                        err = 2
                    endif
                    call ftgkye (lun,'EXPOSURE',exposure,com,status)
                    if(status .ne. 0) then
                        if(backscal .gt. 0) then
                            print *, 'READ_PHA POSITION 12a'
                            print *, 'Missing EXPOSURE keyword'
                        else
                            print *, 'READ_PHA POSITION 12b'
                            print *, 'Missing EXPOSURE keyword'
                            print *, 'Probably due to exclustion of ', &
                              'of the CCD in the selection expression'
                            print *, 'or no selected data on the CCD'
                        endif
                        status = 0
                        err = 2
                    endif
                    call ftgkye (lun,'AREASCAL',areascal,com,status)
                    if(status .ne. 0) then
                        print *, 'READ_PHA POSITION 13'
                        print *, 'Missing AREASCAL keyword'
                        status = 0
                        err = 2
                    endif

!  Read in the column names

                    call ftgkns (lun,'TTYPE',1,10,colnam,ncols,status)

!  Extract the channel numbers

                    ich = 0
                    icou = 0
                    igrp = 0
                    iqual = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'CHANNEL') ich = icol
                        if(colnam(icol) .eq. 'COUNTS') icou = icol
                        if(colnam(icol) .eq. 'GROUPING') igrp = icol
                        if(colnam(icol) .eq. 'QUALITY') iqual = icol
                    enddo
                    do irow=1,nchan
                        channel(irow) = 0
                        counts(irow) = 0
                        group(irow) = 0
                        quality(irow) = 0
                    enddo
                    do i=1,nchan
                        if(ich .ne. 0) &
                            call ftgcvj(lun,ich,i,1,1,0, &
                                channel(i),anyf,status)
                        if(icou .ne. 0) &
                            call ftgcve(lun,icou,i,1,1,0, &
                                counts(i),anyf,status)
                        if(igrp .ne. 0) &
                            call ftgcvj(lun,igrp,i,1,1,0, &
                                group(i),anyf,status)
                        if(iqual .ne. 0) &
                            call ftgcvj(lun,iqual,i,1,1,0, &
                                quality(i),anyf,status)
                    enddo

!  Calculate the uncertainties

                    do irow=1,nchan
                        stat(irow) = sqrt(abs(counts(irow)))
                    enddo
                endif
            endif
            call ftclos (lun,status)
        endif

!  Clean up

        call ftfiou (lun,status)

        return
        end subroutine read_pha

        subroutine read_rmf (obj_rmf,mchan,detm,chan, &
            eminin,emaxin,nchanrin,eminout,emaxout,nchanrout)

        implicit none

        integer :: mchan
        integer :: blockn, chan(mchan), err, &
            fchan(mchan,20), fchant(mchan), &
            fnum, hdutype, i, ich, icol, idetm, iehi, ielo, &
            iifchan, igrp, ii, imin, imax, inchan, &
            ioff, irow, k, kk, lun, mnum, nchan, nchanr(mchan,20), &
            nchanrin, nchanrout, nchant(mchan), ncols, ngrp(mchan), &
            nnum, status, tfields, tnull, varidat
        real :: detm(mchan,mchan), detmt(mchan), emaxin(mchan), &
            emaxout(mchan), eminin(mchan), eminout(mchan)
        double precision :: tscal, tzero
        character(len=1) :: dtyp
        character(len=8) :: colnam(10), tdisp, tform(10), ttyp,  &
            ttype(10), tuni, tunit(10)
        character(len=20) :: extname
        character(len=80) :: obj_rmf
        logical anyf

        status = 0
        err = 0

!  Get an unused lun

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, '  '
            print *, 'STATUS FTGIOU =', status
            print *, 'READ_RMF POSITION 31'
        endif

!  Open the FITS file

        call ftopen (lun,obj_rmf,0,blockn,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, '  '
            print *, 'STATUS FTOPEN =', status
            print *, 'READ_RMF POSITION 32'
            print *, obj_rmf
        endif

!  Move to the matrix extension

        if(err .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, '  '
                print *, 'STATUS FTMRHD =', status
                print *, 'POSITION 33'
                print *, 'Could not find the matrix extension'
            else

!  Read the FITS file header information

                call ftghbn (lun,10,nchan,tfields,ttype,tform, &
                    tunit,extname,varidat,status)
                nchanrin = nchan
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, '  '
                    print *, 'STATUS FTGHBN =', status
                    print *, 'READ_RMF POSITION 34'
                    print *, 'Could not find the matrix extension'
                endif
                if(extname .ne. 'MATRIX') then
                    print *, extname
                    print *, '  '
                    print *, 'READ_RMF'
                    print *, 'This was supposed to be a matrix ext'
                    stop
                endif
                if(mchan .lt. nchan) then
                    print *, obj_rmf
                    print *,' 3', mchan, nchan
                    print *, '  '
                    print *, 'READ_RMF'
                    print *, 'Number of array elements smaller than', &
                            ' number of channels in matrix'
                    err = 3
                    return
                elseif(status .ne. 0) then
                    err = 1
                    print *, '  '
                    print *, 'STATUS FTGHPR =', status
                    print *, 'READ_RMF POSITION 35'
                    print *, 'Failure on reading the header ', &
                            'information'
                else

!  Read in the column names

                    call ftgkns (lun,'TTYPE',1,10,colnam,ncols,status)
                    if(status .ne. 0) then
                        err = 1
                        print *, '  '
                        print *, 'STATUS FTGKNS =', status
                        print *, 'READ_RMF POSITION 36'
                        print *, 'Failure on reading the column names'
                    endif

!  Extract the channel numbers

                    ielo = 0
                    iehi = 0
                    igrp = 0
                    iifchan = 0
                    inchan = 0
                    idetm = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'ENERG_LO') ielo = icol
                        if(colnam(icol) .eq. 'ENERG_HI') iehi = icol
                        if(colnam(icol) .eq. 'N_GRP') igrp = icol
                        if(colnam(icol) .eq. 'F_CHAN') iifchan = icol
                        if(colnam(icol) .eq. 'N_CHAN') inchan = icol
                        if(colnam(icol) .eq. 'MATRIX') idetm = icol
                    enddo
                    do irow=1,nchan
                        eminin(irow) = 0
                        emaxin(irow) = 0
                        ngrp(irow) = 0
                        do ii=1,20
                            nchanr(irow,ii) = 0
                            fchan(irow,ii) = 0
                        enddo
                        do ii=1,mchan
                            detm(irow,ii) = 0
                        enddo
                    enddo
                    call ftgbcl(lun,iifchan,ttyp,tuni,dtyp,fnum, &
                        tscal,tzero,tnull,tdisp,status)
                    call ftgbcl(lun,inchan,ttyp,tuni,dtyp,nnum, &
                        tscal,tzero,tnull,tdisp,status)
                    do i=1,nchan
                        if(ielo .ne. 0) &
                            call ftgcve(lun,ielo,i,1,1,0, &
                                eminin(i),anyf,status)
                        if(iehi .ne. 0) &
                            call ftgcve(lun,iehi,i,1,1,0, &
                                emaxin(i),anyf,status)
                        if(igrp .ne. 0) &
                            call ftgcvj(lun,igrp,i,1,1,0, &
                                ngrp(i),anyf,status)
                        if(iifchan .ne. 0) then
                            call ftgcvj(lun,iifchan,i,1,fnum,0, &
                                    fchant,anyf,status)
                            do ii=1,fnum
                                fchan(i,ii) = fchant(ii)
                            enddo
                        endif
                        if(inchan .ne. 0) then
                            call ftgcvj(lun,inchan,i,1,nnum,0, &
                                    nchant,anyf,status)
                            do ii=1,nnum
                                nchanr(i,ii) = nchant(ii)
                            enddo
                        endif
                        if(idetm .ne. 0) then
                            call ftgdes(lun,idetm,i,mnum,ioff,status)
                            call ftgcve(lun,idetm,i,1,mnum,0, &
                                    detmt,anyf,status)
                            kk = 0
                            do k=1,fnum
                                do ii=1,nchant(k)
                                    kk = kk + 1
                                    ioff = fchant(k) + ii
                                    detm(i,ioff) = detmt(kk)
                                enddo
                            enddo
                        endif
                        if((status .ne. 0) .and. (err .eq. 0)) then
                            err = 1
                            print *, 'STATUS FTGCV =', status
                            print *, 'POSITION 37'
                            print *, 'Failure on reading data'
                        endif
                    enddo
                endif
            endif
        endif

!  Move to the EBOUNDS extension

        if(err .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, '  '
                print *, 'STATUS FTMRHD =', status
                print *, 'POSITION 33'
                print *, 'Could not find the EBOUNDS extension'
            else

!  Read the FITS file header information

                call ftghbn (lun,10,nchan,tfields,ttype,tform, &
                    tunit,extname,varidat,status)
                nchanrout = nchan
                if((status .ne. 0) .and. (err .eq. 0)) then
                    err = 1
                    print *, '  '
                    print *, 'STATUS FTGHBN =', status
                    print *, 'READ_RMF POSITION 34'
                    print *, 'Could not find the spectrum extension'
                endif
                if(extname .ne. 'EBOUNDS') then
                    print *, extname
                    print *, '  '
                    print *, 'READ_RMF'
                    print *, 'This was supposed to be a energy ext'
                    stop
                endif
                if(mchan .lt. nchan) then
                    print *, obj_rmf
                    print *,' 3', mchan, nchan
                    print *, '  '
                    print *, 'READ_RMF'
                    print *, 'Number of array elements smaller than', &
                            ' number of channels in spectrum'
                    err = 3
                    return
                elseif(status .ne. 0) then
                    err = 1
                    print *, '  '
                    print *, 'STATUS FTGHPR =', status
                    print *, 'READ_RMF POSITION 35'
                    print *, 'Failure on reading the header ', &
                            'information'
                else

!  Read in the column names

                    call ftgkns (lun,'TTYPE',1,10,colnam,ncols,status)
                    if(status .ne. 0) then
                        err = 1
                        print *, '  '
                        print *, 'STATUS FTGKNS =', status
                        print *, 'READ_RMF POSITION 36'
                        print *, 'Failure on reading the column names'
                    endif

!  Extract the channel numbers

                    ich = 0
                    imin = 0
                    imax = 0
                    do icol=1,ncols
                        if(colnam(icol) .eq. 'CHANNEL') ich = icol
                        if(colnam(icol) .eq. 'E_MIN') imin = icol
                        if(colnam(icol) .eq. 'E_MAX') imax = icol
                    enddo
                    do irow=1,nchan
                        chan(irow) = 0
                        eminout(irow) = 0
                        emaxout(irow) = 0
                    enddo
                    do i=1,nchan
                        if(ich .ne. 0) &
                            call ftgcvj(lun,ich,i,1,1,0, &
                                chan(i),anyf,status)
                        if(imin .ne. 0) &
                            call ftgcve(lun,imin,i,1,1,0, &
                                eminout(i),anyf,status)
                        if(imax .ne. 0) &
                            call ftgcve(lun,imax,i,1,1,0, &
                                emaxout(i),anyf,status)
                        if((status .ne. 0) .and. (err .eq. 0)) then
                            err = 1
                            print *, 'STATUS FTGCV =', status
                            print *, 'POSITION 37'
                            print *, 'Failure on reading data'
                        endif
                    enddo
                endif
            endif
            if(status .ne. 0) then
                print *, 'Failure in READ_RMF getting energy bounds'
                print *, 'STATUS = ', status
            endif
            call ftclos (lun,status)
        endif

!  Clean up

        call ftfiou (lun,status)

        return
        endsubroutine read_rmf


        subroutine REPOL1(DX,DY,DZ,DR,DPHI,DTHETA)

!C    Converts rectangular coordinates into polar coordinates
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:10
!
!A  author : KOD        date: 15-JUN-1989 18:31
!U  update : KOD        date: 20-MAR-1990 15:09
!
!T  status: tested
!
!   general description
!G  Subroutine to convert the rectangular coordinates (DX,DY,DZ)
!G  into the polar coordinates (DR,DPHI,DTHETA) - mathematical
!G  notation.
!
!   call_var.          type I/O description
!P  DX                  R8  I   x-value
!P  DY                  R8  I   y-value
!P  DZ                  R8  I   z-value
!P
!P  DR                  R8    O radius   (same unit as DX,DY,DZ)
!P  DPHI                R8    O azimuthal angle  (radians)
!P  DTHETA              R8    O  polar    angle  (radians)
!
!   include_block name          description
!I  CGENL.CMN          general parameter common block
!I  IGENL.INC         parameter block with trig. constants
!
!   routines_called    type     description

        IMPLICIT NONE

!        INCLUDE 'cgenl.cmn'
!        INCLUDE 'igenl.inc'

        DOUBLE PRECISION :: DX, DY, DZ, DR, DPHI, DTHETA, DH

        DR = dsqrt(DX*DX+DY*DY+DZ*DZ)

        if (DR.eq.0.0D0) then
            DPHI   = 0.0D0
            DTHETA = 0.0D0
        else
            DTHETA = dacos(DZ/DR)
            DH = dsqrt(DX*DX+DY*DY)
            if (DH.eq.0.0D0) then
                DPHI = 0.0D0
            else
                DPHI = dasin(DY/DH)
                if (DX .lt. 0.0D0) DPHI = DPI  - DPHI
                if (DPHI .lt. 0.0D0) DPHI = DPHI + D2PI
            endif
        endif

        RETURN
        end subroutine repol1

        subroutine RESVAL(DXMIN,DXMAX,DXOLD,DXNEW,IERR)

!C    'Resets' periodic variables to their basic interval
!
!************************ FFORM VERSION 1.2 ************ 30-MAR-90 17:10
!
!A  author : KOD        date: 19-JUL-1989 18:31
!U  update : KOD        date: 20-MAR-1990 15:10
!
!T  status: tested
!
!   general description
!G  Subroutine to 'reset' the values of periodically defined
!G  variables to their basic interval.
!
!   call_var.          type I/O description
!P  DXMIN               R8  I   left  boundary of basic interval
!P  DXMAX               R8  I   right boundary of basic interval
!P
!P  DXOLD               R8  I   value before resetting
!P  DXNEW               R8    O value after  resetting
!P
!P  IERR                I4    O = 0 no error
!P                              = 1 warning
!P                              = 2 error
!P                              = 3 severe error
!
!   include_block name          description
!I  CGENL.CMN          general parameter common block
!
!   routines_called    type     description
!R  WRERR               SR      formats error code in standard form

        IMPLICIT NONE

!        INCLUDE 'cgenl.cmn'

        DOUBLE PRECISION :: DXMIN, DXMAX, DXOLD, DXNEW, DDIFF, DH
        INTEGER :: IERR
        CHARACTER(len=8) :: RTNAME = 'RESVAL'
        CHARACTER(len=80) :: LOG

        IERR = 0

        if (DXMIN .ge. DXMAX) then
            IERR = 2
            LOG = 'Error in input parameters DXMIN,DXMAX'
            call WRERR(RTNAME,IERR,LOG)
        endif

        if (IERR .lt. 2) then
            DDIFF = DXMAX - DXMIN
            DH = (DXOLD-DXMIN) / DDIFF
            DH = DH - DINT(DH)
            if (DH.lt.0.D0) DH = DH + 1.D0
            DXNEW = DXMIN + DH * DDIFF
        endif

        RETURN
        end subroutine resval

        subroutine retfit(infile,rarray,i4array, &
            iflag,isize1,isize2,ctype,crval,cdelt,crpix,status)
!
!  INFILE    file name of input file
!  RARRAY    REAL*4 array for input
!  I4ARRAY   INTEGER*4 array for input
!  IFLAG     flag for input file type selection
!            1 => R*4 array
!            2 => I*4 array
!  ISIZE1    array dimension 1
!  ISIZE2    array dimension 2
!  CRVAL1    reference coordinate for first axis (RA)
!  CDELT1    pixel size for first axis
!  CRVAL2    reference coordinate for second axis (Dec)
!  CDELT2    pixel size for second axis
!  STATUS    FITSIO status flag

        implicit none

        integer :: isize1, isize2
        integer :: bitpix, blockn, hdutype, &
            i4array(isize1,isize2), iflag, &
            gcount, naxes(10), naxis, lun, pcount, status
        real :: rarray(isize1,isize2)
        character(len=8) :: ctype(2)
        character(len=80) :: com
        character(len=132) :: infile
        double precision :: cdelt(2), crpix(2), crval(2)
        logical anyf, extend, simple
        integer :: err = 0

        status = 0

!  Get an unused lun

        call ftgiou(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'RETFIT POSITION 1'
            print *, 'STATUS FTGIOU =', status
        endif

!  Read the array

        write(errstr,*) "RETFIT file name: ", trim(infile)
        call message(AppMsg,Noisy)

!  Open the FITS file for the first observation exposure

        call ftopen(lun,infile,0,blockn,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            print *, 'RETFIT POSITION 2'
            print *, 'STATUS FTOPEN =', status
            print *, infile
            call ftclos (lun,status)
            call ftfiou (lun,status)
            return
        endif
 
!  Read the FITS file header information

        call ftghpr (lun,10,simple,bitpix,naxis,naxes, &
                            pcount,gcount,extend,status)

!  Check to see if the primary has the image

        if(naxis .eq. 0) then
            call ftmrhd (lun,1,hdutype,status)
            call ftghpr (lun,10,simple,bitpix,naxis,naxes, &
                            pcount,gcount,extend,status)
        endif

        if((isize1 .ne. naxes(1)) .or. (isize2 .ne. naxes(2))) then
            print *, 'RETFIT POSITION 3a, STATUS = ', STATUS
            print *, infile
            print *, 'Input array size not equal to image'
            print *, isize1, isize2, ' --  ', naxes(1), naxes(2)
            stop
        endif
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'RETFIT POSITION 3b'
            print *, 'STATUS FTGHPR =', status
            print *, infile
            stop
        endif

!  Get the useful keywords

        call ftgkys(lun,'CTYPE1',ctype(1),com,status)
        call ftgkyd(lun,'CRVAL1',crval(1),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CTYPE1'
            stop
        endif
        call ftgkyd(lun,'cdelt1',cdelt(1),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CDELT1'
            stop
        endif
        call ftgkyd(lun,'crpix1',crpix(1),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CRPIX1'
            stop
        endif
        call ftgkys(lun,'CTYPE2',ctype(2),com,status)
        call ftgkyd(lun,'CRVAL2',crval(2),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CRVAL2'
            stop
        endif
        call ftgkyd(lun,'CDELT2',cdelt(2),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CDELT2'
            stop
        endif
        call ftgkyd(lun,'CRPIX2',crpix(2),com,status)
        if(status .ne. 0) then
            print *, 'RETFIT failure on CRPIX2'
            stop
        endif

!  Read in an R*4 file

        if(iflag .eq. 1) then
            call ftg2de (lun,0,0,naxes(1),naxes(1),naxes(2),rarray, &
                    anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'RETFIT POSITION 4'
                print *, 'STATUS FTG2DE =', status
                print *, infile
            endif
 
!  Read in an I*4 file
 
        elseif(iflag .eq. 2) then
            call ftg2dj (lun,0,0,naxes(1),naxes(1),naxes(2),i4array, &
                    anyf,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                err = 1
                print *, 'RETFIT POSITION 6'
                print *, 'STATUS FTG2DJ =', status
                print *, infile
            endif
        endif

!  Close the FITS file

        call ftclos(lun,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            err = 1
            print *, 'RETFIT POSITION 7'
            print *, 'STATUS FTCLOS =', status
            print *, infile
        endif

!  Clean up

        status = 0
        call ftfiou (lun,status)

        return
        end subroutine retfit

        subroutine solve (ma,na,uu,yy,ssig,aa,cchisq,chicon,cvm)

        implicit none

        integer i, ii, j, ma, na
        real aa(na), cchisq, chicon(ma), ssig(ma), uu(ma,na), yy(ma)
        double precision chisq, chitmp, cvm(na,na), sum, &
            thresh, tmp, wmax
        double precision :: tol = 1.0e-5
        double precision, allocatable, dimension(:) :: a, b, &
            sig, w, y
        double precision, allocatable, dimension(:,:) :: u, v

        allocate(a(na))  
        allocate(b(ma)) 
        allocate(sig(ma))
        allocate(u(ma,na)) 
        allocate(v(na,na)) 
        allocate(w(na))  
        allocate(y(ma)) 

        print *, 'Number of Parameters: ', na
        print *, 'Number of Equations: ', ma

        do i=1,ma
            y(i) = yy(i)
            sig(i) = ssig(i)
            do ii=1,na
               u(i,ii) = uu(i,ii)
            enddo
        enddo

        do i=1,ma
            tmp = 1./sig(i)
            do ii=1,na
                u(i,ii) = u(i,ii)*tmp
            enddo
            b(i) = y(i)*tmp
        enddo

        call svdcmp(u,ma,na,w,v)

        WMAX = 0.

        DO J=1,NA
            IF(W(J) .GT. WMAX) WMAX = W(J)
        ENDDO

        THRESH = TOL*WMAX

        DO J=1,NA
            IF(W(J) .LT. THRESH) W(J) = 0.
        ENDDO

        CALL SVBKSB(U,W,V,ma,na,B,A)

        CALL SVDVAR(V,na,W,CVM)

! Calculate (UU*AA-YY)**2 for the fit as well as
! the contribution from each equation 
 
        CHISQ = 0.

        DO I=1,MA
            SUM = 0.
            DO J=1,NA
                SUM = SUM + A(J)*UU(I,J)
            ENDDO
            CHITMP = ((Y(I)-SUM)/SIG(I))**2
            IF(CHITMP .GT. 10.) THEN
                PRINT 2000, I, CHITMP, Y(I), SIG(I), SUM
 2000           FORMAT(1x,I5,4F10.2)
            ENDIF
            CHISQ = CHISQ + CHITMP
            CHICON(I) = CHITMP
        ENDDO

        DO I=1,NA
            AA(I) = A(I)
        ENDDO
        CCHISQ = CHISQ

        RETURN
        end subroutine solve



        SUBROUTINE SVDCMP(A,M,N,W,V)

        IMPLICIT NONE

        INTEGER I, ITS, J, K, L, m, mmax, N, NM
        DOUBLE PRECISION A(m,n), ANORM, C, F, G, H, S, SCALE, &
            W(n), V(n,n), X, Y, Z
        double precision rv1(5000)

        G = 0.0
        SCALE = 0.0
        ANORM = 0.0
        DO I=1,N
            L = I + 1
            RV1(I) = SCALE*G
            G = 0.0
            S = 0.0
            SCALE = 0.0
            IF (I.LE.M) THEN
                DO K=I,M
                    SCALE = SCALE + DABS(A(K,I))
                ENDDO
                IF (SCALE .NE. 0.0) THEN
                    DO K=I,M
                        A(K,I) = A(K,I)/SCALE
                        S = S + A(K,I)*A(K,I)
                    ENDDO
                    F = A(I,I)
                    G = -DSIGN(DSQRT(S),F)
                    H = F*G - S
                    A(I,I) = F - G
                    IF (I .NE. N) THEN
                        DO J=L,N
                            S = 0.0
                            DO K=I,M
                                S = S + A(K,I)*A(K,J)
                            ENDDO
                            F = S/H
                            DO K=I,M
                                A(K,J) = A(K,J) + F*A(K,I)
                            ENDDO
                        ENDDO
                    ENDIF
                    DO K= I,M
                        A(K,I) = SCALE*A(K,I)
                    ENDDO
                ENDIF
            ENDIF
            W(I) = SCALE*G
            G = 0.0
            S = 0.0
            SCALE = 0.0
            IF ((I .LE. M) .AND. (I .NE. N)) THEN
                DO K=L,N
                    SCALE = SCALE + DABS(A(I,K))
                ENDDO
                IF (SCALE .NE. 0.0) THEN
                    DO K=L,N
                        A(I,K) = A(I,K)/SCALE
                        S = S + A(I,K)*A(I,K)
                    ENDDO
                    F = A(I,L)
                    G = -DSIGN(DSQRT(S),F)
                    H = F*G - S
                    A(I,L) = F - G
                    DO K=L,N
                        RV1(K) = A(I,K)/H
                    ENDDO

                    IF (I .NE. M) THEN
                        DO J=L,M
                            S = 0.0
                            DO K=L,N
                                S = S + A(J,K)*A(I,K)
                            ENDDO
                            DO K=L,N
                                A(J,K) = A(J,K) + S*RV1(K)
                            ENDDO
                        ENDDO
                    ENDIF
                    DO K=L,N
                        A(I,K) = SCALE*A(I,K)
                    ENDDO
                ENDIF
            ENDIF
            ANORM = MAX(ANORM,(DABS(W(I))+DABS(RV1(I))))
        ENDDO

        DO I=N,1,-1
            IF (I .LT. N) THEN
                IF (G .NE. 0.0) THEN
                    DO J=L,N
                        V(J,I) = (A(I,J)/A(I,L))/G
                    ENDDO
                    DO J=L,N
                        S = 0.0
                        DO K=L,N
                            S = S + A(I,K)*V(K,J)
                        ENDDO
                        DO K=L,N
                            V(K,J) = V(K,J) + S*V(K,I)
                        ENDDO
                    ENDDO
                ENDIF
                DO J=L,N
                    V(I,J) = 0.0
                    V(J,I) = 0.0
                ENDDO
            ENDIF
            V(I,I) = 1.0
            G = RV1(I)
            L = I
        ENDDO

        DO I=N,1,-1
            L = I + 1
            G = W(I)
            IF (I.LT.N) THEN
                DO J=L,N
                    A(I,J) = 0.0
                ENDDO
            ENDIF
            IF (G .NE. 0.0) THEN
                G = 1.0/G
                IF (I .NE. N) THEN
                    DO J=L,N
                        S = 0.0
                        DO K=L,M
                            S = S + A(K,I)*A(K,J)
                        ENDDO
                        F = (S/A(I,I))*G
                        DO K=I,M
                            A(K,J) = A(K,J) + F*A(K,I)
                        ENDDO
                    ENDDO
                ENDIF
                DO J=I,M
                    A(J,I) = A(J,I)*G
                ENDDO
            ELSE
                DO J= I,M
                    A(J,I) = 0.0
                ENDDO
            ENDIF
            A(I,I) = A(I,I) + 1.0
        ENDDO

        DO K=N,1,-1
            DO ITS=1,30
                DO L=K,1,-1
                    NM = L - 1
                    IF ((ABS(RV1(L))+ANORM).EQ.ANORM)  GO TO 2
                    IF ((ABS(W(NM))+ANORM).EQ.ANORM)  GO TO 1
                ENDDO
    1           C = 0.0
                S = 1.0
                DO I=L,K
                    F = S*RV1(I)
                    IF ((ABS(F)+ANORM) .NE. ANORM) THEN
                        G = W(I)
                        H = SQRT(F*F+G*G)
                        W(I) = H
                        H = 1.0/H
                        C = (G*H)
                        S = -(F*H)
                        DO J=1,M
                            Y = A(J,NM)
                            Z = A(J,I)
                            A(J,NM) = (Y*C)+(Z*S)
                            A(J,I) = -(Y*S)+(Z*C)
                        ENDDO
                    ENDIF
                ENDDO
    2           Z = W(K)
                IF (L .EQ. K) THEN
                    IF (Z .LT. 0.0) THEN
                        W(K) = -Z
                        DO J=1,N
                            V(J,K) = -V(J,K)
                        ENDDO
                    ENDIF
                    GO TO 3
                ENDIF
!                IF (ITS.EQ.30) PAUSE 'No convergence in 30 iterations'
                X = W(L)
                NM = K-1
                Y = W(NM)
                G = RV1(NM)
                H = RV1(K)
                F = ((Y-Z)*(Y+Z)+(G-H)*(G+H))/(2.0*H*Y)
                G = SQRT(F*F+1.0)
                F = ((X-Z)*(X+Z)+H*((Y/(F+SIGN(G,F)))-H))/X
                C = 1.0
                S = 1.0
                DO J=L,NM
                    I = J + 1
                    G = RV1(I)
                    Y = W(I)
                    H=S*G
                    G=C*G
                    Z=SQRT(F*F+H*H)
                    RV1(J)=Z
                    C=F/Z
                    S=H/Z
                    F= (X*C)+(G*S)
                    G=-(X*S)+(G*C)
                    H=Y*S
                    Y=Y*C
                    DO NM=1,N
                        X=V(NM,J)
                        Z=V(NM,I)
                        V(NM,J)= (X*C)+(Z*S)
                        V(NM,I)=-(X*S)+(Z*C)
                    ENDDO
                    Z=SQRT(F*F+H*H)
                    W(J)=Z
                    IF (Z.NE.0.0) THEN
                        Z=1.0/Z
                        C=F*Z
                        S=H*Z
                    ENDIF
                    F= (C*G)+(S*Y)
                    X=-(S*G)+(C*Y)
                    DO NM=1,M
                        Y=A(NM,J)
                        Z=A(NM,I)
                        A(NM,J)= (Y*C)+(Z*S)
                        A(NM,I)=-(Y*S)+(Z*C)
                    ENDDO
                ENDDO
                RV1(L)=0.0
                RV1(K)=F
                W(K)=X
            ENDDO
    3       CONTINUE
        ENDDO

        RETURN
        end subroutine svdcmp


        SUBROUTINE SVBKSB(U,W,V,m,n,B,X)

        IMPLICIT NONE

        INTEGER I, J, JJ, m, n
        DOUBLE PRECISION S, u(m,n), w(n), v(n,n), b(m), x(n)
        double precision, allocatable, dimension(:) :: tmp

        allocate(tmp(m))

        DO J=1,n
            S = 0.
            IF(W(J) .NE. 0.)THEN
                DO I=1,M
                    S = S + U(I,J)*B(I)
                ENDDO
                S = S/W(J)
            ENDIF
            TMP(J) = S
        ENDDO

        DO J=1,N
            S = 0.
            DO JJ=1,N
                S = S + V(J,JJ)*TMP(JJ)
            ENDDO
            X(J) = S
        ENDDO

        RETURN
        end subroutine svbksb



        SUBROUTINE SVDVAR(V,n,W,CVM)

        integer :: n
        double precision :: cvm(n,n), v(n,n), w(n)
        integer :: I, J, K
        double precision :: SUM 
        double precision, allocatable, dimension(:) :: wti

        allocate (wti(n))

        DO I=1,n
          WTI(I) = 0.
          IF(W(I) .NE. 0.) WTI(I)=1./(W(I)*W(I))
        ENDDO

        DO I=1,n
          DO J=1,I
            SUM = 0.
            DO K=1,n
              SUM = SUM + V(I,K)*V(J,K)*WTI(K)
            ENDDO
            CVM(I,J) = SUM
            CVM(J,I) = SUM
          ENDDO
        ENDDO

        RETURN
        end subroutine svdvar


        subroutine spscale (xmmdir,xmmd,mosnum, &
                maskfile,specfile,back,c)
!
!  Date: 15 November 2006
!  Conversion to F90 - 7 November 2012
!
!  Calculates the relative contribution of SP flares by region
!
!  spscale uses SP flare detector maps and region images to
!  find the average SP flare contribution over the region.  It
!  also returns the BACKSCALE value from the spectrum for the
!  region.
!  Input:
!    QPB data directory
!    MOS number
!    Mask image file name
!    Spectrum file name
!  Output:
!    BACKSCAL value for the region
!    Average SP intensity for the region

        implicit none

        integer :: blockn, hdutype, ini4(780,780), i, ii, iii, &
            lunspec, mosnum, n, nchan, tfields, varidat
        integer :: err = 0
        integer :: status = 0
        real :: back, c, mask(780,780), sp(780,780)

        double precision :: cdelt(2), crpix(2), crval(2)

        character(len=1) :: db_fi(132), sp_fi(25), xmmd(132)
        character(len=8) :: ctype(2), tform(10), ttype(10), tunit(10)
        character(len=25) :: sp_file
        character(len=20) :: extname
        character(len=60) :: com, maskfile, specfile
        character(len=132) :: db_file, tempfile, xmmdir

        equivalence (db_file,db_fi)
        equivalence (sp_file,sp_fi)

        db_file = xmmdir

!  Set the SP detector maps

        if(mosnum .eq. 1) then
            sp_file = '/mos1-sp-tot.fits.gz'
        elseif(mosnum .eq. 2) then
            sp_file = '/mos1-sp-tot.fits.gz'
        else
            sp_file = '/pn-sp-tot.fits.gz'
        endif

!  Create the SP detector map FITS file name including the
!  directory string

        ii = 1
        do while (xmmd(ii) .ne. ' ')
            db_fi(ii) = xmmd(ii)
            ii = ii + 1
        enddo
        do iii=0,24
            db_fi(ii+iii) = sp_fi(iii+1)
        enddo
        write(errstr,*) "Opening image: ", db_fi
        call message(AppMsg,Noisy)

!  Read in the SP image

        tempfile = db_file
        call retfit(tempfile,sp,ini4, &
                1,780,780,ctype,crval,cdelt,crpix,status)
        if(status .ne. 0) then
            write(errstr,*) "01: STATUS=",status, &
                            " on return from retfit"
            call message(AppMsg,Always)
            call error ("noRet","Cannot return SP image")
        endif

!  Read in the mask image

        tempfile = maskfile
        call retfit(tempfile,mask,ini4, &
                1,780,780,ctype,crval,cdelt,crpix,status)
        if(status .ne. 0) then
            write(errstr,*) "01: STATUS=",status, &
                            " on return from retfit"
            call message(AppMsg,Always)
            call error ("noRet","Cannot return mask image")
        endif

!  Get an unused lun for the spectrum

        call ftgiou(lunspec,status)
        if((status .ne. 0) .and. (err .eq. 0)) then
            write(errstr,*) "02: STATUS=",status, &
                    " on return from ftgiou"
            call message(AppMsg,Always)
            call error ("NoLUN","Cannot aquire LUN")
        endif

!  Open the mask spectrum FITS file

        write(errstr,*) "Opening spectrum: ", trim(specfile)
        call message(AppMsg,Noisy)
        call ftopen (lunspec,specfile,0,blockn,status)
        if(status .ne. 0) then
            write(errstr,*) "03: STATUS=",status, &
                    " on return from ftopen"
            call message(AppMsg,Always)
            call error ("NoOpen","Cannot open spectrum file")
        endif

!  Move to the spectrum extension

        if(err .eq. 0) then
            call ftmrhd (lunspec,1,hdutype,status)
            if((status .ne. 0) .and. (err .eq. 0)) then
                write(errstr,*) "04: STATUS=",status, &
                        " on return from ftmrhd"
                call message(AppMsg,Always)
                call error ("NoMove","Cannot move to extension")
            else

!  Read the FITS file header information

                call ftghbn (lunspec,10,nchan,tfields, &
                        ttype,tform,tunit,extname,varidat,status)
                if(extname .ne. 'SPECTRUM') then
                    write(errstr,*) "05: STATUS=",status, &
                           " on return from ftghbn"
                    call message(AppMsg,Always)
                    write(errstr,*) "Wrong extension", extname
                    call message(AppMsg,Always)
                    call error ("WrongExtension",&
                           "Incorrect extension")
                else

!  Read in the header information

                    call ftgkye (lunspec,'BACKSCAL',back,com,status)
                    if(status .ne. 0) then
                        write(errstr,*) "06: STATUS=",status, &
                                 " on return from ftgkye"
                        call message(AppMsg,Always)
                        call error ("No Key", &
                                 "Cannot return BACKSCAL keyword")
                    endif
                endif
            endif
        endif
        call ftclos (lunspec,status)
        if(status .ne. 0) then
            write(errstr,*) "07: STATUS=",status, &
                    " on return from ftclos"
            call message(AppMsg,Always)
            call error ("noClose","Cannot close file")
        endif
        back = back/1440000.0
        write(errstr,*) "Backscale: ", back
        call message(AppMsg,Sparse)

!  Loop over the image

        n = 0
        c = 0
        do i=1,780
            do ii=1,780
                if (mask(i,ii) .gt. 0) then
                    n = n + 1
                    c = c + sp(i,ii)
                endif
            enddo
        enddo
        c = c/n

        return

    end subroutine spscale

        SUBROUTINE WRERR (RTNAME,IERR,TEXT)

        INTEGER IERR
        CHARACTER*8 RTNAME
        CHARACTER*80 TEXT

        print *, 'Error in Routine: ', RTNAME
        print *, 'Error Number: ', IERR
        print *, text

        RETURN
        end subroutine wrerr

end module subsn_mod
